/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.trifork.stamdata.persistence;

import static com.google.inject.Key.get;
import static com.google.inject.name.Names.named;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.inject.Named;
import javax.inject.Provider;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.log4j.Logger;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.TypeLiteral;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;

public class PersistenceModule extends AbstractModule
{
    private static final TypeLiteral<Set<Object>> A_SET_OF_OBJECTS = new TypeLiteral<Set<Object>>() {};

    public static final String JNDI_DATASOURCE_NAME = "sdm.JNDIName";

    public static final String JDBC_URL_PROP = "db.connection.jdbcURL";
    public static final String DB_USERNAME_PROP = "db.connection.username";
    public static final String DB_PASSWORD_PROP = "db.connection.password";

    private static HikariConfig config = new HikariConfig();
    private static HikariDataSource ds = null;

    Map<String,DataSource> dataSourceMap = new HashMap<String,DataSource>();

    @Override
    protected void configure()
    {
        requireBinding(get(String.class, named(JNDI_DATASOURCE_NAME)));
    }

    @Provides
    protected Connection provideConnection(@Named(JNDI_DATASOURCE_NAME) String jndiDatasource,
                                           @Named(JDBC_URL_PROP) String jdbcUrl,
                                           @Named(DB_USERNAME_PROP) String jdbcUsername,
                                           @Named(DB_PASSWORD_PROP) String jdbcPassword)
    {
        try {
            // In tests jndiDatasource is empty and we use jdbc directly
            if (jndiDatasource == null || jndiDatasource.length() == 0) {
                if (ds == null) {
                    config.setJdbcUrl(jdbcUrl);
                    config.setUsername(jdbcUsername);
                    config.setPassword(jdbcPassword);
                    config.addDataSourceProperty("cachePrepStmts", "true");
                    config.addDataSourceProperty("prepStmtCacheSize", "250");
                    config.addDataSourceProperty("prepStmtCacheSqlLimit", "2048");
                    config.setMaximumPoolSize(500);
                    ds = new HikariDataSource(config);

                }
                Connection connection = ds.getConnection();
                connection.setAutoCommit(false);

                return connection;
            } else {
                Connection connection = getDataSource(jndiDatasource).getConnection();
                connection.setAutoCommit(false);

                return connection;
            }
        } catch (NamingException e) {
            throw  new RuntimeException("Cannot get connection", e);
        } catch (SQLException e) {
            throw  new RuntimeException("Cannot get connection", e);
        }
    }

    public DataSource getDataSource(String jndiDatasource) throws NamingException {
        if (!dataSourceMap.containsKey(jndiDatasource)) {
            Context initialContext = new InitialContext();
            dataSourceMap.put(jndiDatasource, (DataSource) initialContext.lookup(jndiDatasource));
        }
        return dataSourceMap.get(jndiDatasource);
    }

    public static boolean checkDatabase(Provider<Connection> connectionProvider, Logger logger) {
        Connection connection = null;
        try {
            connection = connectionProvider.get();
            connection.createStatement().execute("SELECT 1");
            connection.commit();
        } catch (SQLException e) {
            logger.error("Can not access database");
            return false;
        } finally {
            closeConnection(connection);
        }
        return true;
    }

    public static void closeConnection(Connection connection) {
        if (connection != null) {
            try {
                if (!connection.isClosed()) {
                    connection.commit();
                    connection.close();
                }
            } catch (SQLException e) {
                throw new RuntimeException("Could not close database connection", e);
            }
        }
    }

    public static void closePS(PreparedStatement preparedStatement) {
        if(preparedStatement != null) {
            try {
                preparedStatement.close();
            } catch (SQLException e) {
                throw new RuntimeException("Could not close prepared statement", e);
            }
        }
    }

    public static void closeStatement(Statement statement) {
        if(statement != null) {
            try {
                statement.close();
            } catch (SQLException e) {
                throw new RuntimeException("Could not close statement", e);
            }
        }
    }

    public static void close(Connection connection) {
        closeConnection(connection);
    }

    public static void close(PreparedStatement preparedStatement) {
        closePS(preparedStatement);
    }

    public static void close(Statement statement) {
        closeStatement(statement);
    }

}
