/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.testing;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Random;

import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import com.trifork.stamdata.models.BarnRelationVO;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO.FMRTypeKode;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.SikredeVO;
import com.trifork.stamdata.models.YderVO;


public class Factories
{
    public static final String DUMMY_STRING = "D";
    public static final Timestamp DUMMY_DATE = Timestamp.valueOf("2000-01-01 00:00:00");

    private static Random random = new Random();

    public static final Date YEAR_2100 = new DateTime(2100, 1, 1, 0, 0, 0).toDate();
    public static final Date YEAR_2000 = new DateTime(2000, 1, 1, 0, 0, 0).toDate();
    public static final Date YEAR_1999 = new DateTime(1999, 1, 1, 0, 0, 0).toDate();

    public static final Date TWO_DAYS_AGO = DateTime.now().minusDays(2).withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).withMillisOfSecond(0).toDate();
    public static final Date YESTERDAY_PLUS_2_YEARS = DateTime.now().minusDays(1).withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).withMillisOfSecond(0).plusYears(2).toDate();
    public static final Date YESTERDAY = DateTime.now().minusDays(1).withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).withMillisOfSecond(0).toDate();
    public static final Date TOMORROW = DateTime.now().plusDays(1).withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).withMillisOfSecond(0).toDate();
    public static final Date IN_TWO_DAYS = DateTime.now().plusDays(2).withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).withMillisOfSecond(0).toDate();
    public static final Date FIRST_UNIX_DATE = new DateTime(1970, 1, 1, 0, 0, 0).toDate();

    public static String generateRandomCPR()
    {

        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, random.nextInt(98) + 1);
        cal.set(Calendar.MONTH, random.nextInt(11));
        cal.set(Calendar.DAY_OF_YEAR, random.nextInt(364) + 1);
        int no = random.nextInt(9999);

        return new SimpleDateFormat("ddMMyy").format(cal.getTime()) + String.format("%04d", no);
    }


    public static PersonVO createPersonWithCPR(String cpr)
    {
        PersonVO person = createPerson();
        person.setCpr(cpr);
        return person;
    }


    public static PersonVO createPersonWithBirthday(Date birthday)
    {
        PersonVO person = createPerson();
        person.setFoedselsdato(birthday);
        return person;
    }


    public static PersonVO createPerson() {
        return createPerson(true, "6666");
    }

    public static PersonVO createPerson(String postCode) {
        return createPerson(true, postCode);
    }

    public static PersonVO createPerson(boolean hasHouseNo, String postCode)
    {
        PersonVO person = new PersonVO();

        person.setFornavn("Peter");
        person.setMellemnavn("Sigurd");
        person.setEfternavn("Andersen");
        person.setNavnTilAdressering("Peter,Andersen");

        person.setCpr(generateRandomCPR());

        // Yes, they are not identical.
        person.setGaeldendeCPR(generateRandomCPR());

        person.setKoen("M");

        person.setFoedselsdato(TWO_DAYS_AGO);

        person.setCoNavn("Søren Petersen");

        person.setKommuneKode("0461");
        person.setVejKode("0234");
        if (hasHouseNo) {
            person.setHusnummer("10");
        } else {
            person.setHusnummer("");
        }
        person.setBygningsnummer("A");
        person.setLokalitet("Birkely");
        person.setBynavn("BirkeBy");

        person.setVejnavn("Ørstedgade");
        person.setVejnavnTilAdressering("Østergd.");
        person.setEtage("12");
        person.setSideDoerNummer("tv");
        person.setFoedselsdatoMarkering("*");
        person.setStatus("01");
        person.setStatusDato(YESTERDAY);

        person.setPostnummer(postCode);
        person.setPostdistrikt("Überwald");

        person.setNavnebeskyttelsestartdato(null);
        person.setNavnebeskyttelseslettedato(null);

        person.setModifiedDate(TWO_DAYS_AGO);
        person.setCreatedDate(TWO_DAYS_AGO);
        person.setValidFrom(YESTERDAY);
        person.setValidTo(TOMORROW);

        return person;
    }


    public static PersonVO createPersonWithAddressProtection()
    {
        PersonVO person = createPerson();

        person.setNavnebeskyttelsestartdato(YESTERDAY);
        person.setNavnebeskyttelseslettedato(TOMORROW);

        return person;
    }

    public static SikredeVO createSikredeFor(PersonVO person, YderVO yder, String groupCode, DateTime ikraftDato, DateTime iKraftYder)
    {
        SikredeVO sikrede = createDummySikrede();

        sikrede.setCPRnr(person.getCpr());
        sikrede.setSYdernr(yder.getYdernrYder());
        sikrede.setSSikrGrpKode(groupCode);
        sikrede.setSIkraftDatoGrp(sikredeRecordDateString(ikraftDato));
        sikrede.setSIkraftDatoYder(sikredeRecordDateString(iKraftYder));
		sikrede.setSSKGyldigFra("");
		sikrede.setSSKGyldigTil("");

		return sikrede;
    }

    public static SikredeVO createSikredeFor(PersonVO person, YderVO yder, String groupCode, DateTime ikraftDato)
    {
        SikredeVO sikrede = createDummySikrede();

        sikrede.setCPRnr(person.getCpr());
        sikrede.setSYdernr(yder.getYdernrYder());
        sikrede.setSStatus(DUMMY_STRING);
        sikrede.setSSikrGrpKode(groupCode);
        sikrede.setSIkraftDatoGrp(sikredeRecordDateString(ikraftDato));
        sikrede.setSSKGyldigFra("");
        sikrede.setSSKGyldigTil("");

        return sikrede;
    }

    public static SikredeVO createSikredeFor(PersonVO person, YderVO yder, YderVO yderGL, String groupCode, DateTime ikraftDato)
    {
        SikredeVO sikrede = createDummySikrede();

        sikrede.setCPRnr(person.getCpr());
        sikrede.setSYdernr(yder.getYdernrYder());
        sikrede.setSYdernrGl(yderGL.getYdernrYder());
        sikrede.setSStatus(DUMMY_STRING);
        sikrede.setSSikrGrpKode(groupCode);
        sikrede.setSIkraftDatoGrp(sikredeRecordDateString(ikraftDato));
        sikrede.setSSKGyldigFra("");
        sikrede.setSSKGyldigTil("");

        return sikrede;
    }

    public static ForaeldreMyndighedRelationVO createForaeldreMyndighedRelationFor(PersonVO person, FMRTypeKode typeKode, PersonVO relation)
    {
        ForaeldreMyndighedRelationVO foraeldreMyndighedRelation = new ForaeldreMyndighedRelationVO();
        String relationCPR = (relation != null) ? relation.getCpr() : null;

        foraeldreMyndighedRelation.setId(person.getCpr()+"-"+typeKode.getId(relationCPR));
        foraeldreMyndighedRelation.setCPR(person.getCpr());
        foraeldreMyndighedRelation.setTypeKode(typeKode.getKode());
        foraeldreMyndighedRelation.setTypeTekst(typeKode.getTekst());
        foraeldreMyndighedRelation.setRelationCpr(relationCPR);
        foraeldreMyndighedRelation.setRelationCprStartDato(DUMMY_DATE);
        foraeldreMyndighedRelation.setForaeldreMyndighedStartDato(DUMMY_DATE);
        foraeldreMyndighedRelation.setForaeldreMyndighedStartDatoMarkering(DUMMY_STRING);
        foraeldreMyndighedRelation.setForaeldreMyndighedSletteDato(DUMMY_DATE);
        foraeldreMyndighedRelation.setCreatedDate(Timestamp.valueOf("2000-01-01 00:00:00"));
        foraeldreMyndighedRelation.setModifiedDate(DUMMY_DATE);
        foraeldreMyndighedRelation.setValidFrom(DUMMY_DATE);
        foraeldreMyndighedRelation.setValidTo(Timestamp.valueOf("2050-01-01 00:00:00"));

        return foraeldreMyndighedRelation;
    }

    public static BarnRelationVO createBarnRelationFor(PersonVO person, PersonVO barn)
    {
        BarnRelationVO barnRelation = new BarnRelationVO();

        barnRelation.setId(person.getCpr()+"-"+barn.getCpr());
        barnRelation.setCPR(person.getCpr());
        barnRelation.setBarnCPR(barn.getCpr());
        barnRelation.setCreatedDate(Timestamp.valueOf("2000-01-01 00:00:00"));
        barnRelation.setModifiedDate(DUMMY_DATE);
        barnRelation.setValidFrom(DUMMY_DATE);
        barnRelation.setValidTo(Timestamp.valueOf("2050-01-01 00:00:00"));

        return barnRelation;
    }


    public static SikredeVO createDummySikrede()
    {
        SikredeVO sikrede = new SikredeVO();

        sikrede.setCPRnr(DUMMY_STRING);
        sikrede.setSYdernr(DUMMY_STRING);
        sikrede.setSIkraftDatoYder(DUMMY_STRING);
        sikrede.setSRegDatoYder(DUMMY_STRING);
        sikrede.setSSikrGrpKode(DUMMY_STRING);
        sikrede.setSIkraftDatoGrp(DUMMY_STRING);
        sikrede.setSRegDatoGrp(DUMMY_STRING);
        sikrede.setSSikrKomKode(DUMMY_STRING);
        sikrede.setSIkraftDatoKomKode(DUMMY_STRING);
        sikrede.setSYdernrGl(DUMMY_STRING);
        sikrede.setSIkraftDatoYderGl(DUMMY_STRING);
        sikrede.setSRegDatoYderGl(DUMMY_STRING);
        sikrede.setSSikrGrpKodeGl(DUMMY_STRING);
        sikrede.setSIkraftDatoGrpGl(DUMMY_STRING);
        sikrede.setSRegDatoGrpGl(DUMMY_STRING);
        sikrede.setSYdernrFrem(DUMMY_STRING);
        sikrede.setSIkraftDatoYderFrem(DUMMY_STRING);
        sikrede.setSRegDatoYderFrem(DUMMY_STRING);
        sikrede.setSSikrGrpKodeFrem(DUMMY_STRING);
        sikrede.setSIkraftDatoGrpFrem(DUMMY_STRING);
        sikrede.setSRegDatoGrpFrem(DUMMY_STRING);
        sikrede.setSKon(DUMMY_STRING);
        sikrede.setSAlder(DUMMY_STRING);
        sikrede.setSFolgerskabsPerson(DUMMY_STRING);
        sikrede.setSStatus(DUMMY_STRING);
        sikrede.setSBevisDato(DUMMY_STRING);
        sikrede.setPNavn(DUMMY_STRING);
        sikrede.setSBSStatsborgerskabKode(DUMMY_STRING);
        sikrede.setSBSStatsborgerskab(DUMMY_STRING);
        sikrede.setSSKAdrLinie1(DUMMY_STRING);
        sikrede.setSSKAdrLinie2(DUMMY_STRING);
        sikrede.setSSKBopelsLand(DUMMY_STRING);
        sikrede.setSSKBopelsLAndKode(DUMMY_STRING);
        sikrede.setSSKEmailAdr(DUMMY_STRING);
        sikrede.setSSKFamilieRelation(DUMMY_STRING);
        sikrede.setSSKFodselsdato(DUMMY_STRING);
        sikrede.setSSKGyldigFra(DUMMY_STRING);
        sikrede.setSSKGyldigTil(DUMMY_STRING);
        sikrede.setSSKMobilNr(DUMMY_STRING);
        sikrede.setSSKPostNrBy(DUMMY_STRING);
        sikrede.setSSLForsikringsinstans(DUMMY_STRING);
        sikrede.setSSLForsikringsinstansKode(DUMMY_STRING);
        sikrede.setSSLForsikringsnr(DUMMY_STRING);
        sikrede.setSSLGyldigFra(DUMMY_STRING);
        sikrede.setSSLGyldigTil(DUMMY_STRING);
        sikrede.setSSLSocSikretLand(DUMMY_STRING);
        sikrede.setSSLSocSikretLandKode(DUMMY_STRING);
        sikrede.setModifiedDate(DUMMY_DATE);
        sikrede.setValidFrom(DUMMY_DATE);
        sikrede.setValidTo(Timestamp.valueOf("2050-01-01 00:00:00"));

        return sikrede;
    }

    public static SikredeVO createSikredeWithFrontierInfo(PersonVO person, YderVO yder, String groupCode, DateTime ikraftDato,
            DateTime iKraftYder, boolean includeFrontierInfo, boolean includeSocialSecurity, boolean includeSocialSecurityAgenzy,boolean includeOpholdsland)
    {
        SikredeVO sikrede = createDummySikrede();

        sikrede.setCPRnr(person.getCpr());
        sikrede.setSYdernr(yder.getYdernrYder());
        sikrede.setSSikrGrpKode(groupCode);
        sikrede.setSIkraftDatoGrp(sikredeRecordDateString(ikraftDato));
        sikrede.setSIkraftDatoYder(sikredeRecordDateString(iKraftYder));
        sikrede.setSSKGyldigFra(sikredeRecordFrontierDateString(ikraftDato));
        sikrede.setSSKGyldigTil(sikredeRecordFrontierDateString(ikraftDato.plusYears(2)));
        if (includeFrontierInfo) {
            String sskBopelsLand = "";
            String sskBopelsLAndKode = "";
            String sslSocSikretLand = "";
            String sslSocSikretLandKode = "";
            String sslForsikringsinstans = "";
            String sslForsikringsinstansKode = "";
            String sslForsikringsnr = "";
            if (includeSocialSecurity) {
                sslSocSikretLand = "Danmark";
                sslSocSikretLandKode = "DK";
                if (includeSocialSecurityAgenzy) {
                    sslForsikringsinstans = "FOO sikring";
                    sslForsikringsinstansKode = "BAR";
                    sslForsikringsnr = "12345";
                }
            }
            if (includeOpholdsland) {
                sskBopelsLand = "Tyskland";
                sskBopelsLAndKode = "DE";
            }
            sikrede.setSBSStatsborgerskab("England");
            sikrede.setSBSStatsborgerskabKode("GB");
            sikrede.setSSKBopelsLand(sskBopelsLand);
            sikrede.setSSKBopelsLAndKode(sskBopelsLAndKode);
            sikrede.setSSLSocSikretLand(sslSocSikretLand);
            sikrede.setSSLSocSikretLandKode(sslSocSikretLandKode);
            sikrede.setSSLForsikringsinstans(sslForsikringsinstans);
            sikrede.setSSLForsikringsinstansKode(sslForsikringsinstansKode);
            sikrede.setSSLForsikringsnr(sslForsikringsnr);
        } else {
            sikrede.setSBSStatsborgerskab("");
            sikrede.setSBSStatsborgerskabKode("");
            sikrede.setSSKBopelsLand("");
            sikrede.setSSKBopelsLAndKode("");
            sikrede.setSSLSocSikretLand("");
            sikrede.setSSLSocSikretLandKode("");
            sikrede.setSSLForsikringsinstans("");
            sikrede.setSSLForsikringsinstansKode("");
            sikrede.setSSLForsikringsnr("");
        }

        return sikrede;
    }

    public static YderVO createYder(String ydernummer)
    {
        YderVO yder = new YderVO();

        yder.setId("langmd5her...");
        yder.setHistIdYder(DUMMY_STRING);
        yder.setAmtKodeYder(DUMMY_STRING);
        yder.setAmtTxtYder(DUMMY_STRING);
        yder.setYdernrYder(ydernummer);
        yder.setPrakBetegn("Klinikken");
        yder.setAdrYder("Margrethepladsen 44, 8000 Århus");
        yder.setPostnrYder("8000");
        yder.setPostdistYder("Århus");
        yder.setTilgDatoYder(DUMMY_STRING);
        yder.setAfgDatoYder(DUMMY_STRING);
        yder.setHvdSpecKode(DUMMY_STRING);
        yder.setHvdSpecTxt(DUMMY_STRING);
        yder.setHvdTlf("12345678");
        yder.setEmailYder("test@example.com");
        yder.setWWW(DUMMY_STRING);
        yder.setCVR("12345678");
        yder.setModifiedDate(DUMMY_DATE);
        yder.setValidFrom(DUMMY_DATE);
        yder.setValidTo(Timestamp.valueOf("2050-01-01 00:00:00"));

        return yder;
    }



    private static String sikredeRecordDateString(DateTime date)
    {
        DateTimeFormatter formatter = DateTimeFormat.forPattern("yyyyMMdd");
        return formatter.print(date);
    }

    private static String sikredeRecordFrontierDateString(DateTime date)
    {
        DateTimeFormatter formatter = DateTimeFormat.forPattern("yyyy-MM-dd");
        return formatter.print(date);
    }
}
