/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.testing;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicHeader;

import com.unboundid.ldap.sdk.controls.AssertionRequestControl;

import static org.junit.Assert.assertEquals;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;


public class RequestHelper {

    public static String sendRequest(String url, String action, String docXml, boolean failOnError) throws IOException {
        HttpURLConnection uc = null;
        OutputStream os = null;
        InputStream is = null;
        try {
            URL u = new URL(url);
            uc = (HttpURLConnection) u.openConnection();
            uc.setDoOutput(true);
            uc.setDoInput(true);
            uc.setRequestMethod("POST");
            uc.setRequestProperty("SOAPAction", "\"" + action + "\"");
            uc.setRequestProperty("Content-Type", "text/xml; encoding=utf-8");
            os = uc.getOutputStream();
            IOUtils.write(docXml, os, "UTF-8");
            os.flush();
            if (uc.getResponseCode() != 200) {
                is = uc.getErrorStream();
            } else {
                is = uc.getInputStream();
            }
            String res = IOUtils.toString(is, "UTF-8");
            if (uc.getResponseCode() != 200 && (uc.getResponseCode() != 500 || failOnError)) {
                throw new RuntimeException("Got unexpected response " + uc.getResponseCode() +" from " + url);
            }
            return res;
        } finally {
            if (os != null) IOUtils.closeQuietly(os);
            if (is != null) IOUtils.closeQuietly(is);
            if (uc != null) uc.disconnect();
        }
    }
    
    
    public static String sendRequest(String url, String soapAction, String postBody) throws IOException {
		DefaultHttpClient defaultHttpClient = new DefaultHttpClient();
		defaultHttpClient.getParams().setParameter("http.connection.timeout", 30000);

		HttpPost httpPost = new HttpPost(url);
		httpPost.addHeader(new BasicHeader("Content-Type", "text/xml; charset=utf-8"));
		httpPost.addHeader(new BasicHeader("SOAPAction", "\"" + soapAction + "\""));
		httpPost.addHeader(new BasicHeader("X-NSI-GW-PARTITION", "test"));
		httpPost.addHeader(new BasicHeader("SOSIGW_PARTITION", "test"));
		httpPost.setEntity(new StringEntity(postBody, "UTF-8"));

		HttpResponse response = defaultHttpClient.execute(httpPost);

		String result = "";
		int statusCode = response.getStatusLine().getStatusCode();
		if (statusCode == 200 || statusCode == 500) {
			InputStream is = null;
			try {
				is = response.getEntity().getContent();
				result = IOUtils.toString(is, "UTF-8");
			} finally {
				if (is != null) {
					is.close();
				}
			}
		} else if (statusCode == 202) {
			return ""; //Accepted without any response
		} else {
			result = response.getStatusLine().getReasonPhrase();
			throw new IOException("HTTP POST failed (" + statusCode + "): " + result);
		}
		return result;
	}

	public static String getFaultstring(String responseXml) {
		int startIndex = responseXml.indexOf("<faultstring>")+13;
		int endIndex = responseXml.indexOf("</faultstring>", startIndex);
		String faultstring = responseXml.substring(startIndex, endIndex);
		return faultstring;
	}
	
	public static String getLineNumber(String responseXml) {
		int startIndex = responseXml.indexOf("line number: ")+13;
		int endIndex = responseXml.indexOf(", column number: ");
		String lineNumber = responseXml.substring(startIndex, endIndex);
		return lineNumber;
	}
	
	public static String getColumnNumber(String responseXml) {
		int startIndex = responseXml.indexOf("column number: ")+15;
		int endIndex = responseXml.indexOf("; Element type \"foo\"");
		String columnNumber = responseXml.substring(startIndex, endIndex);
		return columnNumber;
	}
	
	public static String getMessage(String responseXml) {
		int startIndex = responseXml.indexOf("Element type \"foo\" must");
		int endIndex = responseXml.indexOf("</faultstring>", startIndex);
		String message = responseXml.substring(startIndex, endIndex);
		return message;
	}
	
	public static String readFile(String file) throws IOException {
	    BufferedReader reader = new BufferedReader(new FileReader (file));
	    String         line = null;
	    StringBuilder  stringBuilder = new StringBuilder();
	    String         ls = System.getProperty("line.separator");
	
	    try {
	        while((line = reader.readLine()) != null) {
	            stringBuilder.append(line);
	            stringBuilder.append(ls);
	        }
	
	        return stringBuilder.toString();
	    } finally {
	        reader.close();
	    }
	}
}
