/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.testing;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.joda.time.DateTime;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.trifork.stamdata.models.AuthorizationVO;
import com.trifork.stamdata.models.BarnRelationVO;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO.FMRTypeKode;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.SikredeVO;
import com.trifork.stamdata.models.YderVO;

public class StamdataTestHelper {
    private List<PersonVO> personList = Lists.newArrayList();
    private List<YderVO> yderList = Lists.newArrayList();
    private List<SikredeVO> sikredeList = Lists.newArrayList();
    private List<ForaeldreMyndighedRelationVO> fmrList = Lists.newArrayList();
    private List<BarnRelationVO> brList = Lists.newArrayList();
    private Map<String,String> familieMap = Maps.newHashMap();
    private Map<String,String> seperatedMap = Maps.newHashMap();
    private List<String> unmarriedList = Lists.newArrayList();
    private Map<String,String> vaergeMap = Maps.newHashMap();
    public static final String CPR_VAERGE = "0102800005";
    public static final String CPR_MEDMOR = "0102800004";
    public static final String CPR_MEDFAR = "0102800003";
    public static final String CPR_MOR = "0102800002";
    public static final String CPR_FAR = "0102800001";
    public static final String CPR_BARN4 = "1111110004";
    public static final String CPR_BARN3 = "1111110003";
    public static final String CPR_BARN2 = "1111110002";
    public static final String CPR_BARN1 = "1111110001";
    public static final String CPR_SPOUSE = "0101014321";
    public static final String PRIOR_CPR = "0102030405";
    public static final String CPR_MISSING = "0102800006";

    SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");

    public Date formatSimpleDate(String date) throws ParseException {
        return dateFormatter.parse(date);
    }

    public void addPerson(PersonVO person) {
        personList.add(person);
    }

    public void addYder(YderVO yder) {
        yderList.add(yder);
    }

    public void addSikrede(SikredeVO sikrede) {
        sikredeList.add(sikrede);
    }

    public void addFarMorRelation(ForaeldreMyndighedRelationVO farMorReleation) {
        fmrList.add(farMorReleation);
    }

    public void addBarnRelation(BarnRelationVO barnRelation) {
        brList.add(barnRelation);
    }

    public PersonVO createPerson() {
        PersonVO person = Factories.createPerson();
        addPerson(person);

        return person;
    }

    public PersonVO createPersonWithPostcode(String postcode) {
        PersonVO person = Factories.createPerson(postcode);
        addPerson(person);

        return person;
    }

    public PersonVO createPerson(String cpr) {
        PersonVO person = Factories.createPersonWithCPR(cpr);
        addPerson(person);

        return person;
    }

    public PersonVO createMand(String cpr) {
        PersonVO person = createPerson(cpr);
        person.setKoen("M");

        return person;
    }

    public PersonVO createKvinde(String cpr) {
        PersonVO person = createPerson(cpr);
        person.setKoen("K");

        return person;
    }

    public PersonVO createMand() {
        PersonVO person = createPerson();
        person.setKoen("M");

        return person;
    }

    public PersonVO createKvinde() {
        PersonVO person = createPerson();
        person.setKoen("K");

        return person;
    }

    public YderVO createYder(String nummer) {
        YderVO yder = Factories.createYder(nummer);
        yderList.add(yder);

        return yder;
    }

    public SikredeVO createSikrede(PersonVO person, YderVO yder, String groupCode, DateTime ikraftDato, DateTime iKraftYder) {
        SikredeVO sikrede = Factories.createSikredeFor(person, yder, groupCode, ikraftDato, iKraftYder);
        sikredeList.add(sikrede);

        return sikrede;
    }

    public SikredeVO createSikrede(PersonVO person, YderVO yder, String groupCode, DateTime ikraftDato)
    {
        SikredeVO sikrede = Factories.createSikredeFor(person, yder, groupCode, ikraftDato);
        sikredeList.add(sikrede);

        return sikrede;
    }

    public SikredeVO createSikrede(PersonVO person, YderVO yder, YderVO yderGL, String groupCode, DateTime ikraftDato) {
        SikredeVO sikrede = Factories.createSikredeFor(person, yder, yderGL, groupCode, ikraftDato);
        sikredeList.add(sikrede);

        return sikrede;
    }

    public SikredeVO createSikredeWithFrontierInfo(PersonVO person, YderVO yder,String groupCode, DateTime ikraftDato, DateTime iKraftYder) {
        return createSikredeWithFrontierInfo(person, yder, groupCode, ikraftDato, iKraftYder, true, true, true, true);
    }

    public SikredeVO createSikredeWithFrontierInfo(PersonVO person, YderVO yder,String groupCode, DateTime ikraftDato,
            DateTime iKraftYder, boolean includeFrontierInfo) {
        return createSikredeWithFrontierInfo(person, yder, groupCode, ikraftDato, iKraftYder, includeFrontierInfo, true, true, true);
    }

    public SikredeVO createSikredeWithFrontierInfo(PersonVO person, YderVO yder,String groupCode, DateTime ikraftDato,
            DateTime iKraftYder, boolean includeFrontierInfo, boolean includeSocialSecurity) {
        return createSikredeWithFrontierInfo(person, yder, groupCode, ikraftDato, iKraftYder, includeFrontierInfo, includeSocialSecurity, true, true);
    }

    public SikredeVO createSikredeWithFrontierInfo(PersonVO person, YderVO yder,String groupCode, DateTime ikraftDato,
            DateTime iKraftYder, boolean includeFrontierInfo, boolean includeSocialSecurity, boolean includeSocialSecurityAgenzy) {
        return createSikredeWithFrontierInfo(person, yder, groupCode, ikraftDato, iKraftYder, includeFrontierInfo, includeSocialSecurity, includeSocialSecurityAgenzy, true);
    }

    public SikredeVO createSikredeWithFrontierInfo(PersonVO person, YderVO yder,String groupCode, DateTime ikraftDato,
            DateTime iKraftYder, boolean includeFrontierInfo, boolean includeSocialSecurity, boolean includeSocialSecurityAgenzy,boolean includeOpholdsland) {
        SikredeVO sikrede = Factories.createSikredeWithFrontierInfo(person, yder, groupCode, ikraftDato,iKraftYder,includeFrontierInfo, includeSocialSecurity, includeSocialSecurityAgenzy, includeOpholdsland);
        sikredeList.add(sikrede);

        return sikrede;
    }

    public PersonVO createPersonWithAddressProtection() {
        PersonVO person = Factories.createPersonWithAddressProtection();
        addPerson(person);

        return person;
    }

    public ForaeldreMyndighedRelationVO createForaeldreMyndighedRelation(PersonVO person, FMRTypeKode type) {
        ForaeldreMyndighedRelationVO fmt = Factories.createForaeldreMyndighedRelationFor(person, type,null);
        fmrList.add(fmt);

        return fmt;
    }

    public ForaeldreMyndighedRelationVO createForaeldreMyndighedRelation(PersonVO person, FMRTypeKode type, PersonVO relation) {
        ForaeldreMyndighedRelationVO fmr = Factories.createForaeldreMyndighedRelationFor(person, type, relation);
        fmrList.add(fmr);

        return fmr;
    }

    public void createVaerge(PersonVO person, PersonVO vaerge) {
        vaergeMap.put(person.getCpr(), (vaerge == null) ? "" : vaerge.getCpr());
    }

    public BarnRelationVO createBarnRelation(PersonVO person, PersonVO barn) {
        BarnRelationVO br = Factories.createBarnRelationFor(person, barn);
        brList.add(br);

        return br;
    }

    public void createSpouse(PersonVO person, PersonVO spouse) {
        familieMap.put(person.getCpr(), spouse.getCpr());
    }

    public void createSeperated(PersonVO person, PersonVO spouse) {
        seperatedMap.put(person.getCpr(), spouse.getCpr());
    }

    public void createUnmarried(PersonVO person) {
        unmarriedList.add(person.getCpr());
    }

    public void deleteAllPersons(Connection connection) throws SQLException {
        connection.createStatement().executeUpdate("DELETE FROM v2_Person");
        connection.commit();
    }

    public PersonVO findPerson(String cpr) {
        for (PersonVO person : personList) {
            if (person.getCpr().equals(cpr)) {
                return person;
            }
        }
        return null;
    }

    public void prepareTestdata(Connection connection) throws SQLException {
        connection.createStatement().executeUpdate("DELETE FROM v2_Person");
    	connection.createStatement().executeUpdate("DELETE FROM Sikrede");
    	connection.createStatement().executeUpdate("DELETE FROM Yderregister_v3");
    	connection.createStatement().executeUpdate("DELETE FROM AssignedDoctor");
    	connection.createStatement().executeUpdate("DELETE FROM v2_ForaeldreMyndighedRelation");
    	connection.createStatement().executeUpdate("DELETE FROM v2_BarnRelation");
    	connection.createStatement().executeUpdate("DELETE FROM PersonKontakt");
    	connection.createStatement().executeUpdate("DELETE FROM HistoriskePersonnumre");
        connection.createStatement().executeUpdate("DELETE FROM PersonFamilie");
        connection.createStatement().executeUpdate("DELETE FROM v2_UmyndiggoerelseVaergeRelation");

    	for (PersonVO person : personList) {
    	    persist(connection,person);
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String tilflytningsDato = dateFormat.format(person.getStatusDato());
            String sql = "INSERT INTO PersonKontakt (CPR, TilflytningKommuneDato, CreatedDate, ModifiedDate, ValidFrom) VALUES ('"+person.getCpr()+"','2000-01-01 00:00:00',now(),now(),now())";
            connection.createStatement().executeUpdate(sql);
            sql = "INSERT INTO HistoriskePersonnumre (CPR, Id, HistoriskCPR, CreatedDate, ModifiedDate, ValidFrom, ValidTo) VALUES ('"+person.getCpr()+"','','0000000000',now(),now(),'2000-01-01 00:00:00',now())";
            connection.createStatement().executeUpdate(sql);
            sql = "INSERT INTO HistoriskePersonnumre (CPR, Id, HistoriskCPR, CreatedDate, ModifiedDate, ValidFrom) VALUES ('"+person.getCpr()+"','','"+PRIOR_CPR+"',now(),now(),now())";
            connection.createStatement().executeUpdate(sql);
            if (familieMap.containsKey(person.getCpr())) {
                PersonVO spouse = findPerson(familieMap.get(person.getCpr()));
                if (person.getKoen().equals(spouse.getKoen())) {
                    sql = "INSERT INTO PersonFamilie (CPR, Civilstand, AegtefaelleCPR, CreatedDate, ModifiedDate, ValidFrom) VALUES ('"+person.getCpr()+"','P','"+familieMap.get(person.getCpr())+"',now(),now(),now())";
                } else {
                    sql = "INSERT INTO PersonFamilie (CPR, Civilstand, AegtefaelleCPR, CreatedDate, ModifiedDate, ValidFrom) VALUES ('"+person.getCpr()+"','G','"+familieMap.get(person.getCpr())+"',now(),now(),now())";
                }
                connection.createStatement().executeUpdate(sql);
            }
            if (seperatedMap.containsKey(person.getCpr())) {
                sql = "INSERT INTO PersonFamilie (CPR, Civilstand, AegtefaelleCPR, CreatedDate, ModifiedDate, ValidFrom) VALUES ('"+person.getCpr()+"','F','"+seperatedMap.get(person.getCpr())+"',now(),now(),now())";
                connection.createStatement().executeUpdate(sql);
            }
            if (unmarriedList.contains(person.getCpr())) {
                sql = "INSERT INTO PersonFamilie (CPR, Civilstand, CreatedDate, ModifiedDate, ValidFrom) VALUES ('"+person.getCpr()+"','U',now(),now(),now())";
                connection.createStatement().executeUpdate(sql);
            }
            if (vaergeMap.containsKey(person.getCpr())) {
                sql = "INSERT INTO v2_UmyndiggoerelseVaergeRelation (Id, CPR, TypeKode, TypeTekst, RelationCpr, CreatedDate, ModifiedDate, ValidFrom, ValidTo) VALUES ('"+person.getCpr()+"-1','"+person.getCpr()+"','1','Værges CPR findes','"+vaergeMap.get(person.getCpr())+"',now(),now(),now(),'2100-12-12')";
                connection.createStatement().executeUpdate(sql);
            }
    	}

    	DateTime dt = new DateTime();
    	dt = dt.minusDays(1);

    	for (SikredeVO sikrede: sikredeList) {
    	    sikrede.setValidFrom(dt.toDate());
    	    persist(connection, sikrede);
    	}

    	for (YderVO yder: yderList) {
    	    if (yder.getValidFrom() == Factories.DUMMY_DATE) {
    	        yder.setValidFrom(dt.toDate());
    	    }
            persist(connection, yder);
    	}

    	if (fmrList != null) {
    		for (ForaeldreMyndighedRelationVO fmr: fmrList) {
    		    fmr.setValidFrom(dt.toDate());
    	        persist(connection, fmr);
    		}
    	}

    	if (brList != null) {
    		for (BarnRelationVO br: brList) {
    		    br.setValidFrom(dt.toDate());
    	        persist(connection, br);
    		}
    	}
    	connection.commit();
    }

    public static void persist(Connection connection, PersonVO person) throws SQLException {
        String sql = "INSERT INTO v2_Person(CPR, Koen, Fornavn, Mellemnavn, Efternavn, CoNavn, Lokalitet, "+
                "Vejnavn, Bygningsnummer, Husnummer, Etage, SideDoerNummer, Bynavn, Postnummer, PostDistrikt, Status, NavneBeskyttelseStartDato, NavneBeskyttelseSletteDato, GaeldendeCPR, "+
                "Foedselsdato, Stilling, VejKode, KommuneKode, NavnTilAdressering, VejnavnTilAdressering, FoedselsdatoMarkering, StatusDato, Unreliable, "+
                "CreatedDate, ModifiedDate, ValidFrom, ValidTo) " +
             "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
        PreparedStatement ps = connection.prepareStatement(sql);

        int index = 1;
        ps.setObject(index++, person.getCpr());
        ps.setObject(index++, person.getKoen());
        ps.setObject(index++, person.getFornavn());
        ps.setObject(index++, person.getMellemnavn());
        ps.setObject(index++, person.getEfternavn());
        ps.setObject(index++, person.getCoNavn());
        ps.setObject(index++, person.getLokalitet());
        ps.setObject(index++, person.getVejnavn());
        ps.setObject(index++, person.getBygningsnummer());
        ps.setObject(index++, person.getHusnummer());
        ps.setObject(index++, person.getEtage());
        ps.setObject(index++, person.getSideDoerNummer());
        ps.setObject(index++, person.getBynavn());
        ps.setObject(index++, person.getPostnummer());
        ps.setObject(index++, person.getPostdistrikt());
        ps.setObject(index++, person.getStatus());
        ps.setObject(index++, person.getNavnebeskyttelsestartdato());
        ps.setObject(index++, person.getNavnebeskyttelseslettedato());
        ps.setObject(index++, person.getGaeldendeCPR());
        ps.setObject(index++, person.getFoedselsdato());
        ps.setObject(index++, person.getStilling());
        ps.setObject(index++, person.getVejKode());
        ps.setObject(index++, person.getKommuneKode());
        ps.setObject(index++, person.getNavnTilAdressering());
        ps.setObject(index++, person.getVejnavnTilAdressering());
        ps.setObject(index++, person.getFoedselsdatoMarkering());
        ps.setObject(index++, person.getStatusDato());
        ps.setObject(index++, person.getUnreliable());
        ps.setObject(index++, person.getCreatedDate());
        ps.setObject(index++, person.getModifiedDate());
        ps.setObject(index++, person.getValidFrom());
        ps.setObject(index++, person.getValidTo());

        ps.executeUpdate();
        connection.commit();
    }

    public static void persist(Connection connection, ForaeldreMyndighedRelationVO fmr) throws SQLException {
        String sql = "INSERT INTO v2_ForaeldreMyndighedRelation(CPR, Id, TypeKode, TypeTekst, RelationCpr, RelationCprStartDato, ForaeldreMyndighedStartDato, ForaeldreMyndighedStartDatoMarkering, ForaeldreMyndighedSletteDato, "+
                        "Unreliable, CreatedDate, ModifiedDate, ValidFrom, ValidTo) " +
                "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
        PreparedStatement ps = connection.prepareStatement(sql);

        int index = 1;
        ps.setObject(index++, fmr.getCPR());
        ps.setObject(index++, fmr.getId());
        ps.setObject(index++, fmr.getTypeKode());
        ps.setObject(index++, fmr.getTypeTekst());
        ps.setObject(index++, fmr.getRelationCpr());
        ps.setObject(index++, fmr.getRelationCprStartDato());
        ps.setObject(index++, fmr.getForaeldreMyndighedStartDato());
        ps.setObject(index++, fmr.getForaeldreMyndighedStartDatoMarkering());
        ps.setObject(index++, fmr.getForaeldreMyndighedSletteDato());
        ps.setObject(index++, fmr.getUnreliable());
        ps.setObject(index++, fmr.getCreatedDate());
        ps.setObject(index++, fmr.getModifiedDate());
        ps.setObject(index++, fmr.getValidFrom());
        ps.setObject(index++, fmr.getValidTo());

        ps.executeUpdate();
        connection.commit();
    }

    public static void persist(Connection connection, BarnRelationVO br) throws SQLException {
        String sql = "INSERT INTO v2_BarnRelation(Id, CPR, BarnCPR, Unreliable, CreatedDate, ModifiedDate, ValidFrom, ValidTo) "+
                     "VALUES (?,?,?,?,?,?,?,?)";
        PreparedStatement ps = connection.prepareStatement(sql);

        int index = 1;
        ps.setObject(index++, br.getId());
        ps.setObject(index++, br.getCPR());
        ps.setObject(index++, br.getBarnCPR());
        ps.setObject(index++, br.getUnreliable());
        ps.setObject(index++, br.getCreatedDate());
        ps.setObject(index++, br.getModifiedDate());
        ps.setObject(index++, br.getValidFrom());
        ps.setObject(index++, br.getValidTo());

        ps.executeUpdate();
        connection.commit();
    }

    public static void persist(Connection connection, YderVO yder) throws SQLException {
        String sql = "INSERT INTO Yderregister_v3(Id, HistIdYder , AmtKodeYder, AmtTxtYder , YdernrYder, PrakBetegn , AdrYder , PostnrYder, PostdistYder , TilgDatoYder, AfgDatoYder, HvdSpecKode, "+
                "HvdSpecTxt , HvdTlf, EmailYder , WWW , CVR, ModifiedDate, ValidFrom, ValidTo) " +
                "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
        PreparedStatement ps = connection.prepareStatement(sql);

        int index = 1;
        ps.setObject(index++, yder.getId());
        ps.setObject(index++, yder.getHistIdYder());
        ps.setObject(index++, yder.getAmtKodeYder());
        ps.setObject(index++, yder.getAmtTxtYder());
        ps.setObject(index++, yder.getYdernrYder());
        ps.setObject(index++, yder.getPrakBetegn());
        ps.setObject(index++, yder.getAdrYder());
        ps.setObject(index++, yder.getPostnrYder());
        ps.setObject(index++, yder.getPostdistYder());
        ps.setObject(index++, yder.getTilgDatoYder());
        ps.setObject(index++, yder.getAfgDatoYder());
        ps.setObject(index++, yder.getHvdSpecKode());
        ps.setObject(index++, yder.getHvdSpecTxt());
        ps.setObject(index++, yder.getHvdTlf());
        ps.setObject(index++, yder.getEmailYder());
        ps.setObject(index++, yder.getWWW());
        ps.setObject(index++, yder.getCVR());
        ps.setObject(index++, yder.getModifiedDate());
        ps.setObject(index++, yder.getValidFrom());
        ps.setObject(index++, yder.getValidTo());

        ps.executeUpdate();
        connection.commit();
    }

    public static void persist(Connection connection, SikredeVO sikrede) throws SQLException {
        String sql = "INSERT INTO Sikrede(CPRnr, SYdernr, SIkraftDatoYder, SRegDatoYder, SSikrGrpKode, SIkraftDatoGrp, SRegDatoGrp, SSikrKomKode, SIkraftDatoKomKode, SYdernrGl, SIkraftDatoYderGl, "+
                "SRegDatoYderGl, SSikrGrpKodeGl, SIkraftDatoGrpGl, SRegDatoGrpGl, SYdernrFrem, SIkraftDatoYderFrem, SRegDatoYderFrem, SSikrGrpKodeFrem, SIkraftDatoGrpFrem, SRegDatoGrpFrem, "+
                "SKon, SAlder, SFolgerskabsPerson, SStatus, SBevisDato, PNavn, SBSStatsborgerskabKode, SBSStatsborgerskab, SSKAdrLinie1, SSKAdrLinie2, SSKBopelsLand, SSKBopelsLAndKode, "+
                "SSKEmailAdr, SSKFamilieRelation, SSKFodselsdato, SSKGyldigFra, SSKGyldigTil, SSKMobilNr, SSKPostNrBy, SSLForsikringsinstans, SSLForsikringsinstansKode, SSLForsikringsnr, "+
                "SSLGyldigFra, SSLGyldigTil, SSLSocSikretLand, SSLSocSikretLandKode, ModifiedDate, ValidFrom, ValidTo) "+
                "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

        PreparedStatement ps = connection.prepareStatement(sql);

        int index = 1;
        ps.setObject(index++, sikrede.getCPRnr());
        ps.setObject(index++, sikrede.getSYdernr());
        ps.setObject(index++, sikrede.getSIkraftDatoYder());
        ps.setObject(index++, sikrede.getSRegDatoYder());
        ps.setObject(index++, sikrede.getSSikrGrpKode());
        ps.setObject(index++, sikrede.getSIkraftDatoGrp());
        ps.setObject(index++, sikrede.getSRegDatoGrp());
        ps.setObject(index++, sikrede.getSSikrKomKode());
        ps.setObject(index++, sikrede.getSIkraftDatoKomKode());
        ps.setObject(index++, sikrede.getSYdernrGl());
        ps.setObject(index++, sikrede.getSIkraftDatoYderGl());
        ps.setObject(index++, sikrede.getSRegDatoYderGl());
        ps.setObject(index++, sikrede.getSSikrGrpKodeGl());
        ps.setObject(index++, sikrede.getSIkraftDatoGrpGl());
        ps.setObject(index++, sikrede.getSRegDatoGrpGl());
        ps.setObject(index++, sikrede.getSYdernrFrem());
        ps.setObject(index++, sikrede.getSIkraftDatoYderFrem());
        ps.setObject(index++, sikrede.getSRegDatoYderFrem());
        ps.setObject(index++, sikrede.getSSikrGrpKodeFrem());
        ps.setObject(index++, sikrede.getSIkraftDatoGrpFrem());
        ps.setObject(index++, sikrede.getSRegDatoGrpFrem());
        ps.setObject(index++, sikrede.getSKon());
        ps.setObject(index++, sikrede.getSAlder());
        ps.setObject(index++, sikrede.getSFolgerskabsPerson());
        ps.setObject(index++, sikrede.getSStatus());
        ps.setObject(index++, sikrede.getSBevisDato());
        ps.setObject(index++, sikrede.getPNavn());
        ps.setObject(index++, sikrede.getSBSStatsborgerskabKode());
        ps.setObject(index++, sikrede.getSBSStatsborgerskab());
        ps.setObject(index++, sikrede.getSSKAdrLinie1());
        ps.setObject(index++, sikrede.getSSKAdrLinie2());
        ps.setObject(index++, sikrede.getSSKBopelsLand());
        ps.setObject(index++, sikrede.getSSKBopelsLAndKode());
        ps.setObject(index++, sikrede.getSSKEmailAdr());
        ps.setObject(index++, sikrede.getSSKFamilieRelation());
        ps.setObject(index++, sikrede.getSSKFodselsdato());
        ps.setObject(index++, sikrede.getSSKGyldigFra());
        ps.setObject(index++, sikrede.getSSKGyldigTil());
        ps.setObject(index++, sikrede.getSSKMobilNr());
        ps.setObject(index++, sikrede.getSSKPostNrBy());
        ps.setObject(index++, sikrede.getSSLForsikringsinstans());
        ps.setObject(index++, sikrede.getSSLForsikringsinstansKode());
        ps.setObject(index++, sikrede.getSSLForsikringsnr());
        ps.setObject(index++, sikrede.getSSLGyldigFra());
        ps.setObject(index++, sikrede.getSSLGyldigTil());
        ps.setObject(index++, sikrede.getSSLSocSikretLand());
        ps.setObject(index++, sikrede.getSSLSocSikretLandKode());
        ps.setObject(index++, sikrede.getModifiedDate());
        ps.setObject(index++, sikrede.getValidFrom());
        ps.setObject(index++, sikrede.getValidTo());

        ps.executeUpdate();
        connection.commit();
    }

    public void clear() {
        personList.clear();
    	sikredeList.clear();
    	yderList.clear();
    	fmrList.clear();
    	brList.clear();
    }

    public static void assertDates(DateTime date1, Date date2) {
        assertDates(date1.toDate(), date2);;
    }

    public static void assertDates(Date date1, Date date2) {
        assertNotNull(date1);
        assertNotNull(date2);
        if (date2 instanceof java.sql.Timestamp) {
            date2 = new Date(date2.getTime());
        }
        assertEquals(date1.toString(),date2.toString());
    }

    public void persist(Connection connection, AuthorizationVO authorization) throws SQLException {
        String sql = "INSERT INTO Autorisation2(Autorisationsnummer, CPR, Fornavn, Efternavn, UddannelsesKode, CreatedDate, ModifiedDate, ValidFrom, ValidTo) "+
                "VALUES (?,?,?,?,?,?,?,?,?)";
        PreparedStatement ps = connection.prepareStatement(sql);

        int index = 1;
        ps.setObject(index++, authorization.getAuthorizationCode());
        ps.setObject(index++, authorization.getCpr());
        ps.setObject(index++, authorization.getFirstName());
        ps.setObject(index++, authorization.getLastName());
        ps.setObject(index++, authorization.getEducationCode());
        ps.setObject(index++, authorization.getCreatedDate());
        ps.setObject(index++, authorization.getModifiedDate());
        ps.setObject(index++, authorization.getValidFrom());
        ps.setObject(index++, authorization.getValidTo());

        ps.executeUpdate();
        connection.commit();
    }
}
