/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.trifork.stamdata.authorization.models;


import static com.trifork.stamdata.Preconditions.checkNotNull;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.models.AuthorizationVO;
import com.trifork.stamdata.persistence.PersistenceModule;

/**
 * Data access object for authorizations.
 *
 * This class is thread safe.
 *
 * @author Thomas Børlum (thb@trifork.com)
 */
public class AuthorizationDao {

    private Provider<Connection> connectionProvider;

	@Inject
	AuthorizationDao(Provider<Connection> connectionProvider) throws IOException {

		this.connectionProvider = checkNotNull(connectionProvider);
	}

	@SuppressWarnings("unchecked")
	public List<AuthorizationVO> getAuthorizationsCpr(String cpr, Date currentTime) throws SQLException {
	    Connection connection = null;
        PreparedStatement preparedStatement = null;
        List<AuthorizationVO> authorizations = new ArrayList<AuthorizationVO>();

        try {
            connection = connectionProvider.get();

            String queryString = "SELECT Autorisationsnummer, CPR, Fornavn, Efternavn, UddannelsesKode, CreatedDate, ModifiedDate, ValidFrom, ValidTo from Autorisation2 "+
                    "where CPR=? AND ValidFrom <= ? AND (ValidTo > ? OR ValidTo IS NULL)";

            preparedStatement = connection.prepareStatement(queryString);
            int index=1;
            preparedStatement.setObject(index++, cpr);
            preparedStatement.setObject(index++, currentTime);
            preparedStatement.setObject(index++, currentTime);
            ResultSet resultSet = preparedStatement.executeQuery();
            while (resultSet.next()) {
                authorizations.add(mapAuthorization(resultSet));
            }
        } finally {
            PersistenceModule.close(preparedStatement);
            PersistenceModule.close(connection);
        }

		return authorizations;
	}

	private AuthorizationVO mapAuthorization(ResultSet resultSet) throws SQLException {
	    AuthorizationVO authorization = new AuthorizationVO();

	    authorization.setAuthorizationCode(resultSet.getString("Autorisationsnummer"));
	    authorization.setCpr(resultSet.getString("CPR"));
	    authorization.setFirstName(resultSet.getString("Fornavn"));
	    authorization.setLastName(resultSet.getString("Efternavn"));
	    authorization.setEducationCode(resultSet.getString("UddannelsesKode"));
	    authorization.setCreatedDate(resultSet.getDate("CreatedDate"));
	    authorization.setModifiedDate(resultSet.getDate("ModifiedDate"));
        authorization.setValidFrom(resultSet.getDate("ValidFrom"));
        authorization.setValidTo(resultSet.getDate("ValidTo"));

        return authorization;
    }

    @SuppressWarnings("unchecked")
	public List<AuthorizationVO> getAuthorizationsAuthorizationCode(String authorizationCode, Date currentTime) throws SQLException {
        Connection connection = null;
        PreparedStatement preparedStatement = null;
        List<AuthorizationVO> authorizations = new ArrayList<AuthorizationVO>();

        try {
            connection = connectionProvider.get();

            String queryString = "SELECT Autorisationsnummer, CPR, Fornavn, Efternavn, UddannelsesKode, CreatedDate, ModifiedDate, ValidFrom, ValidTo from Autorisation2 "+
                    "where Autorisationsnummer=? AND ValidFrom <= ? AND (ValidTo > ? OR ValidTo IS NULL)";

            preparedStatement = connection.prepareStatement(queryString);
            int index=1;
            preparedStatement.setObject(index++, authorizationCode);
            preparedStatement.setObject(index++, currentTime);
            preparedStatement.setObject(index++, currentTime);
            ResultSet resultSet = preparedStatement.executeQuery();
            while (resultSet.next()) {
                authorizations.add(mapAuthorization(resultSet));
            }
        } finally {
            PersistenceModule.close(preparedStatement);
            PersistenceModule.close(connection);
        }

        return authorizations;
	}
}
