/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.trifork.stamdata.authorization.webservice;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jws.WebService;
import javax.xml.ws.Holder;

import org.apache.log4j.Logger;

import com.google.inject.Inject;
import com.sun.xml.ws.developer.SchemaValidation;
import com.trifork.stamdata.authorization.SoapUtils;
import com.trifork.stamdata.authorization.models.AuthorizationDao;
import com.trifork.stamdata.jaxws.GuiceInstanceResolver.GuiceWebservice;
import com.trifork.stamdata.models.AuthorizationVO;
import com.trifork.stamdata.util.SlaConsts;
import dk.nsi.stamdata.security.ClientVocesCvr;
import dk.nsi.stamdata.jaxws.generated.AuthorizationPortType;
import dk.nsi.stamdata.jaxws.generated.AuthorizationRequestType;
import dk.nsi.stamdata.jaxws.generated.AuthorizationResponseType;
import dk.nsi.stamdata.jaxws.generated.AuthorizationType;
import dk.nsi.stamdata.jaxws.generated.DGWSFault;
import dk.nsi.stamdata.jaxws.generated.Header;
import dk.nsi.stamdata.jaxws.generated.ObjectFactory;
import dk.nsi.stamdata.jaxws.generated.Security;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.security.Whitelisted;
import dk.sds.nsp.audit.Audit;
import dk.sds.nsp.audit.AuditBuilder;
import dk.sds.nsp.audit.AuditBuilder.TypeOfInformation;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sosi.seal.model.constants.FaultCodeValues;
import dk.sosi.seal.vault.renewal.model.Response;


@WebService(endpointInterface="dk.nsi.stamdata.jaxws.generated.AuthorizationPortType")
@GuiceWebservice
@SchemaValidation
public class AuthorizationLookupImpl implements AuthorizationPortType
{
	private final WhitelistService whitelist;
	private final AuthorizationDao authorizationDao;
    private final String cvr;
    private final SLALogItem slaLogItem;
    Logger logger = Logger.getLogger(AuthorizationLookupImpl.class);

	@Inject
	AuthorizationLookupImpl(@ClientVocesCvr String cvr, WhitelistService whitelist, AuthorizationDao authorizationDao,
                            SLALogItem slaLogItem)
	{
		this.cvr = cvr;
        this.authorizationDao = authorizationDao;
		this.whitelist = whitelist;
        this.slaLogItem = slaLogItem;
	}

	@Override
    @Whitelisted
	public AuthorizationResponseType authorization(Holder<Security> wsseHeader,
	        Holder<Header> medcomHeader, AuthorizationRequestType parameters)
	        throws DGWSFault {
        Header value = medcomHeader.value;
        if (value.getLinking() != null && value.getLinking().getMessageID() != null) {
            String messageID = value.getLinking().getMessageID();
            slaLogItem.setMessageId(messageID);
            slaLogItem.setSourceOperation("authorization");
            slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_CVR, cvr);
        }
        SoapUtils.setHeadersToOutgoing(wsseHeader, medcomHeader);

        String cpr = parameters.getCpr();
        Date now = new Date();

        List<AuthorizationVO> authorizations = null;
        List<String> authorizationsCodes = new ArrayList<String>();

        try {
            authorizations = authorizationDao.getAuthorizationsCpr(cpr,now);
        } catch (SQLException e) {
            logger.error(e.getMessage());
            throw new DGWSFault("The requested authorization does not exist.", FaultCodeValues.PROCESSING_PROBLEM);
        }


		for (int i = 0; i<authorizations.size(); i++) {
		    authorizationsCodes.add(authorizations.get(i).getAuthorizationCode());
        }

		Collections.sort(authorizationsCodes);

        AuthorizationResponseType response = new ObjectFactory().createAuthorizationResponseType();
        response.setCpr(cpr);

        TypeOfInformation informationType = AuditBuilder.TypeOfInformation.SensitivePersonalInformation;
    	String component = "SDM";
    	String context = "SAES AuthorizationLookup";

    	AuditBuilder auditBuilder = Audit.createAuditBuilder();
        auditBuilder.addAuditInformation(component, context, informationType, "cpr", cpr);
        auditBuilder.addAuditInformation(component, context, informationType, "authorization code", authorizationsCodes);

        if (!authorizations.isEmpty())
        {
            response.setFirstName(authorizations.get(0).getFirstName());
            response.setLastName(authorizations.get(0).getLastName());
        }

        for (AuthorizationVO authorization : authorizations)
        {
            AuthorizationType authorizationType = new ObjectFactory().createAuthorizationType();
            authorizationType.setAuthorizationCode(authorization.getAuthorizationCode());
            authorizationType.setEducationCode(authorization.getEducationCode());

            // Map the name of the education if we know it.

            if (educations.containsKey(authorization.getEducationCode()))
            {
                authorizationType.setEducationName(educations.get(authorization.getEducationCode()));
            }

            response.getAuthorization().add(authorizationType);
        }

        return response;
    }

    private static Map<String,String> educations = new HashMap<String, String>(); {

        educations.put("4498", "Optiker");
        educations.put("5151", "Fysioterapeut");
        educations.put("5153", "Ergoterapeut");
        educations.put("5155", "Fodterapeut");
        educations.put("5158", "Radiograf");
        educations.put("5159", "Bioanalytiker");
        educations.put("5166", "Sygeplejerske");
        educations.put("5175", "Jordemoder");
        educations.put("5265", "Kiropraktor");
        educations.put("5431", "Tandplejer");
        educations.put("5432", "Klinisk Tandtekniker");
        educations.put("5433", "Tandlæge");
        educations.put("5451", "Klinisk diætist");
        educations.put("7170", "Læge");
        educations.put("9495", "Bandagist");
    }
}
