/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.trifork.stamdata.authorization;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertThat;

import java.net.URL;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.xml.namespace.QName;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.common.collect.Lists;
import com.google.inject.Provider;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.AuthorizationVO;
import com.trifork.stamdata.persistence.PersistenceModule;

import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.jaxws.generated.AuthorizationPortType;
import dk.nsi.stamdata.jaxws.generated.AuthorizationRequestType;
import dk.nsi.stamdata.jaxws.generated.AuthorizationResponseType;
import dk.nsi.stamdata.jaxws.generated.AuthorizationService;
import dk.nsi.stamdata.jaxws.generated.AuthorizationType;
import dk.nsi.stamdata.jaxws.generated.DGWSFault;
import dk.nsi.stamdata.jaxws.generated.ObjectFactory;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import dk.nsi.stamdata.testing.TestServer;
import dk.sosi.seal.model.AuthenticationLevel;


@RunWith(GuiceTestRunner.class)
public class AuthorizationServletIdCardLevelAttackTest
{
    TestServer server;
    List<AuthorizationVO> authorizations = Lists.newArrayList();

    static final String WHITELISTED_CVR = "19343634";
    static final String NON_WHITELISTED_CVR = "12341234";

    String cvr = WHITELISTED_CVR;

    @Inject
    private Provider<Connection> connectionProvider;

    @Inject
    Connection connection;

    StamdataTestHelper testHelper = new StamdataTestHelper();

    private AuthorizationRequestType request = new ObjectFactory().createAuthorizationRequestType();
    private AuthorizationResponseType response = null;
    private AuthorizationPortType client;

    @Before
    public void setUp() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();

        server = new TestServer().start();
    }


    @After
    public void tearDown() throws Exception
    {
        if (server != null) {
            server.stop();
        }
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelOneIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(1);
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelTwoIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(2);
    }

    @Test
    public void requestWithLevelthreeIdCardDoesNotRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(3);
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelFourIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(4);
    }
    private void sendRequestWithGivenSecurityLevel(int securityLevel) throws Exception
    {
        AuthorizationVO authorization1 = createAuthorization();
        authorization1.setCpr("1111122222");
        authorization1.setEducationCode("2131");
        authorization1.setFirstName("Peter");
        authorization1.setLastName("Andersen");
        authorization1.setAuthorizationCode("B1114");

        // Add another to make sure the right one is selected.

        AuthorizationVO authorization2 = createAuthorization();
        authorization2.setCpr("2222211111");

        request.setCpr(authorization1.getCpr());

        sendRequest(securityLevel);

        assertThat(response.getFirstName(), is(authorization1.getFirstName()));
        assertThat(response.getLastName(), is(authorization1.getLastName()));

        assertThat(response.getAuthorization().size(), is(1));

        AuthorizationType authorization = response.getAuthorization().get(0);

        assertThat(authorization.getEducationCode(), is(authorization1.getEducationCode()));
        assertThat(authorization.getAuthorizationCode(), is(authorization1.getAuthorizationCode()));
    }

    public AuthorizationVO createAuthorization()
    {
        AuthorizationVO auth = new AuthorizationVO("1234567890", "Ib", "Sørensen", "A1234", "2312");

        auth.setCreatedDate(Factories.DUMMY_DATE);
        auth.setModifiedDate(Factories.DUMMY_DATE);
        auth.setValidFrom(Factories.DUMMY_DATE);
        auth.setValidTo(Factories.YEAR_2100);

        authorizations.add(auth);
        return auth;
    }

    public void persistAuthorizations() throws SQLException
    {
        Connection connection = null;
        try {
            connection = connectionProvider.get();
            connection.createStatement().executeUpdate("DELETE FROM Autorisation2");
            connection.commit();
            for (AuthorizationVO authorization : authorizations) {
                testHelper.persist(connection, authorization);
            }
        } finally {
            PersistenceModule.close(connection);
        }
    }

    private static Map<Integer, AuthenticationLevel> mapFromIntegerLevelToEnum;
    static {
        mapFromIntegerLevelToEnum = new HashMap<Integer, AuthenticationLevel>();

        mapFromIntegerLevelToEnum.put(AuthenticationLevel.NO_AUTHENTICATION.getLevel(),
                AuthenticationLevel.NO_AUTHENTICATION);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.USERNAME_PASSWORD_AUTHENTICATION.getLevel(),
                AuthenticationLevel.USERNAME_PASSWORD_AUTHENTICATION);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.VOCES_TRUSTED_SYSTEM.getLevel(),
                AuthenticationLevel.VOCES_TRUSTED_SYSTEM);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.MOCES_TRUSTED_USER.getLevel(),
                AuthenticationLevel.MOCES_TRUSTED_USER);
    }

    public void sendRequest(int securityLevel) throws Exception
    {
        persistAuthorizations();

        final QName SERVICE_QNAME = new QName("http://nsi.dk/-/stamdata/3.0", "AuthorizationService");

        URL wsdlLocation = new URL("http://localhost:8972/service/AuthorizationService?wsdl");
        AuthorizationService serviceCatalog = new AuthorizationService(wsdlLocation, SERVICE_QNAME);

        // SEAL enforces that the XML prefixes are exactly
        // as it creates them. So we have to make sure we
        // don't change them.

        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());

        client = serviceCatalog.getAuthorizationPort();

        SecurityWrapper headers = DGWSHeaderUtil.getSecurityWrapper(
                mapFromIntegerLevelToEnum.get(securityLevel), WHITELISTED_CVR, "foo", "bar");

        response = client.authorization(headers.getSecurity(), headers.getMedcomHeader(), request);
    }
}
