/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.replication.dao;

import com.google.inject.Provider;
import com.trifork.stamdata.persistence.PersistenceModule;

import dk.nsi.stamdata.replication.exceptions.DynamicViewException;
import dk.nsi.stamdata.replication.vo.ColumnMapVO;
import dk.nsi.stamdata.replication.vo.ViewMapVO;
import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.handlers.ArrayListHandler;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.LinkedList;
import java.util.List;

@Singleton
public class DynamicViewDAO {
    private final Provider<Connection> connectionProvider;

    @Inject
    public DynamicViewDAO(Provider<Connection> connectionProvider)
    {
        this.connectionProvider = connectionProvider;
    }

    public List<String> listAllViews() throws SQLException {
        List<String> viewList = new LinkedList<String>();
        String sql = "SELECT register, datatype, version FROM SKRSViewMapping";
        ArrayListHandler handler = new ArrayListHandler();
        QueryRunner qr = new QueryRunner();
        Connection conn = null;
        try {
            conn = connectionProvider.get();
            List<Object[]> views = qr.query(conn, sql, handler);
            for (Object[] view : views) {
                String register = (String) view[0];
                String datatype = (String) view[1];
                Integer version = (Integer) view[2];
                viewList.add(register + "/" + datatype + "/v" + version);
            }
        } finally {
            PersistenceModule.close(conn);
        }
        return viewList;
    }

    public ViewMapVO getViewMapForView(String register, String dataType, long version) throws SQLException {
        String sql = "SELECT idSKRSViewMapping,tableName,createdDate FROM SKRSViewMapping WHERE register=? AND " +
                "datatype=? AND version=?";
        ArrayListHandler handler = new ArrayListHandler();
        QueryRunner qr = new QueryRunner();
        Connection conn = null;
        try {
            conn = connectionProvider.get();
            List<Object[]> viewMaps = qr.query(conn, sql, handler, register, dataType, version);
            if (viewMaps.size() < 1) {
                throw new DynamicViewException("View not found for " + register + "/" + dataType + "/" + version);
            } else if (viewMaps.size() > 1) {
                throw new DynamicViewException("Multiple views found for " + register + "/" + dataType + "/" + version);
            }
            Long id = (Long) viewMaps.get(0)[0];
            String tableName = (String) viewMaps.get(0)[1];
            Timestamp createTime = (Timestamp) viewMaps.get(0)[2];

            List<ColumnMapVO> columnMaps = getColumnMapsForView(conn, id);
            ViewMapVO viewMap = new ViewMapVO();
            viewMap.setCreatedDate(createTime);
            viewMap.setDatatype(dataType);
            viewMap.setRegister(register);
            viewMap.setTableName(tableName);
            viewMap.setVersion(version);
            for (ColumnMapVO columnMap : columnMaps) {
                viewMap.addColumn(columnMap);
            }
            return viewMap;
        } finally {
            PersistenceModule.close(conn);
        }
    }

    private List<ColumnMapVO> getColumnMapsForView(Connection conn, Long viewMapId) throws SQLException {
        List<ColumnMapVO> columnMaps = new LinkedList<ColumnMapVO>();
        String sql = "SELECT isPID,tableColumnName,feedColumnName,feedPosition,dataType,maxLength FROM SKRSColumns "+
                " WHERE viewMap=? ORDER BY feedPosition";
        ArrayListHandler handler = new ArrayListHandler();
        QueryRunner qr = new QueryRunner();
        List<Object[]> columnMappings = qr.query(conn, sql, handler, viewMapId);
        for (Object[] columnMap : columnMappings) {
            ColumnMapVO map = new ColumnMapVO();
            map.setPid((Integer) columnMap[0] > 0);
            map.setTableColumnName((String) columnMap[1]);
            map.setFeedColumnName((String) columnMap[2]);
            map.setFeedPosition((Integer) columnMap[3]);
            map.setDataType((Integer) columnMap[4]);
            map.setMaxLength((Integer) columnMap[5]);
            columnMaps.add(map);
        }
        return columnMaps;
    }
}
