/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.replication.dynamic;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.google.inject.Singleton;
import dk.nsi.stamdata.replication.exceptions.DynamicViewException;
import dk.nsi.stamdata.replication.vo.ColumnMapVO;
import dk.nsi.stamdata.replication.vo.ViewMapVO;
import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.handlers.MapListHandler;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.joda.time.Instant;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

@Singleton
public class DynamicRowFetcher {

    private static final Logger logger = Logger.getLogger(DynamicRowFetcher.class);
    private final Provider<Connection> connectionProvider;

    @Inject
    public DynamicRowFetcher(Provider<Connection> connectionProvider) {
        this.connectionProvider = connectionProvider;
    }

    public List<DynamicRow> fetchRows(ViewMapVO view, long fromPid, Instant fromModifiedDate, int limit) throws SQLException {
        List<DynamicRow> resultRows = new LinkedList<DynamicRow>();
        String pidColumn = findPidColumnName(view);
        String[] columns = selectColumnsFromRow(view);
        String columnsForSelect = StringUtils.join(columns, ",");
        logger.debug("Fetching rows with columns: " + columnsForSelect);

        String queryString = String.format("SELECT " + columnsForSelect + " FROM %s WHERE " +
                "(" + pidColumn + " > ? AND ModifiedDate = ?) OR " +
                "(ModifiedDate > ?) " +
                "ORDER BY ModifiedDate, " + pidColumn + " LIMIT %d", view.getTableName(), limit);

        QueryRunner qr = new QueryRunner();
        MapListHandler handler = new MapListHandler();
        Connection conn = null;
        try {
            conn = connectionProvider.get();
            Timestamp fromModifiedTimestamp = new Timestamp(fromModifiedDate.getMillis());
            List<Map<String,Object>> rows = qr
                    .query(conn, queryString, handler, fromPid, fromModifiedTimestamp, fromModifiedTimestamp);
            for (Map<String,Object> objMap : rows) {
                resultRows.add(createDynamicRowFromObjects(view, objMap));
            }
        } finally {
            DbUtils.close(conn);
        }
        return resultRows;
    }

    private String findPidColumnName(ViewMapVO view) {
        for (ColumnMapVO column : view.getColumnMaps()) {
            if (column.isPid()) {
                return column.getTableColumnName();
            }
        }
        throw new DynamicViewException("PID Column not found for view " + view.toString());
    }

    private String[] selectColumnsFromRow(ViewMapVO view) {
        List<ColumnMapVO> columnMaps = view.getColumnMaps();
        List<String> columnNames = new LinkedList<String>();
        for (ColumnMapVO column : columnMaps) {
            columnNames.add(column.getTableColumnName());
        }
        return columnNames.toArray(new String[columnNames.size()]);
    }

    private DynamicRow createDynamicRowFromObjects(ViewMapVO view, Map<String,Object> objMap) {
        return new DynamicRow(view, objMap);
    }
}
