/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.replication.webservice;

import static java.lang.String.format;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.List;

import javax.jws.HandlerChain;
import javax.jws.WebService;
import javax.xml.transform.TransformerException;
import javax.xml.ws.Holder;

import com.trifork.stamdata.util.SlaConsts;
import dk.nsi.stamdata.replication.dynamic.DynamicViewMapper;
import dk.nsi.stamdata.replication.dynamic.DynamicRow;
import dk.nsi.stamdata.replication.dynamic.DynamicRowFetcher;
import dk.nsi.stamdata.replication.vo.ViewMapVO;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import org.apache.log4j.Logger;
import org.apache.log4j.MDC;
import org.joda.time.DateTime;
import org.joda.time.Instant;
import org.w3c.dom.Document;

import com.google.inject.Inject;
import com.sun.xml.ws.developer.SchemaValidation;
import com.trifork.stamdata.jaxws.GuiceInstanceResolver.GuiceWebservice;
import com.trifork.stamdata.persistence.PersistenceModule;

import dk.nsi.stamdata.jaxws.generated.Header;
import dk.nsi.stamdata.jaxws.generated.ObjectFactory;
import dk.nsi.stamdata.jaxws.generated.ReplicationFault;
import dk.nsi.stamdata.jaxws.generated.ReplicationRequestType;
import dk.nsi.stamdata.jaxws.generated.ReplicationResponseType;
import dk.nsi.stamdata.jaxws.generated.Security;
import dk.nsi.stamdata.jaxws.generated.StamdataReplication;
import dk.nsi.stamdata.jaxws.generated.Timestamp;
import dk.nsi.stamdata.replication.models.ClientDao;
import dk.nsi.stamdata.replication.models.ClientVO;
import dk.nsi.stamdata.security.ClientVocesCvr;
import dk.sds.nsp.audit.Audit;
import dk.sds.nsp.audit.AuditBuilder;
import dk.sds.nsp.audit.AuditBuilder.TypeOfInformation;

@WebService(endpointInterface="dk.nsi.stamdata.jaxws.generated.StamdataReplication")
@HandlerChain(file="handler-chain.xml")
@GuiceWebservice
@SchemaValidation
public class StamdataReplicationImpl implements StamdataReplication {

    private static final Logger logger = Logger.getLogger(StamdataReplicationImpl.class);

    private static final int MAX_RECORD_LIMIT = 2000;

    private final String cvr;
    private final ClientDao clients;
    private final SLALogItem slaLogItem;

    @Inject
    private DynamicViewMapper dynamicViewMapper;

    @Inject
    private DynamicRowFetcher dynamicRowFetcher;

    @Inject
    private DynamicViewXmlGenerator dynamicViewXmlGenerator;

    @Inject
    StamdataReplicationImpl(@ClientVocesCvr String cvr, ClientDao clientDao, SLALogItem slaLogItem)
    {
        this.cvr = cvr;
        this.clients = clientDao;
        this.slaLogItem = slaLogItem;
    }


    @Override
    public ReplicationResponseType replicate(Holder<Security> wsseHeader, Holder<Header> medcomHeader, ReplicationRequestType parameters) throws ReplicationFault
    {
        Header value = medcomHeader.value;
        if (value.getLinking() != null && value.getLinking().getMessageID() != null) {
            String messageID = value.getLinking().getMessageID();
            slaLogItem.setMessageId(messageID);
            slaLogItem.setSourceOperation("replicate");
        }

        slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_CVR, cvr);
        slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_DATATYPE, parameters.getDatatype());
        slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_REGISTER, parameters.getRegister());
        slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_REGISTER_VERSION, String.valueOf(parameters.getVersion()));


        auditLogInput(parameters);


    	// Replace the client request securityheader
    	Security security = new Security();
    	Timestamp ts = new Timestamp();
    	ts.setCreated(Calendar.getInstance());
    	security.setTimestamp(ts);
    	wsseHeader.value = security;

        try
        {
            return handleRequestUsingDynamicViews(wsseHeader, medcomHeader, parameters);
        }
        catch (ReplicationFault e)
        {
            // Log an throw is normally an anti-pattern, but since
            // exceptions are part of JAX-WS's flow it is "OK" here.
            //
            logger.warn("The request could not be handled. This is likely the clients mistake.", e);

            throw e;
        }
        catch (Exception e)
        {
            throw new ReplicationFault("An unhandled error occurred.", FaultCodes.INTERNAL_ERROR, e);
        }
    }

	private void auditLogInput(ReplicationRequestType parameters) {
		TypeOfInformation informationType = AuditBuilder.TypeOfInformation.SensitivePersonalInformation;
    	String component = "SDM";
    	String context = "SKRS Replicate";

        AuditBuilder auditBuilder = Audit.createAuditBuilder();
        auditBuilder.addAuditInformation(component, context, informationType, "register", parameters.getRegister());
        auditBuilder.addAuditInformation(component, context, informationType, "datatype", parameters.getDatatype());
        auditBuilder.addAuditInformation(component, context, informationType, "version", parameters.getVersion());
        if (parameters.getMaxRecords() != null) {
            auditBuilder.addAuditInformation(component, context, informationType, "max records", parameters.getMaxRecords());
        } else {
        	auditBuilder.addAuditInformation(component, context, informationType, "max records", "Undefined");
        }
        auditBuilder.addAuditInformation(component, context, informationType, "offset", parameters.getOffset());
	}

    private ReplicationResponseType handleRequestUsingDynamicViews(Holder<Security> wsseHeader, Holder<Header> medcomHeader,
                                                                   ReplicationRequestType parameters) throws ReplicationFault {
        try {
            String viewPath = getViewPath(parameters);

            MDC.put("view", String.valueOf(viewPath));
            MDC.put("cvr", String.valueOf(cvr));

            // Start by checking if the view exist so we can respond with a better error than not authorized.
            if (!dynamicViewMapper.viewMapExists(parameters.getRegister(), parameters.getDatatype(), parameters.getVersion())) {
                throw new ReplicationFault("Requested view: '" + viewPath + "' not found.", FaultCodes.UNKNOWN_VIEW);
            }

            ClientVO client = clients.findByCvr(cvr);
            if (client == null || !client.isAuthorizedFor(viewPath)) {
                throw new ReplicationFault("The provided cvr (" + String.valueOf(cvr) + ") is not authorized to fetch this datatype (" + String.valueOf(viewPath) + ").", FaultCodes.UNAUTHORIZED);
            }

            HistoryOffset offset = getOffset(parameters);
            int limit = getRecordLimit(parameters);

            MDC.put("offset", String.valueOf(offset));
            MDC.put("limit", String.valueOf(limit));

            ////////////////////
            // Do actual work here
            ViewMapVO view = dynamicViewMapper.getViewMapForView(parameters.getRegister(), parameters.getDatatype(), parameters.getVersion());

            Document feedDocument = createFeed(view, parameters, offset, limit);

            // Construct the output container.
            ReplicationResponseType response = new ObjectFactory().createReplicationResponseType();

            response.setAny(feedDocument.getFirstChild());

            // Log that the client successfully accessed the data.
            // Simply for audit purposes.
            //
            // The client details are included in the MDC.
            //
            logger.info("Records fetched, sending response.");

            return response;

        } catch (ReplicationFault e) {
            throw e;
        } catch (Exception e) {
            throw new ReplicationFault("Could not complete the request do to an error.", FaultCodes.INTERNAL_ERROR, e);
        } finally {
            MDC.remove("view");
            MDC.remove("cvr");
            MDC.remove("offset");
            MDC.remove("limit");
        }
    }

    private int getRecordLimit(ReplicationRequestType parameters)
    {
        if (parameters.getMaxRecords() == null) {
            return MAX_RECORD_LIMIT;
        } else {
            return Math.min(parameters.getMaxRecords().intValue(), MAX_RECORD_LIMIT);
        }
    }

    private Document createFeed(ViewMapVO view, ReplicationRequestType parameters, HistoryOffset offset, int limit) {
        long pid = Long.parseLong(offset.getRecordID());
        Instant modifiedDate = new Instant(offset.getModifiedDate());

        try {
            List<DynamicRow> rows = dynamicRowFetcher.fetchRows(view, pid, modifiedDate, limit);
            auditLogOutput(rows);
            return dynamicViewXmlGenerator.generateXml(view, rows, parameters.getRegister(), parameters.getDatatype(), DateTime.now());
        } catch (SQLException e) {
            throw new RuntimeException("Database error", e);
        } catch (TransformerException e) {
            throw new RuntimeException("Transformer error", e);
        }
    }


    private void auditLogOutput(List<DynamicRow> rows) {
		TypeOfInformation informationType = AuditBuilder.TypeOfInformation.SensitivePersonalInformation;
    	String component = "SDM";
    	String context = "SKRS Replicate";

        AuditBuilder auditBuilder = Audit.createAuditBuilder();
        auditBuilder.addAuditInformation(component, context, informationType, "returned elements", rows.size());
        if (rows.size() > 0) {
        auditBuilder.addAuditInformation(component, context, informationType, "returned first element", DynamicViewXmlGenerator.mapRevisionId(rows.get(0)));
        auditBuilder.addAuditInformation(component, context, informationType, "returned last element", DynamicViewXmlGenerator.mapRevisionId(rows.get(rows.size()-1)));
        }
	}


	private String getViewPath(ReplicationRequestType parameters)
    {
        return format("%s/%s/v%d", parameters.getRegister(), parameters.getDatatype(), parameters.getVersion());
    }

    private HistoryOffset getOffset(ReplicationRequestType parameters) throws ReplicationFault
    {
        try {
            return new HistoryOffset(parameters.getOffset());
        } catch (Exception e) {
            throw new ReplicationFault("Invalid offset in the request.", FaultCodes.INVALID_OFFSET, e);
        }
    }
}
