/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.replication;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

import org.apache.commons.dbutils.QueryRunner;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.persistence.PersistenceModule;

import dk.nsi.stamdata.replication.models.ClientDao;
import dk.nsi.stamdata.replication.models.ClientVO;

public class TestTableCreator {
    public final static String TEST_TABLE = "testtable_skrs";
    public final static String TEST_REGISTER = "testreg";
    public final static String TEST_DATATYPE = "testtype";
    public static final String WHITELISTED_CVR = "25520041";

    @Inject
    private Provider<Connection> connectionProvider;
    @Inject
    private ClientDao clientDao;

    public void createTestTable() throws SQLException {
        String sql = "CREATE  TABLE IF NOT EXISTS `testtable_skrs` (\n" +
                "  `PID` BIGINT(15) NOT NULL AUTO_INCREMENT ,\n" +
                "  `tekst` VARCHAR(200) NULL ,\n" +
                "  `tal` INT NULL ,\n" +
                "  `stort_tal` BIGINT NULL ,\n" +
                "  `decimal_tal` DECIMAL(8,2) NULL ,\n" +
                "  `dato` DATE NULL ,\n" +
                "  `dato_tid` DATETIME NULL ,\n" +
                "  `floatingpoint` FLOAT NULL ,\n" +
                "  `flag` TINYINT(1) NULL ,\n" +
                "  `ModifiedDate` DATETIME NULL ,\n" +
                "  `ValidFrom` DATETIME NULL ,\n" +
                "  `ValidTo` DATETIME NULL ,\n" +
                "  PRIMARY KEY (`PID`) )\n" +
                "ENGINE = InnoDB";
        Connection conn = null;
        QueryRunner run = new QueryRunner();
        try {
            conn = connectionProvider.get();
            run.update(conn, sql);
            run.update(conn, "DELETE FROM testtable_skrs");
        } finally {
            PersistenceModule.close(conn);
        }
    }


    public long createAndWhiteListForTestView(long version) throws SQLException {
        deleleOldView(TEST_REGISTER, TEST_DATATYPE, version);
        long testViewId = insertNewView(TEST_REGISTER, TEST_DATATYPE, version, TEST_TABLE);
        addColumnToView(testViewId, true,  "PID",          0, -5, null);
        addColumnToView(testViewId, false, "tekst",        1, 12, 200);
        addColumnToView(testViewId, false, "tal",          2,  4, null);
        addColumnToView(testViewId, false, "stort_tal",    3, -5, null);
        addColumnToView(testViewId, false, "decimal_tal",  4,  3, null);
        addColumnToView(testViewId, false, "dato",         5, 91, null);
        addColumnToView(testViewId, false, "dato_tid",     6, 93, null);
        addColumnToView(testViewId, false, "floatingpoint",7,  6, null);
        addColumnToView(testViewId, false, "flag",         8, 16, null);
        addColumnToView(testViewId, false, "ModifiedDate", 9, 93, null);
        addColumnToView(testViewId, false, "ValidFrom",   10, 93, null);
        addColumnToView(testViewId, false, "ValidTo",     11, 93, null);
        whiteList(TEST_REGISTER + "/" + TEST_DATATYPE + "/v" + version);

        return testViewId;
    }

    private void deleleOldView(String register, String datatype, long version) throws SQLException {
        Connection conn = null;
        QueryRunner run = new QueryRunner();
        try {
            conn = connectionProvider.get();
            run.update(conn, "DELETE FROM SKRSColumns WHERE viewMap in (select idSKRSViewMapping from SKRSViewMapping WHERE register=? AND datatype=? AND version=?)", register, datatype, version);
            run.update(conn, "DELETE FROM SKRSViewMapping WHERE register=? AND datatype=? AND version=?", register, datatype, version);
        } finally {
            PersistenceModule.close(conn);
        }
    }


    public void removeView(long viewId) throws SQLException {
        Connection conn = null;
        QueryRunner run = new QueryRunner();
        try {
            conn = connectionProvider.get();
            run.update(conn, "DELETE FROM SKRSViewMapping WHERE idSKRSViewMapping=?", viewId);
        } finally {
            PersistenceModule.close(conn);
        }
    }

    public void insertTestRow(String tekst, int tal, int stortTal, double decimal, java.util.Date dato, java.util.Date datoTid,
                               float flydende, boolean falg) throws SQLException {
        String sql = "INSERT INTO testtable_skrs(tekst, tal, stort_tal, decimal_tal, dato, dato_tid, floatingpoint, flag," +
                " ModifiedDate, ValidFrom, ValidTo) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
        Timestamp now = new Timestamp((new java.util.Date()).getTime());
        Connection conn = null;
        QueryRunner run = new QueryRunner();
        try {
            conn = connectionProvider.get();
            run.update(conn, sql, tekst, tal, stortTal, decimal, dato, new Timestamp((new java.util.Date()).getTime()), flydende,
                    falg, now, now, null);
        } finally {
            PersistenceModule.close(conn);
        }
    }

    public void updateModifiedDateOnRecordsWithTekst(String tekst) throws SQLException {
        String sql = "UPDATE testtable_skrs SET ModifiedDate=? WHERE tekst=?";
        Timestamp now = new Timestamp((new java.util.Date()).getTime());
        Connection conn = null;
        QueryRunner run = new QueryRunner();
        try {
            conn = connectionProvider.get();
            run.update(conn, sql, now, tekst);
        } finally {
            PersistenceModule.close(conn);
        }
    }

    private void addColumnToView(long viewId, boolean isPid, String columnName, int position, int dataType, Integer maxLength) throws SQLException {
        String sql = "INSERT INTO SKRSColumns(viewMap, isPID, tableColumnName, feedColumnName, feedPosition, dataType, maxLength)";
        sql += " VALUES (?,?,?,?,?,?,?)";
        Connection conn = null;
        QueryRunner run = new QueryRunner();
        try {
            conn = connectionProvider.get();
            run.update(conn, sql, viewId, isPid, columnName, columnName, position, dataType, maxLength);
        } finally {
            PersistenceModule.close(conn);
        }
    }

    private long insertNewView(String register, String datatype, Long version, String tableName) throws SQLException {
        String sql1 = "INSERT INTO SKRSViewMapping(register, datatype, version, tableName, createdDate) ";
        sql1 += " VALUES(?,?,?,?,?)";
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        QueryRunner run = new QueryRunner();
        try {
            conn = connectionProvider.get();
            preparedStatement = conn.prepareStatement(sql1, Statement.RETURN_GENERATED_KEYS);

            run.fillStatement(preparedStatement, register, datatype, version, tableName, new Timestamp((new java.util.Date()).getTime()));
            preparedStatement.executeUpdate();
            ResultSet generatedKeys = preparedStatement.getGeneratedKeys();
            if (generatedKeys.next()) {
                return generatedKeys.getLong(1);
            }
        } finally {
            PersistenceModule.close(preparedStatement);
            PersistenceModule.close(conn);
        }
        return -1;
    }

    public void whiteList(String permission) throws SQLException {
        whiteList(permission,WHITELISTED_CVR);
    }

    public void whiteList(String permission, String cvr) throws SQLException {
        ClientVO client = clientDao.findByCvr(cvr);
        Connection connection = null;
        try {
            connection = connectionProvider.get();
            if (client == null) {
                String sql = "INSERT INTO Client(name,subjectSerialNumber) VALUES ('Region Syd','"+String.format("CVR:%s-UID:1234", cvr)+"')";
                connection.createStatement().executeUpdate(sql);
                connection.commit();
                client = clientDao.findByCvr(cvr);
            }
            String sql = "INSERT INTO Client_permissions(client_id,permissions) VALUES ("+client.getId()+",'"+permission+"')";
            connection.createStatement().executeUpdate(sql);
            connection.commit();
        } finally {
            PersistenceModule.close(connection);
        }
    }
}
