/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.replication.dao;

import com.google.inject.Inject;
import com.trifork.stamdata.persistence.PersistenceModule;

import dk.nsi.stamdata.replication.vo.ColumnMapVO;
import dk.nsi.stamdata.replication.vo.ViewMapVO;
import dk.nsi.stamdata.replication.webservice.GuiceTestRunner;
import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.handlers.ScalarHandler;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(GuiceTestRunner.class)
public class DynamicViewDAOTest {

    private final String testRegister = "test_register";
    private final String testDatatype = "test_type";
    private final String testTableName = "TestTable";

    @Inject
    private DynamicViewDAO dao;

    @Inject
    private Connection connection;

    private Long id = -1L;

    @Before
    public void setUp() throws SQLException {
        if (id != -1L) {
            removeMapping(id);
            id = -1L;
        }
        id = insertTestMapping();
    }

    @After
    public void tearDown() throws SQLException {
        removeMapping(id);
        PersistenceModule.close(connection);
    }

    @Test
    public void testGetViewMap() throws SQLException {

        ViewMapVO viewMapForView = dao.getViewMapForView(testRegister, testDatatype, 1);
        assertNotNull(viewMapForView);

        assertEquals(testRegister, viewMapForView.getRegister());
        assertEquals(testDatatype, viewMapForView.getDatatype());
        assertEquals(1, viewMapForView.getVersion());
        assertEquals(testTableName, viewMapForView.getTableName());
        // Not sure what created date is, but make sure its not null at least
        assertNotNull(viewMapForView.getCreatedDate());

        List<ColumnMapVO> columnMaps = viewMapForView.getColumnMaps();
        assertEquals(1, columnMaps.size());

        ColumnMapVO column1 = columnMaps.iterator().next();
        assertEquals(-1, column1.getDataType());
        assertEquals(1, column1.getFeedPosition());
        assertEquals((Integer)100, column1.getMaxLength());
        assertEquals("test_column", column1.getTableColumnName());
        assertEquals("test_feed_column", column1.getFeedColumnName());

        removeMapping(id);
    }

    private void removeMapping(Long id) throws SQLException {
        QueryRunner qr = new QueryRunner();
        qr.update(connection, "DELETE FROM SKRSViewMapping WHERE idSKRSViewMapping=?", id);
        connection.commit();
    }

    private Long insertTestMapping() throws SQLException {
        ScalarHandler handler = new ScalarHandler();
        QueryRunner qr = new QueryRunner();
        String sql = "INSERT INTO SKRSViewMapping (register, datatype, version, tableName, createdDate) VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE tableName=?, createdDate=?";
        Timestamp createDate = new Timestamp((new Date()).getTime());
        qr.update(connection, sql, testRegister, testDatatype, 1, testTableName, createDate, testTableName, createDate);

        String select = "SELECT idSKRSViewMapping FROM SKRSViewMapping WHERE register=? AND datatype=? AND version=?";
        Long id = (Long) qr.query(connection, select, handler, testRegister, testDatatype, 1);
        insertTestColumnMapping(connection, id);
        connection.commit();
        return id;
    }

    private void insertTestColumnMapping(Connection connection, Long viewId) throws SQLException {
        QueryRunner qr = new QueryRunner();
        String insert = "INSERT INTO SKRSColumns (viewMap, isPID, tableColumnName, feedColumnName, feedPosition, dataType, maxLength) " +
                        " VALUES(?, ?, ?, ?, ?, ?, ?)" +
                        " ON DUPLICATE KEY UPDATE isPID=?, feedColumnName=?, feedPosition=?, dataType=?, maxLength=?";
        qr.update(connection, insert, viewId, true, "test_column", "test_feed_column", 1, -1, 100, true, "test_feed_column", 1, -1, 100);
        connection.commit();
    }

}
