/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import static com.google.inject.name.Names.bindProperties;

import java.util.Set;

import javax.servlet.ServletContextEvent;

import org.apache.log4j.Logger;

import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Stage;
import com.google.inject.TypeLiteral;
import com.google.inject.servlet.GuiceServletContextListener;
import com.google.inject.servlet.ServletModule;
import com.trifork.stamdata.ComponentMonitor;
import com.trifork.stamdata.ConfigurationLoader;
import com.trifork.stamdata.MonitoringModule;
import com.trifork.stamdata.persistence.PersistenceModule;

import dk.nsi.stamdata.cpr.security.SecurityModule;
import dk.nsi.stamdata.security.DenGodeWebServiceFilter;
import dk.nsi.stamdata.security.DenGodeWebServiceModule;


public class ComponentController extends GuiceServletContextListener
{
	private static final TypeLiteral<Set<String>> A_SET_OF_STRINGS = new TypeLiteral<Set<String>>() {};
	private static final String DISPLAY_SOAP_FAULT_STACK_TRACE = "com.sun.xml.ws.fault.SOAPFaultBuilder.disableCaptureStackTrace";
	public static final String COMPONENT_NAME = "stamdata-cpr-ws";

    private static Logger logger;

    @Override
    public void contextDestroyed(ServletContextEvent servletContextEvent) {
        super.contextDestroyed(servletContextEvent);
        getLogger().info(servletContextEvent.getServletContext().getServletContextName() + " [Shutdown]");
    }

    @Override
    public void contextInitialized(ServletContextEvent servletContextEvent) {
        super.contextInitialized(servletContextEvent);
        getLogger().info(servletContextEvent.getServletContext().getServletContextName() + " [Started]");
    }

    @Override
	protected Injector getInjector()
	{
		return Guice.createInjector(Stage.PRODUCTION, new ComponentModule(), new ServiceModule());
	}


	public static class ComponentModule extends AbstractModule
	{
		@Override
		protected void configure()
		{
			// Load the components configuration and bind it to named
			// dependencies.

			bindProperties(binder(), ConfigurationLoader.loadForName(COMPONENT_NAME));
            getLogger().info("Configuring ComponentModule for " + COMPONENT_NAME);

            getLogger().info("Installing PersistenceModule");
			install(new PersistenceModule());

		}
	}


	private class ServiceModule extends ServletModule
	{
		@Override
		protected void configureServlets()
		{
			// We don't want JAX-WS to expose the stack trace
			// when an exception occurs.
            getLogger().info("Configuring ServiceModule for " + COMPONENT_NAME);
			System.setProperty(DISPLAY_SOAP_FAULT_STACK_TRACE, "false");

			// Tell the monitoring module how to monitor the component.
			// All monitor pages are bound to the URL /status.

			bind(ComponentMonitor.class).to(ComponentMonitorImpl.class);
            getLogger().info("Installing MonitoringModule");
			install(new MonitoringModule());


            //Install whitelist service modules
            getLogger().info("Installing SecurityModule");
            install(new SecurityModule());


			// Filter everything through the DGWS filter,
			// but exclude the status page.
			filterRegex("(?!/status)/.*").through(DenGodeWebServiceFilter.class);

            getLogger().info("Installing DenGodeWebServiceModule");
			install(new DenGodeWebServiceModule());

		}
	}

    private static Logger getLogger()
    {
        if (logger == null) {
            logger = Logger.getLogger(ComponentController.class);
        }
        return logger;
    }
}
