/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import static com.trifork.stamdata.Preconditions.checkNotNull;

import java.util.Calendar;
import java.util.TimeZone;
import java.util.UUID;

import javax.xml.soap.SOAPConstants;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPFault;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import org.apache.log4j.Logger;

import com.trifork.stamdata.util.SlaConsts;

import dk.nsi.stamdata.cpr.medcom.DetGodeCPROpslagCommon;
import dk.nsi.stamdata.cpr.medcom.FaultMessages;
import dk.nsi.stamdata.jaxws.generated.Header;
import dk.nsi.stamdata.jaxws.generated.Linking;
import dk.nsi.stamdata.jaxws.generated.Security;
import dk.nsi.stamdata.jaxws.generated.Timestamp;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sosi.seal.model.constants.FlowStatusValues;


public final class SoapUtils
{
	private static final TimeZone DGWS_TIMEZONE = TimeZone.getTimeZone("UTC");
    private static final Logger logger = Logger.getLogger(SoapUtils.class);

	private SoapUtils() {
	}

	public static <T> T newDGWSFault(Holder<Security> securityHeaderHolder,
                                     Holder<Header> medcomHeaderHolder,
                                     String status,
                                     String errorMsg,
                                     Class<T> clazz) {
		checkNotNull(securityHeaderHolder, "securityHeaderHolder");
		checkNotNull(medcomHeaderHolder, "medcomHeaderHolder");
		checkNotNull(status, "status");
		checkNotNull(errorMsg, "errorMsg");

		// The DGWS Specification says that this exact string must be returned.

		final String DGWS_ERROR_MSG = "DGWS error";

		Security securityHeader = new Security();

		Calendar cal = Calendar.getInstance(DGWS_TIMEZONE);
		cal.set(Calendar.MILLISECOND, 0);

		Timestamp timestamp = new Timestamp();
		timestamp.setCreated(cal);
		securityHeader.setTimestamp(timestamp);
		securityHeaderHolder.value = securityHeader;

		Header medcom = new Header();
		medcom.setFlowStatus(FlowStatusValues.FLOW_FINALIZED_SUCCESFULLY);

		Linking linking = new Linking();
		linking.setFlowID(medcomHeaderHolder.value.getLinking().getFlowID());
		linking.setInResponseToMessageID(medcomHeaderHolder.value.getLinking().getMessageID());
		linking.setMessageID(UUID.randomUUID().toString());
		medcom.setLinking(linking);

		medcomHeaderHolder.value = medcom;
		medcomHeaderHolder.value.setFlowStatus(status);

        try {
            return clazz.getConstructor(String.class, String.class).newInstance(errorMsg, DGWS_ERROR_MSG);
        } catch (Exception e) {
            return null;
        }
	}


	public static SOAPFaultException newSOAPSenderFault(String message) {
		checkNotNull(message, "message");

		SOAPFault fault;
        logger.warn("Fejl under processering af besked. "+message);

		try {
			// We have to make sure to use the same protocol version
			// as defined in the WSDL.
			SOAPFactory factory = SOAPFactory.newInstance(SOAPConstants.SOAP_1_1_PROTOCOL);

			fault = factory.createFault();
			fault.setFaultCode(SOAPConstants.SOAP_SENDER_FAULT);

			// TODO: For some reason the xml:lang att. is always "en"
			// even when the locale is set in this next call.

			fault.setFaultString(message);
		} catch (Exception e) {
			throw newServerErrorFault(e);
		}

		return new SOAPFaultException(fault);
	}


	public static RuntimeException newServerErrorFault(Exception e) {
		checkNotNull(e, "e");

		logger.error(FaultMessages.INTERNAL_SERVER_ERROR+" "+e.getMessage());
		return new RuntimeException(FaultMessages.INTERNAL_SERVER_ERROR, e);
	}


	public static void setHeadersToOutgoing(Holder<Security> wsseHeader, Holder<Header> medcomHeader) {
		setSecurityHeaderToOutgoing(wsseHeader);
		setMedcomHeaderToOutgoing(medcomHeader);
	}


	private static void setSecurityHeaderToOutgoing(Holder<Security> wsseHeader) {
		Security sec = new Security();
		Timestamp timestamp = new Timestamp();
		Calendar cal = Calendar.getInstance(DGWS_TIMEZONE);
		cal.set(Calendar.MILLISECOND, 0);
		timestamp.setCreated(cal);
		sec.setTimestamp(timestamp);
		wsseHeader.value = sec;
	}


	private static void setMedcomHeaderToOutgoing(Holder<Header> medcomHeader) {
		Header medcom = new Header();
		medcom.setFlowStatus(FlowStatusValues.FLOW_FINALIZED_SUCCESFULLY);

		Linking linking = new Linking();
		linking.setFlowID(medcomHeader.value.getLinking().getFlowID());
		linking.setInResponseToMessageID(medcomHeader.value.getLinking().getMessageID());
		linking.setMessageID(UUID.randomUUID().toString());
		medcom.setLinking(linking);

		medcomHeader.value = medcom;
	}

    public static void updateSlaLog(Holder<Header> medcomHeader, String operation, SLALogItem slaLogItem, String cvr) {
        Header value = medcomHeader.value;
        if (slaLogItem != null && value.getLinking() != null && value.getLinking().getMessageID() != null) {
            String messageID = value.getLinking().getMessageID();
            slaLogItem.setMessageId(messageID);
            slaLogItem.setSourceOperation(operation);
            slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_CVR, cvr);
        }
    }
}
