/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.zip.DataFormatException;

import org.apache.commons.lang.StringUtils;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.persistence.PersistenceModule;

import dk.nsi.stamdata.cpr.models.HealthCareInformation;

public class SikredeDAO {
    private final Provider<Connection> connectionProvider;

    @Inject
    public SikredeDAO(Provider<Connection> connectionProvider)
    {
        this.connectionProvider = connectionProvider;
    }

    public HealthCareInformation findHealthCareInformation(String cpr, Date currentTime) throws SQLException {
        PreparedStatement preparedStatement = null;
        Connection connection = null;
        try {
            String queryString = "SELECT SSikrGrpKode, SIkraftDatoGrp, SYdernr "+
            		"FROM Sikrede " +
                    "WHERE CPRnr = ? AND ValidFrom <= ? AND (ValidTo > ? OR ValidTo IS NULL) " +
                    "ORDER BY PID DESC LIMIT 1";
            connection = connectionProvider.get();
            preparedStatement = connection.prepareStatement(queryString);
            preparedStatement.setObject(1, cpr);
            preparedStatement.setObject(2, currentTime);
            preparedStatement.setObject(3, currentTime);
            ResultSet resultSet = preparedStatement.executeQuery();
            if (resultSet.next()) {
                String grpKode = resultSet.getString("SSikrGrpKode");
                String grpDate = resultSet.getString("SIkraftDatoGrp");
                String providerNo = resultSet.getString("SYdernr");
                HealthCareInformation hcInfo = new HealthCareInformation(dateFromSIKraftDatoGrp(grpDate), grpKode, providerNo);
                return hcInfo;
            } else {
                return null;
            }
        } finally {
            PersistenceModule.close(preparedStatement);
            PersistenceModule.close(connection);
        }
    }

    public HealthCareInformation findHealthCareInformationWithFrontierInformation(String cpr, Date currentTime) throws SQLException, ParseException {
        PreparedStatement preparedStatement = null;
        Connection connection = null;
        try {
            String queryString = "SELECT SYdernrGl, SStatus, SSikrGrpKode, SIkraftDatoGrp, SYdernr, "+
            		"SSKGyldigFra, SSKGyldigTil, SBSStatsborgerskab, SBSStatsborgerskabKode, SSKBopelsLand, SSKBopelsLAndKode, SSLSocSikretLand, SSLSocSikretLandKode, SSLForsikringsinstans, SSLForsikringsinstansKode, SSLForsikringsnr "+
            		"FROM Sikrede " +
                    "WHERE CPRnr = ? AND ValidFrom <= ? AND (ValidTo > ? OR ValidTo IS NULL) " +
                    "ORDER BY PID DESC LIMIT 1";
            connection = connectionProvider.get();
            preparedStatement = connection.prepareStatement(queryString);
            preparedStatement.setObject(1, cpr);
            preparedStatement.setObject(2, currentTime);
            preparedStatement.setObject(3, currentTime);
            ResultSet resultSet = preparedStatement.executeQuery();
            if (resultSet.next()) {
                String grpKode = resultSet.getString("SSikrGrpKode");
                String grpDate = resultSet.getString("SIkraftDatoGrp");
                String providerNo = resultSet.getString("SYdernr");
                HealthCareInformation hcInfo = new HealthCareInformation(dateFromSIKraftDatoGrp(grpDate), grpKode, providerNo);
                hcInfo.setSYdernrGl(resultSet.getString("SYdernrGl"));
                hcInfo.setSStatus(resultSet.getString("SStatus"));
                hcInfo.setSSKGyldigFra(dateFromSSKDato(resultSet.getString("SSKGyldigFra")));
                hcInfo.setSSKGyldigTil(dateFromSSKDato(resultSet.getString("SSKGyldigTil")));
                hcInfo.setSBSStatsborgerskab(resultSet.getString("SBSStatsborgerskab"));
                hcInfo.setSBSStatsborgerskabKode(resultSet.getString("SBSStatsborgerskabKode"));
                hcInfo.setSSKBopelsLand(resultSet.getString("SSKBopelsLand"));
                hcInfo.setSSKBopelsLAndKode(resultSet.getString("SSKBopelsLAndKode"));
                hcInfo.setSSLSocSikretLand(resultSet.getString("SSLSocSikretLand"));
                hcInfo.setSSLSocSikretLandKode(resultSet.getString("SSLSocSikretLandKode"));
                hcInfo.setSSLForsikringsinstans(resultSet.getString("SSLForsikringsinstans"));
                hcInfo.setSSLForsikringsinstansKode(resultSet.getString("SSLForsikringsinstansKode"));
                hcInfo.setSSLForsikringsnr(resultSet.getString("SSLForsikringsnr"));
                return hcInfo;
            } else {
                return null;
            }
        } finally {
            PersistenceModule.close(preparedStatement);
            PersistenceModule.close(connection);
        }
    }

    public static Date dateFromSSKDato(String sDato) throws ParseException {
    	if (StringUtils.isEmpty(sDato)) {
    		return null;
    	}
    	SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
		return formatter.parse(sDato);
    }

    public static Date dateFromSIKraftDatoGrp(String sIkraftDatoGrp) {
        int year = Integer.parseInt(sIkraftDatoGrp.substring(0, 4));
        int month = Integer.parseInt(sIkraftDatoGrp.substring(4, 6));
        int day = Integer.parseInt(sIkraftDatoGrp.substring(6, 8));
        Calendar cal = GregorianCalendar.getInstance();
        cal.set(year, month-1, day, 0, 0, 0);
        return cal.getTime();
    }
}
