/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.mapping;

import static dk.sosi.seal.model.constants.SubjectIdentifierTypeValues.CVR_NUMBER;

import java.math.BigInteger;
import java.util.Date;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Instant;

import com.trifork.stamdata.Preconditions;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.util.SlaConsts;

import dk.nsi.stamdata.cpr.SoapUtils;
import dk.nsi.stamdata.cpr.medcom.FaultMessages;
import dk.nsi.stamdata.security.WhitelistService;
import dk.oio.rep.cpr_dk.xml.schemas.core._2005._11._24.PersonBirthDateStructureType;
import dk.oio.rep.cpr_dk.xml.schemas.core._2006._01._17.PersonCivilRegistrationStatusStructureType;
import dk.oio.rep.cpr_dk.xml.schemas.core._2006._01._17.RegularCPRPersonType;
import dk.oio.rep.cpr_dk.xml.schemas.core._2006._01._17.SimpleCPRPersonType;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.CountryIdentificationCodeType;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.CountryIdentificationSchemeType;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.PersonGenderCodeType;
import dk.oio.rep.itst_dk.xml.schemas._2006._01._17.PersonNameStructureType;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.AssociatedGeneralPractitionerStructureType;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sosi.seal.model.SystemIDCard;

/**
 * Contains mapping functions that are shared between wsdl versions
 */
public class PersonMapper {

    public static enum CPRProtectionLevel {
        CensorCPR,
        DoNotCensorCPR
    }

    protected static final String UKENDT = "UKENDT";
    protected static final int AUTHORITY_CODE_LENGTH = 4;
    protected static final String SERVICE_NAME_DGCPR = WhitelistService.DEFAULT_SERVICE_NAME;
    protected WhitelistService whitelistService;

    protected static final String ADRESSEBESKYTTET = "ADRESSEBESKYTTET";
    protected static final String PROTECTED_CPR = "0000000000";
    protected final SystemIDCard idCard;
    protected final MunicipalityMapper munucipalityMapper;
    protected final SLALogItem slaLogItem;

    public PersonMapper(WhitelistService whitelistService, SystemIDCard idCard, MunicipalityMapper munucipalityMapper,
                        SLALogItem slaLogItem) {
        // Once we get this far the filter should have gotten rid of id cards
        // that are not CVR authenticated System ID Cards.
        this.whitelistService = whitelistService;
        this.idCard = idCard;
        this.munucipalityMapper = munucipalityMapper;
        this.slaLogItem = slaLogItem;
    }

    protected boolean isClientAnAuthority() {
        String careProviderType = idCard.getSystemInfo().getCareProvider().getType();
        Preconditions.checkState(CVR_NUMBER.equals(careProviderType), "ID Card Care provider is not a CVR. This is a programming error.");
        String clientCVR = idCard.getSystemInfo().getCareProvider().getID();
        return whitelistService.isCvrWhitelisted(clientCVR, SERVICE_NAME_DGCPR);
    }

    public boolean isPersonProtected(PersonVO person) {
        if (person.getNavnebeskyttelsestartdato() == null) return false;

        // We have to make the guard above to avoid null being passed into the
        // Instant
        // it is converted to the beginning of the era.

        Preconditions.checkState(person.getNavnebeskyttelseslettedato() != null, "The protection end date was not present. This is most unexpected and a programming error.");

        Instant protectionStart = new Instant(person.getNavnebeskyttelsestartdato());
        Instant protectionEnd = new Instant(person.getNavnebeskyttelseslettedato());

        return protectionStart.isEqualNow() || (protectionStart.isBeforeNow() && protectionEnd.isAfterNow());
    }

    protected CountryIdentificationCodeType mapCountryCodeFromStatus(String sstatus) {
        if (StringUtils.isNotBlank(sstatus)) {
            long status = Long.parseLong(sstatus);
            if (status == 1L || status == 3L || status == 5L || status == 7L) {
                CountryIdentificationCodeType country = new CountryIdentificationCodeType();
                // Two alpha-numerical characters.
                country.setScheme(CountryIdentificationSchemeType.ISO_3166_ALPHA_2);
                if (status == 1L || status == 3L) {
                    country.setValue("DK");
                } else {
                    country.setValue("GL");
                }
                return country;
            }
        }
        return null;
    }

    protected PersonGenderCodeType mapGenderToGenderCode(String genderString) {
        if ("M".equalsIgnoreCase(genderString)) {
            return PersonGenderCodeType.MALE;
        } else if ("K".equalsIgnoreCase(genderString)) {
            return PersonGenderCodeType.FEMALE;
        } else {
            return PersonGenderCodeType.UNKNOWN;
        }
    }

    protected RegularCPRPersonType createRegularPerson(PersonVO person, CPRProtectionLevel cprProtection) {
        RegularCPRPersonType regularCprPerson = new RegularCPRPersonType();
        SimpleCPRPersonType simpleCprPerson = new SimpleCPRPersonType();
        regularCprPerson.setSimpleCPRPerson(simpleCprPerson);

        // PERSON NAME STRUCTURE SECTION
        //
        // Many of these values might not be present in the database
        // this is because that 'Such is life' and sometimes we simply
        // don't know a person's first name, address or birthday.
        PersonNameStructureType personName = new PersonNameStructureType();
        simpleCprPerson.setPersonNameStructure(personName);
        personName.setPersonGivenName(actualOrUnknown("PersonGivenName", person.getFornavn()));

        // Middle name is optional.
        personName.setPersonMiddleName(actualOrNull(person.getMellemnavn()));
        personName.setPersonSurnameName(actualOrUnknown("PersonSurnameName", person.getEfternavn()));
        if (cprProtection == CPRProtectionLevel.DoNotCensorCPR) {
            simpleCprPerson.setPersonCivilRegistrationIdentifier(person.getCpr());
        } else {
            simpleCprPerson.setPersonCivilRegistrationIdentifier(PROTECTED_CPR);
        }

        regularCprPerson.setPersonNameForAddressingName(
                actualOrUnknown("PersonNameForAddressingName", person.getNavnTilAdressering()));
        regularCprPerson.setPersonGenderCode(mapGenderToGenderCode(person.getKoen()));
        regularCprPerson.setPersonInformationProtectionIndicator(isPersonProtected(person));

        regularCprPerson.setPersonBirthDateStructure(createPersonBirthDateStructure(person));

        // CIVIL STATUS
        if (person.getStatus() == null) {
            // Vi kan ikke give noget fornuftigt svar hvis personen ikke engang har en status
            throw SoapUtils.newSOAPSenderFault(FaultMessages.INCOMPLETE_DATA_FOUND_FAULT_MSG);
        }
        PersonCivilRegistrationStatusStructureType personCivil = new PersonCivilRegistrationStatusStructureType();
        personCivil.setPersonCivilRegistrationStatusCode(new BigInteger(person.getStatus()));
        personCivil.setPersonCivilRegistrationStatusStartDate(newXMLGregorianCalendar(person.getStatusDato()));

        regularCprPerson.setPersonCivilRegistrationStatusStructure(personCivil);
        return regularCprPerson;
    }

    protected PersonBirthDateStructureType createPersonBirthDateStructure(PersonVO person) {
        // BIRTH DATE
        PersonBirthDateStructureType personBirthDate = new PersonBirthDateStructureType();

        personBirthDate.setBirthDate(newXMLGregorianCalendar(person.getFoedselsdato()));
        boolean foedselsdatoMarkering = false;
        if (person.getFoedselsdatoMarkering() != null) {
            String markering = person.getFoedselsdatoMarkering();
            if (markering.length() > 0 && !markering.equals("0")) {
                foedselsdatoMarkering = true;
            }
        }
        personBirthDate.setBirthDateUncertaintyIndicator(foedselsdatoMarkering);
        return personBirthDate;
    }

    protected RegularCPRPersonType createCensoredRegularPerson(String cpr, CPRProtectionLevel cprProtection) {
        RegularCPRPersonType regularCprPerson = new RegularCPRPersonType();
        SimpleCPRPersonType simpleCprPerson = new SimpleCPRPersonType();

        // PERSON NAME STRUCTURE SECTION

        PersonNameStructureType personName = new PersonNameStructureType();
        simpleCprPerson.setPersonNameStructure(personName);

        personName.setPersonGivenName(ADRESSEBESKYTTET);
        personName.setPersonMiddleName(null);
        personName.setPersonSurnameName(ADRESSEBESKYTTET);

        // For some searches we also need to censor the CPR number.

        boolean isCPRHidden = cprProtection == CPRProtectionLevel.CensorCPR;
        simpleCprPerson.setPersonCivilRegistrationIdentifier(isCPRHidden ? PROTECTED_CPR : cpr);

        regularCprPerson.setSimpleCPRPerson(simpleCprPerson);
        regularCprPerson.setPersonNameForAddressingName(ADRESSEBESKYTTET);

        // Even though the CPR can be read from the CPR (when the CPR is
        // included),
        // we might as well always protect it.
        regularCprPerson.setPersonGenderCode(PersonGenderCodeType.UNKNOWN);
        regularCprPerson.setPersonInformationProtectionIndicator(true);

        // BIRTH DATE
        PersonBirthDateStructureType personBirthDate = new PersonBirthDateStructureType();
        personBirthDate.setBirthDate(newXMLGregorianCalendar(new Date(0)));
        personBirthDate.setBirthDateUncertaintyIndicator(true);
        regularCprPerson.setPersonBirthDateStructure(personBirthDate);

        // CIVIL STATUS
        PersonCivilRegistrationStatusStructureType personCivil = new PersonCivilRegistrationStatusStructureType();
        personCivil.setPersonCivilRegistrationStatusCode(BigInteger.ONE);
        personCivil.setPersonCivilRegistrationStatusStartDate(newXMLGregorianCalendar(new Date(0)));
        regularCprPerson.setPersonCivilRegistrationStatusStructure(personCivil);

        return regularCprPerson;
    }

    protected AssociatedGeneralPractitionerStructureType createDummyPractitioner(String placeholderText)
    {
        Preconditions.checkNotNull(placeholderText, "placeholderText");

        AssociatedGeneralPractitionerStructureType associatedGeneralPractitioner = new AssociatedGeneralPractitionerStructureType();
        associatedGeneralPractitioner.setAssociatedGeneralPractitionerIdentifier(BigInteger.ZERO);
        associatedGeneralPractitioner.setAssociatedGeneralPractitionerOrganisationName(placeholderText);
        associatedGeneralPractitioner.setDistrictName(placeholderText);
        associatedGeneralPractitioner.setEmailAddressIdentifier(placeholderText + "@example.com");
        associatedGeneralPractitioner.setPostCodeIdentifier("0000");

        associatedGeneralPractitioner.setStandardAddressIdentifier(placeholderText);
        associatedGeneralPractitioner.setTelephoneSubscriberIdentifier("00000000");

        return associatedGeneralPractitioner;
    }

    // Postcode identifier must be exactly 4 characters according to xsd so pad with zeroes, since production
    // data is not always compliant with this.
    public static String normalizePostCode(String postCode) {
        if (postCode == null) {
            return "0000";
        }
        return StringUtils.leftPad(postCode, 4, '0');
    }

    public static String normalizePostDistrict(String postDistrict) {
        // Skema maks længde = 20
        if (postDistrict != null && postDistrict.length() > 20) {
            return postDistrict.substring(0, 20);
        }
        return actualOrUnknown(postDistrict);
    }

    protected void addMutatedToSla(String fieldName, String to) {
        if (slaLogItem != null) {
            slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_MUTATED + "." + fieldName, "Value set to: " + to);
        }
    }

    //////////////////////////////////////////////////

    protected String actualOrUnknown(String fieldName, String actual) {
        if (StringUtils.isBlank(actual)) {
            addMutatedToSla(fieldName, UKENDT);
            return UKENDT;
        } else {
            return actual;
        }
    }

    protected static String actualOrUnknown(String actual)
    {
        return !StringUtils.isBlank(actual) ? actual : UKENDT;
    }

    protected static String actualOrNull(String actual)
    {
        return !StringUtils.isBlank(actual) ? actual : null;
    }

    public static XMLGregorianCalendar newXMLGregorianCalendar(Date date)
    {
        DatatypeFactory factory;
        try {
            factory = DatatypeFactory.newInstance();
        } catch (DatatypeConfigurationException e) {
            throw new RuntimeException(e);
        }
        return factory.newXMLGregorianCalendar(new DateTime(date).toGregorianCalendar());
    }

}
