/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.mapping.v102;

import java.util.Date;

import javax.xml.datatype.DatatypeConfigurationException;

import org.apache.commons.lang.StringUtils;

import com.google.inject.Inject;
import com.google.inject.servlet.RequestScoped;
import com.trifork.stamdata.Nullable;
import com.trifork.stamdata.Preconditions;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.YderVO;

import dk.nsi.stamdata.cpr.mapping.CivilRegistrationStatusCodes;
import dk.nsi.stamdata.cpr.mapping.MunicipalityMapper;
import dk.nsi.stamdata.cpr.mapping.YderregisterRecordToAssociatedGeneralPractitionerMapper;
import dk.nsi.stamdata.cpr.models.HealthCareInformation;
import dk.nsi.stamdata.security.WhitelistService;
import dk.oio.rep.cpr_dk.xml.schemas.core._2006._01._17.RegularCPRPersonType;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.CountryIdentificationCodeType;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.AssociatedGeneralPractitionerStructureType;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sosi.seal.model.SystemIDCard;
import oio.medcom.cprservice._1_0.AddressAccessType;
import oio.medcom.cprservice._1_0.AddressCompleteType;
import oio.medcom.cprservice._1_0.AddressPostalType;
import oio.medcom.cprservice._1_0.ObjectFactory;
import oio.medcom.cprservice._1_0.PersonAddressStructureType;
import oio.medcom.cprservice._1_0.PersonHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0.PersonInformationStructureType;
import oio.medcom.cprservice._1_0.PersonPublicHealthInsuranceType;
import oio.medcom.cprservice._1_0.PersonWithHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0.PublicHealthInsuranceGroupIdentifierType;

/**
 * Person mapper for wsdl version 1.0.2
 */
@RequestScoped
public class PersonMapper extends dk.nsi.stamdata.cpr.mapping.PersonMapper {

    @Inject
    public PersonMapper(WhitelistService whitelistService, SystemIDCard idCard, MunicipalityMapper munucipalityMapper,
                        SLALogItem slaLogItem) {
        super(whitelistService, idCard, munucipalityMapper, slaLogItem);
    }

    public PersonInformationStructureType map(PersonVO person, CPRProtectionLevel cprProtection) {
        boolean censorData = isPersonProtected(person);
        return (censorData) ? createOutputWithCensoredDate(person, cprProtection) :
                createOutputWithRealData(person, cprProtection);
    }

    public PersonInformationStructureType createOutputWithCensoredDate(
            PersonVO person, CPRProtectionLevel cprProtection) {
        PersonInformationStructureType output = new ObjectFactory().createPersonInformationStructureType();
        if (cprProtection == CPRProtectionLevel.DoNotCensorCPR) {
            mapCurrentPersonCivilRegistrationIdentifier(person, output);
        }

        RegularCPRPersonType regularCprPerson = createCensoredRegularPerson(person.getCpr(), cprProtection);

        // PERSON ADDRESS
        output.setPersonAddressStructure(createFakePersonAddressStructure(ADRESSEBESKYTTET));
        output.getPersonAddressStructure().setPersonInformationProtectionStartDate(newXMLGregorianCalendar(person.getNavnebeskyttelsestartdato()));
        output.setRegularCPRPerson(regularCprPerson);

        return output;
    }

    public PersonInformationStructureType createOutputWithRealData(PersonVO person, CPRProtectionLevel cprProtection) {
        // There are many cases in which we cannot fulfill the output format
        // just by using data present in the db. Therefore we have to 'fill'
        // in the gaps with e.g. 'UNKNOWN'.
        PersonInformationStructureType output = new ObjectFactory().createPersonInformationStructureType();

        if (cprProtection == CPRProtectionLevel.DoNotCensorCPR) {
            mapCurrentPersonCivilRegistrationIdentifier(person, output);
        }
        RegularCPRPersonType regularCprPerson = createRegularPerson(person,cprProtection);

        // PERSON ADDRESS
        //
        // There are also records for which the record's values should not be
        // used. For instance if a person is dead or missing his address should
        // not be filled in. Yet the output schema requires them to be there.
        boolean shouldIncludeAddress = !CivilRegistrationStatusCodes.STATUSES_WITH_NO_ADDRESS.contains(person.getStatus());
        if (shouldIncludeAddress) {
            PersonAddressStructureType personAddress = new PersonAddressStructureType();

            if (person.getNavnebeskyttelsestartdato() != null) {
                personAddress.setPersonInformationProtectionStartDate(newXMLGregorianCalendar(person.getNavnebeskyttelsestartdato()));
            }

            // The output requires that the authority codes (municipality and
            // county) are exactly four char long.
            AddressCompleteType addressComplete = new AddressCompleteType();
            AddressAccessType addressAccess = new AddressAccessType();
            addressComplete.setAddressAccess(addressAccess);

            if (StringUtils.isNotBlank(person.getKommuneKode())) {
                String municipalityCode = StringUtils.leftPad(person.getKommuneKode(), AUTHORITY_CODE_LENGTH, "0");
                addressAccess.setMunicipalityCode(municipalityCode);
                personAddress.setCountyCode(munucipalityMapper.toCountyCode(municipalityCode));
            } else {
                addressAccess.setMunicipalityCode("0000");
                addMutatedToSla("MunicipalityCode", "0000");
                personAddress.setCountyCode("0000");
                addMutatedToSla("CountyCode", "0000");
            }

            personAddress.setCareOfName(actualOrNull(person.getCoNavn()));
            if (person.getVejKode() == null) {
                person.setVejKode("");
                addMutatedToSla("StreetCode", "0000");
            }
            addressAccess.setStreetCode(StringUtils.leftPad(person.getVejKode(), AUTHORITY_CODE_LENGTH, "0"));
            addressAccess.setStreetBuildingIdentifier(actualOrNull(person.getHusnummer()));

            AddressPostalType addressPostal = new AddressPostalType();
            addressComplete.setAddressPostal(addressPostal);

            // The following field is not included in the source
            // "CPR Registeret"
            // therefore we cannot fill the element.
            //
            // if (StringUtils.isNotBlank(""))
            // {
            // This is:
            // The given name of a farm, estate, building or dwelling, which is
            // used as a additional postal address identifier.
            //
            // addressPostal.setMailDeliverySublocationIdentifier("Fake Value");
            // }

            addressPostal.setStreetName(actualOrUnknown("StreetName", person.getVejnavn()));
            addressPostal.setStreetNameForAddressingName(actualOrNull(person.getVejnavnTilAdressering()));
            addressPostal.setStreetBuildingIdentifier(actualOrNull(person.getHusnummer()));

            addressPostal.setFloorIdentifier(actualOrNull(person.getEtage()));
            addressPostal.setSuiteIdentifier(actualOrNull(person.getSideDoerNummer()));

            addressPostal.setDistrictSubdivisionIdentifier(actualOrNull(person.getBynavn()));

            // Post Box is excluded since a persons address cannot be a Post Box.
            if (person.getPostnummer() == null) {
                addMutatedToSla("PostCodeIdentifier", "0000");
            }
            addressPostal.setPostCodeIdentifier(PersonMapper.normalizePostCode(person.getPostnummer()));
            String postDistrict = PersonMapper.normalizePostDistrict(person.getPostdistrikt());
            if (!postDistrict.equals(person.getPostdistrikt())) {
                addMutatedToSla("DistrictName", postDistrict);
            }
            addressPostal.setDistrictName(postDistrict);

            CountryIdentificationCodeType countryIdentificationCode = mapCountryCodeFromStatus(person.getStatus());
            if (countryIdentificationCode != null) addressPostal.setCountryIdentificationCode(countryIdentificationCode);

            personAddress.setAddressComplete(addressComplete);
            output.setPersonAddressStructure(personAddress);
        } else {
            output.setPersonAddressStructure(createFakePersonAddressStructure(UKENDT));
        }

        output.setRegularCPRPerson(regularCprPerson);

        return output;
    }


    public PersonWithHealthCareInformationStructureType map(PersonVO person,
                                                            @Nullable HealthCareInformation healthCareGroup,
                                                            @Nullable YderVO yder)
            throws DatatypeConfigurationException {
        Preconditions.checkNotNull(person, "person");

        PersonWithHealthCareInformationStructureType personWithHealthCare = new PersonWithHealthCareInformationStructureType();

        PersonInformationStructureType personInformation = map(person, CPRProtectionLevel.DoNotCensorCPR);
        personWithHealthCare.setPersonInformationStructure(personInformation);

        PersonHealthCareInformationStructureType personHealthCareInformation = new PersonHealthCareInformationStructureType();
        personWithHealthCare.setPersonHealthCareInformationStructure(personHealthCareInformation);

        // Fill the associated general practitioner.
        AssociatedGeneralPractitionerStructureType associatedGeneralPractitioner;
        if (yder == null) {
            associatedGeneralPractitioner = createDummyPractitioner(UKENDT);
        } else {
            YderregisterRecordToAssociatedGeneralPractitionerMapper yderregisterRecordToAssociatedGeneralPractitionerMapper = new YderregisterRecordToAssociatedGeneralPractitionerMapper();
            associatedGeneralPractitioner = yderregisterRecordToAssociatedGeneralPractitionerMapper.map(yder, slaLogItem);
        }

        personHealthCareInformation.setAssociatedGeneralPractitionerStructure(associatedGeneralPractitioner);
        PersonPublicHealthInsuranceType personPublicHealthInsurance;
        if (healthCareGroup == null) {
            personPublicHealthInsurance = createDummyPublicHealthInsurance(UKENDT);
        } else {
            HealthCareGroupToPersonPublicHealthInsuranceMapper healthCareGroupToPersonPublicHealhInsuranceMapper =
                    new HealthCareGroupToPersonPublicHealthInsuranceMapper();
            personPublicHealthInsurance = healthCareGroupToPersonPublicHealhInsuranceMapper.map(healthCareGroup);
        }

        personHealthCareInformation.setPersonPublicHealthInsurance(personPublicHealthInsurance);
        return personWithHealthCare;
    }

    public PersonPublicHealthInsuranceType createDummyPublicHealthInsurance(String placeholderText) throws DatatypeConfigurationException {
        Preconditions.checkNotNull(placeholderText, "placeholderText");
        PersonPublicHealthInsuranceType personPublicHealthInsurance = new PersonPublicHealthInsuranceType();
        personPublicHealthInsurance.setPublicHealthInsuranceGroupStartDate(newXMLGregorianCalendar(new Date(0)));
        personPublicHealthInsurance.setPublicHealthInsuranceGroupIdentifier(PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_1);
        return personPublicHealthInsurance;
    }

    private PersonAddressStructureType createFakePersonAddressStructure(String placeholderText)
    {
        PersonAddressStructureType personAddress = new PersonAddressStructureType();
        personAddress.setPersonInformationProtectionStartDate(null);

        personAddress.setCountyCode("0000");

        AddressCompleteType addressComplete = new AddressCompleteType();

        AddressAccessType addressAccess = new AddressAccessType();
        addressAccess.setMunicipalityCode("0000");
        addressAccess.setStreetCode("0000");

        addressComplete.setAddressAccess(addressAccess);
        AddressPostalType addressPostal = new AddressPostalType();
        addressComplete.setAddressPostal(addressPostal);

        addressPostal.setStreetName(placeholderText);
        addressPostal.setPostCodeIdentifier("0000");
        addressPostal.setDistrictName(placeholderText);

        personAddress.setAddressComplete(addressComplete);

        return personAddress;
    }

    private void mapCurrentPersonCivilRegistrationIdentifier(PersonVO person, PersonInformationStructureType output) {
        if (StringUtils.isNotBlank(person.getGaeldendeCPR())) {
            output.setCurrentPersonCivilRegistrationIdentifier(person.getGaeldendeCPR());
        }
    }

}
