/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.mapping.v104;

import java.sql.SQLException;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.xml.datatype.DatatypeConfigurationException;

import org.apache.commons.lang.StringUtils;

import com.google.inject.Inject;
import com.google.inject.servlet.RequestScoped;
import com.trifork.stamdata.Nullable;
import com.trifork.stamdata.Preconditions;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.YderVO;

import dk.nsi.stamdata.cpr.SoapUtils;
import dk.nsi.stamdata.cpr.dao.CprDAO;
import dk.nsi.stamdata.cpr.dao.CprDAO.Custody;
import dk.nsi.stamdata.cpr.dao.CprDAO.Spouse;
import dk.nsi.stamdata.cpr.dao.CprDAO.Ward;
import dk.nsi.stamdata.cpr.mapping.CivilRegistrationStatusCodes;
import dk.nsi.stamdata.cpr.mapping.MunicipalityMapper;
import dk.nsi.stamdata.cpr.mapping.YderregisterRecordToAssociatedGeneralPractitionerMapper;
import dk.nsi.stamdata.cpr.medcom.FaultMessages;
import dk.nsi.stamdata.cpr.models.HealthCareInformation;
import dk.nsi.stamdata.security.WhitelistService;
import dk.oio.rep.cpr_dk.xml.schemas.core._2005._11._24.PersonBirthDateStructureType;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.CountryIdentificationCodeType;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.PersonGenderCodeType;
import dk.oio.rep.itst_dk.xml.schemas._2006._01._17.PersonNameStructureType;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.AssociatedGeneralPractitionerStructureType;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sosi.seal.model.SystemIDCard;
import oio.medcom.cprservice._1_0_4.AddressAccessType;
import oio.medcom.cprservice._1_0_4.AddressCompleteType;
import oio.medcom.cprservice._1_0_4.AddressPostalType;
import oio.medcom.cprservice._1_0_4.CurrentGeneralPractitionerStructureType;
import oio.medcom.cprservice._1_0_4.FrontierCitizenshipStructureType;
import oio.medcom.cprservice._1_0_4.FrontierInformationStructureType;
import oio.medcom.cprservice._1_0_4.FrontierResidenceCountryStructureType;
import oio.medcom.cprservice._1_0_4.FrontierSocialSecurityCountryStructureType;
import oio.medcom.cprservice._1_0_4.FrontierSocialSecurityStructureType;
import oio.medcom.cprservice._1_0_4.ObjectFactory;
import oio.medcom.cprservice._1_0_4.PersonAddressStructureType;
import oio.medcom.cprservice._1_0_4.PersonCivilRegistrationStatusStructureType;
import oio.medcom.cprservice._1_0_4.PersonCustodyRelationType;
import oio.medcom.cprservice._1_0_4.PersonCustodyStructureType;
import oio.medcom.cprservice._1_0_4.PersonHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4.PersonInformationStructureType;
import oio.medcom.cprservice._1_0_4.PersonPublicHealthInsuranceType;
import oio.medcom.cprservice._1_0_4.PersonSpouseRelationType;
import oio.medcom.cprservice._1_0_4.PersonSpouseStructureType;
import oio.medcom.cprservice._1_0_4.PersonWardRelationType;
import oio.medcom.cprservice._1_0_4.PersonWardStructureType;
import oio.medcom.cprservice._1_0_4.PersonWithHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4.PriorGeneralPractitionerStructureType;
import oio.medcom.cprservice._1_0_4.PublicHealthInsuranceGroupIdentifierType;
import oio.medcom.cprservice._1_0_4.RegularCPRPersonType;
import oio.medcom.cprservice._1_0_4.SimpleCPRPersonType;

/**
 * Person mapper for wsdl version 1.0.4
 */
@RequestScoped
public class PersonMapper extends dk.nsi.stamdata.cpr.mapping.PersonMapper {

	private CprDAO cprDao;

    @Inject
    public PersonMapper(WhitelistService whitelistService, SystemIDCard idCard, MunicipalityMapper munucipalityMapper,
                        SLALogItem slaLogItem) {
        super(whitelistService, idCard, munucipalityMapper, slaLogItem);
    }

    public PersonInformationStructureType map(PersonVO person, CPRProtectionLevel cprProtection) {
        boolean censorData = isPersonProtected(person);
        return (censorData) ? createPersonInformationStructureCensored(person, cprProtection) :
                createPersonInformationStructure(person);
    }

    // public PersonInformationStructureType

	private PersonInformationStructureType createPersonInformationStructureCensored(
            PersonVO person, CPRProtectionLevel cprProtection) {
        PersonInformationStructureType output = new ObjectFactory().createPersonInformationStructureType();
        mapCurrentPersonCivilRegistrationIdentifier(person, output, cprProtection);
        mapPriorPersonCivilRegistrationIdentifier(person, output, cprProtection);

        RegularCPRPersonType regularCprPerson = createCensoredRegularPerson104(person.getCpr(), cprProtection);

        // PERSON ADDRESS
        output.setPersonAddressStructure(createFakePersonAddressStructure(ADRESSEBESKYTTET));
        output.getPersonAddressStructure().setPersonInformationProtectionStartDate(newXMLGregorianCalendar(person.getNavnebeskyttelsestartdato()));
        output.setRegularCPRPerson(regularCprPerson);

        mapPersonSpouseStructure(person, output);
        mapPersonCustodyStructure(person,output);
        mapPersonWardStructure(person,output);

        return output;
    }

    private PersonInformationStructureType createPersonInformationStructure(PersonVO person) {
        // There are many cases in which we cannot fulfill the output format
        // just by using data present in the db. Therefore we have to 'fill'
        // in the gaps with e.g. 'UNKNOWN'.
        PersonInformationStructureType output = new ObjectFactory().createPersonInformationStructureType();

        mapCurrentPersonCivilRegistrationIdentifier(person, output, CPRProtectionLevel.DoNotCensorCPR);
        mapPriorPersonCivilRegistrationIdentifier(person, output, CPRProtectionLevel.DoNotCensorCPR);
        mapRegularCPRPerson(person, output);
        output.setPersonAddressStructure(createPersonAddressStructure(person));
        mapPersonSpouseStructure(person, output);
        mapPersonCustodyStructure(person,output);
        mapPersonWardStructure(person,output);

        return output;
    }

    private void mapPersonSpouseStructure(PersonVO person, PersonInformationStructureType output) {
        PersonSpouseStructureType personSpouseStructure = createPersonSpouseStructure(person);
        if (personSpouseStructure != null) {
            output.setPersonSpouseStructure(personSpouseStructure);
        }
    }

    private void mapPersonWardStructure(PersonVO person, PersonInformationStructureType output) {
        try {
            Collection<Ward> wardCollection = cprDao.findWard(person.getCpr());
            List<PersonWardStructureType> personWardStructure = output.getPersonWardStructure();

            Iterator<Ward> iter = wardCollection.iterator();
            while (iter.hasNext()) {
                Ward ward = iter.next();
                if (StringUtils.isNotBlank(ward.getIdentifier())) {
                    PersonWardStructureType personWard = new PersonWardStructureType();
                    personWard.setPersonWardIdentifier(ward.getIdentifier());
                    personWard.setPersonWardRelation(PersonWardRelationType.fromValue(ward.getRelation().value()));
                    personWardStructure.add(personWard);
                }
            }
        } catch (SQLException e) {
                throw SoapUtils.newServerErrorFault(e);
        }
    }

    private void mapPersonCustodyStructure(PersonVO person, PersonInformationStructureType output) {
        try {
            Collection<Custody> custodies = cprDao.findCustodies(person.getCpr());
            List<PersonCustodyStructureType> personCustodyStructure = output.getPersonCustodyStructure();

            Iterator<Custody> iter = custodies.iterator();
            while (iter.hasNext()) {
                Custody custody = iter.next();
                if (StringUtils.isNotBlank(custody.getIdentifier())) {
                    PersonCustodyStructureType personCustody = new PersonCustodyStructureType();
                    personCustody.setPersonCustodyIdentifier(custody.getIdentifier());
                    personCustody.setPersonCustodyRelation(PersonCustodyRelationType.fromValue(custody.getRelation().value()));
                    personCustodyStructure.add(personCustody);
                }
            }
        } catch (SQLException e) {
            throw SoapUtils.newServerErrorFault(e);
        }

    }

    private PersonSpouseStructureType createPersonSpouseStructure(PersonVO person) {
        PersonSpouseStructureType personSpouseStructure = new PersonSpouseStructureType();

        try {
            Spouse spouse = cprDao.findSpouse(person.getCpr());
            if (spouse != null && spouse.getRelation() != null && StringUtils.isNotBlank(spouse.getIdentifier())) {
                personSpouseStructure.setPersonSpouseIdentifier(spouse.getIdentifier());
                PersonSpouseRelationType relation = PersonSpouseRelationType.fromValue(spouse.getRelation().value());
                personSpouseStructure.setPersonSpouseRelation(relation);

                return personSpouseStructure;
            } else {
                return null;
            }
        } catch (SQLException e) {
            throw SoapUtils.newServerErrorFault(e);
        }
    }

    private PersonAddressStructureType createPersonAddressStructure(PersonVO person) {
        // There are also records for which the record's values should not be
        // used. For instance if a person is dead or missing his address should
        // not be filled in. Yet the output schema requires them to be there.
        if (CivilRegistrationStatusCodes.STATUSES_WITH_NO_ADDRESS.contains(person.getStatus())) {
            return createFakePersonAddressStructure(UKENDT);
        }

        PersonAddressStructureType personAddress = new PersonAddressStructureType();

        personAddress.setCareOfName(actualOrNull(person.getCoNavn()));
        personAddress.setAddressComplete(createAddressComplete(person));
        if (person.getNavnebeskyttelsestartdato() != null) {
            personAddress.setPersonInformationProtectionStartDate(newXMLGregorianCalendar(person.getNavnebeskyttelsestartdato()));
        }
        if (personAddress.getAddressComplete() != null) {
            personAddress.setCountyCode(munucipalityMapper.toCountyCode(personAddress.getAddressComplete().getAddressAccess().getMunicipalityCode()));
        }

        return personAddress;
    }

    private AddressCompleteType createAddressComplete(PersonVO person) {
        // The output requires that the authority codes (municipality and
        // county) are exactly four char long.
        AddressCompleteType addressComplete = new AddressCompleteType();

        addressComplete.setAddressAccess(createAddressAccess(person));
        addressComplete.setAddressPostal(createAddressPostal(person));

        return addressComplete;
    }

    private AddressPostalType createAddressPostal(PersonVO person) {
        AddressPostalType addressPostal = new AddressPostalType();

        // The MailDeliverySublocationIdentifier field is not included in the source
        // "CPR Registeret". It is optional and so will not be filled

        addressPostal.setStreetName(actualOrUnknown("StreetName", person.getVejnavn()));
        addressPostal.setStreetNameForAddressingName(actualOrNull(person.getVejnavnTilAdressering()));
        addressPostal.setStreetBuildingIdentifier(actualOrNull(person.getHusnummer()));
        addressPostal.setFloorIdentifier(actualOrNull(person.getEtage()));
        addressPostal.setSuiteIdentifier(actualOrNull(person.getSideDoerNummer()));
        addressPostal.setDistrictSubdivisionIdentifier(actualOrNull(person.getBynavn()));
        // The optional field PostBox is excluded since a persons address cannot be a Post Box.
        String normalizePostCode = PersonMapper.normalizePostCode(person.getPostnummer());
        if (!normalizePostCode.equals(person.getPostnummer())) {
            addMutatedToSla("PostCodeIdentifier", "0000");
        }
        addressPostal.setPostCodeIdentifier(normalizePostCode);

        String normalizedPostDistrict = PersonMapper.normalizePostDistrict(person.getPostdistrikt());
        if (!normalizedPostDistrict.equals(person.getPostdistrikt())) {
            addMutatedToSla("DistrictName", normalizedPostDistrict);
        }
        addressPostal.setDistrictName(normalizedPostDistrict);

        CountryIdentificationCodeType countryIdentificationCode = mapCountryCodeFromStatus(person.getStatus());
        if (countryIdentificationCode != null) {
            addressPostal.setCountryIdentificationCode(countryIdentificationCode);
        }
        return addressPostal;
    }

    private AddressAccessType createAddressAccess(PersonVO person) {
        AddressAccessType addressAccess = new AddressAccessType();

        if (StringUtils.isNotBlank(person.getKommuneKode())) {
            String municipalityCode = StringUtils.leftPad(person.getKommuneKode(), AUTHORITY_CODE_LENGTH, "0");
            addressAccess.setMunicipalityCode(municipalityCode);
        } else {
            addressAccess.setMunicipalityCode("0000");
            addMutatedToSla("MunicipalityCode", "0000");
            addMutatedToSla("CountyCode", "0000");
        }

        try {
            addressAccess.setMunicipalityCodeStartDate(newXMLGregorianCalendar(cprDao.findMunicipalityCodeStartDate(person.getCpr())));
        } catch (SQLException e) {
            throw SoapUtils.newServerErrorFault(e);
        }

        if (person.getVejKode() == null) {
            person.setVejKode("");
            addMutatedToSla("StreetCode", "0000");
        }
        addressAccess.setStreetCode(StringUtils.leftPad(person.getVejKode(), AUTHORITY_CODE_LENGTH, "0"));
        addressAccess.setStreetBuildingIdentifier(actualOrNull(person.getHusnummer()));
        return addressAccess;
    }

    private void mapRegularCPRPerson(PersonVO person, PersonInformationStructureType output) {
        RegularCPRPersonType regularCprPerson = new RegularCPRPersonType();
        regularCprPerson.setSimpleCPRPerson(createSimpleCprPerson(person,output));

        regularCprPerson.setPersonNameForAddressingName(actualOrUnknown("PersonNameForAddressingName", person.getNavnTilAdressering()));
        regularCprPerson.setPersonGenderCode(mapGenderToGenderCode(person.getKoen()));
        regularCprPerson.setPersonInformationProtectionIndicator(false);
        regularCprPerson.setPersonBirthDateStructure(createPersonBirthDateStructure(person));
        regularCprPerson.setPersonCivilRegistrationStatusStructure(createPersonCivilRegistrationStructure(person));

        output.setRegularCPRPerson(regularCprPerson);
    }

    private PersonCivilRegistrationStatusStructureType createPersonCivilRegistrationStructure(PersonVO person) {
        if (person.getStatus() == null) {
            // Vi kan ikke give noget fornuftigt svar hvis personen ikke engang har en status
            throw SoapUtils.newSOAPSenderFault(FaultMessages.INCOMPLETE_DATA_FOUND_FAULT_MSG);
        }
        PersonCivilRegistrationStatusStructureType personCivil = new PersonCivilRegistrationStatusStructureType();
        personCivil.setPersonCivilRegistrationStatusCode(person.getStatus());
        personCivil.setPersonCivilRegistrationStatusStartDate(newXMLGregorianCalendar(person.getStatusDato()));
        return personCivil;
    }

    private SimpleCPRPersonType createSimpleCprPerson(PersonVO person, PersonInformationStructureType output) {
        SimpleCPRPersonType simpleCprPerson = new SimpleCPRPersonType();
        PersonNameStructureType personName = new PersonNameStructureType();
        simpleCprPerson.setPersonNameStructure(personName);
        personName.setPersonGivenName(actualOrUnknown("PersonGivenName", person.getFornavn()));

        // Middle name is optional.
        personName.setPersonMiddleName(actualOrNull(person.getMellemnavn()));
        personName.setPersonSurnameName(actualOrUnknown("PersonSurnameName", person.getEfternavn()));
        simpleCprPerson.setPersonCivilRegistrationIdentifier(person.getCpr());
        return simpleCprPerson;
    }

    public PersonWithHealthCareInformationStructureType map(PersonVO person,
                                                            @Nullable HealthCareInformation healthCareInformation,
                                                            @Nullable YderVO yder, @Nullable YderVO yderGL)
            throws DatatypeConfigurationException {
        Preconditions.checkNotNull(person, "person");

        PersonWithHealthCareInformationStructureType personWithHealthCare = new PersonWithHealthCareInformationStructureType();

        personWithHealthCare.setPersonInformationStructure(map(person, CPRProtectionLevel.DoNotCensorCPR));
        personWithHealthCare.setPersonHealthCareInformationStructure(createPersonHealthCareInformationStructure(healthCareInformation, yder, yderGL));

        if (HealthCareInformation.valid(healthCareInformation)) {
            personWithHealthCare.setFrontierInformationStructure(createFrontierHealthCareInformationStructure(healthCareInformation));
        }

        return personWithHealthCare;
    }

    private FrontierInformationStructureType createFrontierHealthCareInformationStructure(
            HealthCareInformation healthCareInformation) {
        FrontierInformationStructureType frontierInformation = new FrontierInformationStructureType();

        frontierInformation.setFrontierStartDate(newXMLGregorianCalendar(healthCareInformation.getSSKGyldigFra()));
        frontierInformation.setFrontierEndDate(newXMLGregorianCalendar(healthCareInformation.getSSKGyldigTil()));

        mapFrontierCitizenshipStructure(healthCareInformation, frontierInformation);
        mapFrontierResidenceCountryStructure(healthCareInformation, frontierInformation);
        mapFrontierSocialSecurityStructure(healthCareInformation, frontierInformation);
        return frontierInformation;
    }

    private void mapFrontierSocialSecurityStructure(HealthCareInformation healthCareInformation, FrontierInformationStructureType frontierInformation) {
        if (healthCareInformation.containsSocialSecurityData()) {
        	FrontierSocialSecurityStructureType socialSecurity = new FrontierSocialSecurityStructureType();

        	FrontierSocialSecurityCountryStructureType ssCountry = new FrontierSocialSecurityCountryStructureType();
        	ssCountry.setFrontierSocialSecurityCountry(healthCareInformation.getSSLSocSikretLand());
        	ssCountry.setFrontierSocialSecurityCountryCode(healthCareInformation.getSSLSocSikretLandKode());
        	socialSecurity.setFrontierSocialSecurityCountryStructure(ssCountry);

        	frontierInformation.setFrontierSocialSecurityStructure(socialSecurity);
        }
    }

    private void mapFrontierResidenceCountryStructure(HealthCareInformation healthCareInformation, FrontierInformationStructureType frontierInformation) {
        if (healthCareInformation.containsResidenceCountryData()) {
        	FrontierResidenceCountryStructureType residenceCountry = new FrontierResidenceCountryStructureType();

        	residenceCountry.setFrontierResidenceCountry(healthCareInformation.getSSKBopelsLand());
        	residenceCountry.setFrontierResidenceCountryCode(healthCareInformation.getSSKBopelsLAndKode());

        	frontierInformation.setFrontierResidenceCountryStructure(residenceCountry);
        }
    }

    private void mapFrontierCitizenshipStructure(HealthCareInformation healthCareInformation, FrontierInformationStructureType frontierInformation) {
        if (healthCareInformation.containsCitizenshipData()) {
        	FrontierCitizenshipStructureType citizenship = new FrontierCitizenshipStructureType();

        	citizenship.setFrontierCitizenship(healthCareInformation.getSBSStatsborgerskab());
        	citizenship.setFrontierCitizenshipCode(healthCareInformation.getSBSStatsborgerskabKode());

        	frontierInformation.setFrontierCitizenshipStructure(citizenship);
        }
    }

    private PersonHealthCareInformationStructureType createPersonHealthCareInformationStructure(
            HealthCareInformation healthCareInformation, YderVO yder, YderVO yderGL)
            throws DatatypeConfigurationException {
        PersonHealthCareInformationStructureType personHealthCareInformation = new PersonHealthCareInformationStructureType();

        personHealthCareInformation.setCurrentGeneralPractitionerStructure(createCurrentGeneralPractitionerStructure(yder));
        if (yderGL != null) {
        	personHealthCareInformation.setPriorGeneralPractitionerStructure(createPriorGeneralPractitionerStructure(yderGL));
        }
        personHealthCareInformation.setPersonPublicHealthInsurance(createPersonPublicHealthInsurance(healthCareInformation));

        return personHealthCareInformation;
    }

    private PersonPublicHealthInsuranceType createPersonPublicHealthInsurance(HealthCareInformation healthCareInformation) throws DatatypeConfigurationException {
        PersonPublicHealthInsuranceType personPublicHealthInsurance = null;
        if (healthCareInformation == null) {
            personPublicHealthInsurance = createDummyPublicHealthInsurance(UKENDT);
        } else {
            personPublicHealthInsurance = new PersonPublicHealthInsuranceType();
            personPublicHealthInsurance.setPublicHealthInsuranceGroupIdentifier(PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_1);
            personPublicHealthInsurance.setPublicHealthInsuranceGroupStartDate(newXMLGregorianCalendar(new Date(0)));

            personPublicHealthInsurance = HealthCareGroupToPersonPublicHealthInsuranceMapper.map(healthCareInformation);
        }
        return personPublicHealthInsurance;
    }

    private PriorGeneralPractitionerStructureType createPriorGeneralPractitionerStructure(YderVO yderGL) {
        PriorGeneralPractitionerStructureType priorGeneralPractitioner = new PriorGeneralPractitionerStructureType();
        priorGeneralPractitioner.setAssociatedGeneralPractitionerStructure(YderregisterRecordToAssociatedGeneralPractitionerMapper.map(yderGL, null));
        return priorGeneralPractitioner;
    }

    private CurrentGeneralPractitionerStructureType createCurrentGeneralPractitionerStructure(YderVO yder) {
        CurrentGeneralPractitionerStructureType currentGeneralPractitionerStructure = new CurrentGeneralPractitionerStructureType();

        AssociatedGeneralPractitionerStructureType associatedGeneralPractitioner = null;

        if (yder == null) {
            associatedGeneralPractitioner = createDummyPractitioner(UKENDT);
        } else {
            associatedGeneralPractitioner = YderregisterRecordToAssociatedGeneralPractitionerMapper.map(yder, slaLogItem);
        }
        currentGeneralPractitionerStructure.setAssociatedGeneralPractitionerStructure(associatedGeneralPractitioner);
        if (yder != null) {
            String cvr = (String) yder.getCVR();
            if (StringUtils.isNotBlank(cvr)) {
                currentGeneralPractitionerStructure.setCVRnumberIdentifier(cvr);
            }
        }
        return currentGeneralPractitionerStructure;
    }

	public PersonPublicHealthInsuranceType createDummyPublicHealthInsurance(String placeholderText) throws DatatypeConfigurationException {
        Preconditions.checkNotNull(placeholderText, "placeholderText");
        PersonPublicHealthInsuranceType personPublicHealthInsurance = new PersonPublicHealthInsuranceType();
        personPublicHealthInsurance.setPublicHealthInsuranceGroupIdentifier(PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_1);
        personPublicHealthInsurance.setPublicHealthInsuranceGroupStartDate(newXMLGregorianCalendar(new Date(0)));
        return personPublicHealthInsurance;
    }

    private PersonAddressStructureType createFakePersonAddressStructure(String placeholderText)
    {
        PersonAddressStructureType personAddress = new PersonAddressStructureType();
        personAddress.setPersonInformationProtectionStartDate(null);

        personAddress.setCountyCode("0000");

        AddressCompleteType addressComplete = new AddressCompleteType();

        AddressAccessType addressAccess = new AddressAccessType();
        addressAccess.setMunicipalityCode("0000");
        addressAccess.setStreetCode("0000");

        addressComplete.setAddressAccess(addressAccess);
        AddressPostalType addressPostal = new AddressPostalType();
        addressComplete.setAddressPostal(addressPostal);

        addressPostal.setStreetName(placeholderText);
        addressPostal.setPostCodeIdentifier("0000");
        addressPostal.setDistrictName(placeholderText);

        personAddress.setAddressComplete(addressComplete);

        return personAddress;
    }

    private void mapCurrentPersonCivilRegistrationIdentifier(PersonVO person, PersonInformationStructureType output, CPRProtectionLevel cprProtection) {
        if (StringUtils.isNotBlank(person.getGaeldendeCPR())) {
            output.setCurrentPersonCivilRegistrationIdentifier(protectCPR(person.getGaeldendeCPR(), cprProtection));
        }
    }

    private void mapPriorPersonCivilRegistrationIdentifier(PersonVO person, PersonInformationStructureType output, CPRProtectionLevel cprProtection) {
        if (cprProtection == CPRProtectionLevel.DoNotCensorCPR) {
            try {
                String priorCPR = cprDao.findPriorCpr(person.getCpr());
                if (StringUtils.isNotBlank(priorCPR)) {
                    output.setPriorPersonCivilRegistrationIdentifier(priorCPR);
                }
            } catch (SQLException e) {
                throw SoapUtils.newServerErrorFault(e);
            }
        }
    }

    protected RegularCPRPersonType createCensoredRegularPerson104(String cpr, CPRProtectionLevel cprProtection) {
        RegularCPRPersonType regularCprPerson = new RegularCPRPersonType();
        SimpleCPRPersonType simpleCprPerson = new SimpleCPRPersonType();

        // PERSON NAME STRUCTURE SECTION

        PersonNameStructureType personName = new PersonNameStructureType();
        simpleCprPerson.setPersonNameStructure(personName);

        personName.setPersonGivenName(ADRESSEBESKYTTET);
        personName.setPersonMiddleName(null);
        personName.setPersonSurnameName(ADRESSEBESKYTTET);

        // For some searches we also need to censor the CPR number.

        simpleCprPerson.setPersonCivilRegistrationIdentifier(protectCPR(cpr, cprProtection));

        regularCprPerson.setSimpleCPRPerson(simpleCprPerson);
        regularCprPerson.setPersonNameForAddressingName(ADRESSEBESKYTTET);

        // Even though the CPR can be read from the CPR (when the CPR is
        // included),
        // we might as well always protect it.
        regularCprPerson.setPersonGenderCode(PersonGenderCodeType.UNKNOWN);
        regularCprPerson.setPersonInformationProtectionIndicator(true);

        // BIRTH DATE
        PersonBirthDateStructureType personBirthDate = new PersonBirthDateStructureType();
        personBirthDate.setBirthDate(newXMLGregorianCalendar(new Date(0)));
        personBirthDate.setBirthDateUncertaintyIndicator(true);
        regularCprPerson.setPersonBirthDateStructure(personBirthDate);

        // CIVIL STATUS
        PersonCivilRegistrationStatusStructureType personCivil = new PersonCivilRegistrationStatusStructureType();
        personCivil.setPersonCivilRegistrationStatusCode(CivilRegistrationStatusCodes.AKTIV);
        personCivil.setPersonCivilRegistrationStatusStartDate(newXMLGregorianCalendar(new Date(0)));
        regularCprPerson.setPersonCivilRegistrationStatusStructure(personCivil);

        return regularCprPerson;
    }

    private String protectCPR(String cpr, CPRProtectionLevel cprProtection) {
        return (cprProtection == CPRProtectionLevel.CensorCPR) ? PROTECTED_CPR : cpr;
    }

    public void setCprDao(CprDAO cprDao) {
            this.cprDao = cprDao;
    }
}
