/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.util;

import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import com.google.inject.Inject;
import com.trifork.stamdata.ConfigurationLoader;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.util.SlaConsts;

import dk.nsi.stamdata.cpr.ComponentController;
import dk.nsi.stamdata.cpr.SoapUtils;
import dk.nsi.stamdata.cpr.medcom.FaultMessages;
import dk.sdsd.nsp.slalog.api.SLALogItem;

public class UnreliableHandler {

    private static final DateTimeFormatter DATETIME_FORMATTER = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss");
    private static final String FIELD_DEFAULT_PREFIX="field.default.";
    private static final String FIELD_DEFAULT_THROW_VALUE="<throw>";

    private static Map<String, String> unreliableMap = null;
    private final SLALogItem slaLogItem;

    @Inject
    public UnreliableHandler(SLALogItem slaLogItem) {
        this.slaLogItem = slaLogItem;
        if (unreliableMap == null) {
            unreliableMap = new HashMap<String, String>();
            Properties properties = ConfigurationLoader.loadForName(ComponentController.COMPONENT_NAME);
            Enumeration<String> enumeration = (Enumeration<String>) properties.propertyNames();
            while (enumeration.hasMoreElements()) {
                String propertyName = enumeration.nextElement();
                if (propertyName.startsWith(FIELD_DEFAULT_PREFIX)) {
                    String value = properties.getProperty(propertyName);
                    unreliableMap.put(propertyName.substring(FIELD_DEFAULT_PREFIX.length()), value);
                }
            }
        }
    }

    /**
     * Indsæt default værdier for utilregnelige felter
     * @param person Person hvis felter skal gennegåes
     */
    public void defaultUnreliableFields(PersonVO person) {
       if (person != null) {
            String unreliableCsv = person.getUnreliable();
            if (unreliableCsv != null && unreliableCsv.length() > 0) {
                String[] unreliables = unreliableCsv.split(",");
                for (String unreliable : unreliables) {
                    defaultFieldNamed(unreliable, person);
                }
            }
        }
    }

    protected void defaultFieldNamed(String fieldName, PersonVO person) {
        for (String propertyName : unreliableMap.keySet()) {
            if (fieldName.equalsIgnoreCase(propertyName)) {
                String propertyValue = unreliableMap.get(propertyName);
                if (propertyValue.equals(FIELD_DEFAULT_THROW_VALUE)) {
                    // Config says we should throw if we are unsure of this value
                    throw SoapUtils.newSOAPSenderFault(FaultMessages.UNRELIABLE_DATA_ERROR);
                } else {
                    setPropertyOnPerson(propertyName, propertyValue, person);
                    addUnreliableToSla(propertyName, propertyValue);
                }
            }
        }
    }

    private void addUnreliableToSla(String propertyName, String propertyValue) {
        if (slaLogItem != null) {
            slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_UNRELIABLE + "." + propertyName, "Value set to: " + propertyValue);
        }
    }

    private void setPropertyOnPerson(String propertyName, String propertyValue, PersonVO person) {
        switch(propertyName) {
        case "koen":
            person.setKoen(propertyValue);
        case "fornavn":
            person.setFornavn(propertyValue);
            break;
        case "mellemnavn":
            person.setMellemnavn(propertyValue);
            break;
        case "efternavn":
            person.setEfternavn(propertyValue);
            break;
        case "conavn":
            person.setCoNavn(propertyValue);
            break;
        case "lokalitet":
            person.setLokalitet(propertyValue);
            break;
        case "vejnavn":
            person.setVejnavn(propertyValue);
            break;
        case "bygningsnummer":
            person.setBygningsnummer(propertyValue);
            break;
        case "husnummer":
            person.setHusnummer(propertyValue);
            break;
        case "etage":
            person.setEtage(propertyValue);
            break;
        case "sideDoerNummer":
            person.setSideDoerNummer(propertyValue);
            break;
        case "bynavn":
            person.setBynavn(propertyValue);
            break;
        case "postnummer":
            person.setPostnummer(propertyValue);
            break;
        case "postdistrikt":
            person.setPostdistrikt(propertyValue);
            break;
        case "status":
            person.setStatus(propertyValue);
            break;
        case "gaeldendeCPR":
            person.setGaeldendeCPR(propertyValue);
            break;
        case "foedselsdato":
            DateTime dateTime = DATETIME_FORMATTER.parseDateTime(propertyValue);
            person.setFoedselsdato(dateTime.toDate());
            break;
        case "stilling":
            person.setStilling(propertyValue);
            break;
        case "vejKode":
            person.setVejKode(propertyValue);
            break;
        case "kommuneKode":
            person.setKommuneKode(propertyValue);
            break;
        default:
            throw new IllegalArgumentException("Unknown method "+propertyName);
        }
    }

}
