/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import static org.hamcrest.CoreMatchers.anyOf;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.common.collect.Sets;
import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO.FMRTypeKode;
import com.trifork.stamdata.models.PersonVO;

import dk.nsi.stamdata.cpr.dao.CprDAO;
import dk.nsi.stamdata.cpr.dao.CprDAO.Custody;
import dk.nsi.stamdata.cpr.dao.CprDAO.CustodyRelation;
import dk.nsi.stamdata.cpr.dao.CprDAO.Spouse;
import dk.nsi.stamdata.cpr.dao.CprDAO.SpouseRelation;
import dk.nsi.stamdata.cpr.dao.CprDAO.Ward;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;

@RunWith(GuiceTestRunner.class)
public class CprDaoTest {
    private static final DateTime YESTERDAY = new DateTime(Factories.YESTERDAY);
    private static final Date CURRENT_TIME = new Date();
    private static final String CPR = "1234567890";
    private static final String CPR1 = "1234567891";
    private static final String CPR2 = "1234567892";
    private static final String CPR3 = "1234567893";
    private static final String CPR4 = "1234567894";
    private static final String CPR5 = "1234567895";
    @Inject
    private Connection connection;
    @Inject
    private CprDAO cprDao;

    StamdataTestHelper testHelper = new StamdataTestHelper();

    @Before
    public void setUp() throws Exception {
        testHelper.clear();
    }

    @Test
    public void findCustodies() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personMor = testHelper.createKvinde(CPR1);
        PersonVO personFar = testHelper.createMand(CPR2);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personFar, person);
        testHelper.prepareTestdata(connection);

        Collection<Custody> custodies = cprDao.findCustodies(CPR);
        assertEquals(2,custodies.size());
        Iterator<Custody> iter = custodies.iterator();
        Custody custody = iter.next();
        assertEquals(CPR1,custody.getIdentifier());
        assertEquals(CustodyRelation.MOR,custody.getRelation());
        custody = iter.next();
        assertEquals(CPR2,custody.getIdentifier());
        assertEquals(CustodyRelation.FAR,custody.getRelation());
    }

    @Test
    public void findCustodiesMedmor() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personMor = testHelper.createKvinde(CPR1);
        PersonVO personMedmor = testHelper.createKvinde(CPR3);
        PersonVO personFar = testHelper.createMand(CPR2);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personMedmor, person);
        testHelper.createBarnRelation(personFar, person);
        testHelper.prepareTestdata(connection);

        Collection<Custody> custodies = cprDao.findCustodies(CPR);
        assertEquals("Custodies "+Arrays.toString(custodies.toArray()),3,custodies.size());
        Set<String> foundCpr = Sets.newHashSet();
        Iterator<Custody> iter = custodies.iterator();
        while (iter.hasNext()) {
            Custody custody = iter.next();
            foundCpr.add(custody.getIdentifier());
            switch (custody.getRelation()) {
            case MOR:
                assertThat(custody.getIdentifier(),anyOf(is(CPR1),is(CPR3)));
                break;
            case FAR:
                assertEquals(CPR2,custody.getIdentifier());
                break;
            case VAERGE:
            case MOR_UDEN:
            case FAR_UDEN:
            default:
                fail("Wrong relation: "+custody.toString());
                break;
            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),3,foundCpr.size());
    }

    @Test
    public void findCustodiesMedfar() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personMor = testHelper.createKvinde(CPR1);
        PersonVO personFar = testHelper.createMand(CPR2);
        PersonVO personMedfar = testHelper.createMand(CPR3);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personFar, person);
        testHelper.createBarnRelation(personMedfar, person);
        testHelper.prepareTestdata(connection);

        Collection<Custody> custodies = cprDao.findCustodies(CPR);
        assertEquals("Custodies "+Arrays.toString(custodies.toArray()),3,custodies.size());
        Set<String> foundCpr = Sets.newHashSet();
        Iterator<Custody> iter = custodies.iterator();
        while (iter.hasNext()) {
            Custody custody = iter.next();
            foundCpr.add(custody.getIdentifier());
            switch (custody.getRelation()) {
            case FAR:
                assertThat(custody.getIdentifier(),anyOf(is(CPR2),is(CPR3)));
                break;
            case MOR_UDEN:
                assertEquals(CPR1,custody.getIdentifier());
                break;
            case VAERGE:
            case MOR:
            case FAR_UDEN:
            default:
                fail("Wrong relation: "+custody.toString());
                break;
            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),3,foundCpr.size());
    }

    @Test
    public void findCustodiesVaerge() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personMor = testHelper.createKvinde(CPR1);
        PersonVO personFar = testHelper.createMand(CPR2);
        PersonVO personVaerge = testHelper.createKvinde(CPR3);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personFar, person);
        testHelper.createVaerge(person, personVaerge);
        testHelper.prepareTestdata(connection);

        Collection<Custody> custodies = cprDao.findCustodies(CPR);
        assertEquals("Custodies "+Arrays.toString(custodies.toArray()),3,custodies.size());
        Set<String> foundCpr = Sets.newHashSet();
        Iterator<Custody> iter = custodies.iterator();
        while (iter.hasNext()) {
            Custody custody = iter.next();
            foundCpr.add(custody.getIdentifier());
            switch (custody.getRelation()) {
            case MOR:
                assertEquals(CPR1,custody.getIdentifier());
                break;
            case FAR_UDEN:
                assertEquals(CPR2,custody.getIdentifier());
                break;
            case VAERGE:
                assertEquals(CPR3,custody.getIdentifier());
                break;
            case MOR_UDEN:
            case FAR:
            default:
                fail("Wrong relation: "+custody.toString());
                break;
            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),3,foundCpr.size());
    }

    @Test
    public void findCustodiesVaergeFar() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personMor = testHelper.createKvinde(CPR1);
        PersonVO personFar = testHelper.createMand(CPR2);
        PersonVO personVaerge = testHelper.createKvinde(CPR3);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personFar, person);
        testHelper.createVaerge(person, personVaerge);
        testHelper.prepareTestdata(connection);

        Collection<Custody> custodies = cprDao.findCustodies(CPR);
        assertEquals("Custodies "+Arrays.toString(custodies.toArray()),3,custodies.size());
        Set<String> foundCpr = Sets.newHashSet();
        Iterator<Custody> iter = custodies.iterator();
        while (iter.hasNext()) {
            Custody custody = iter.next();
            foundCpr.add(custody.getIdentifier());
            switch (custody.getRelation()) {
            case MOR_UDEN:
                assertEquals(CPR1,custody.getIdentifier());
                break;
            case FAR:
                assertEquals(CPR2,custody.getIdentifier());
                break;
            case VAERGE:
                assertEquals(CPR3,custody.getIdentifier());
                break;
            default:
                fail("Wrong relation: "+custody.toString());
                break;
            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),3,foundCpr.size());
    }

    @Test
    public void findCustodiesAI() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personMor = testHelper.createKvinde(CPR1);
        PersonVO personFar = testHelper.createMand(CPR2);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, personMor);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, personFar);
        testHelper.prepareTestdata(connection);

        Collection<Custody> custodies = cprDao.findCustodies(CPR);
        assertEquals("Custodies "+Arrays.toString(custodies.toArray()),2,custodies.size());
        Set<String> foundCpr = Sets.newHashSet();
        Iterator<Custody> iter = custodies.iterator();
        while (iter.hasNext()) {
            Custody custody = iter.next();
            foundCpr.add(custody.getIdentifier());
            switch (custody.getRelation()) {
            case ANDEN:
                assertThat(custody.getIdentifier(),anyOf(is(CPR1),is(CPR2)));
                break;
            case MOR:
            case FAR:
            case VAERGE:
            case MOR_UDEN:
            case FAR_UDEN:
            default:
                fail("Wrong relation: "+custody.toString());
                break;
            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),2,foundCpr.size());
    }

    @Test
    public void findCustodiesAI2() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personMor = testHelper.createKvinde(CPR1);
        PersonVO personMammy = testHelper.createKvinde(CPR2);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, personMor);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI2, personMammy);
        testHelper.prepareTestdata(connection);

        Collection<Custody> custodies = cprDao.findCustodies(CPR);
        assertEquals("Custodies "+Arrays.toString(custodies.toArray()),2,custodies.size());
        Set<String> foundCpr = Sets.newHashSet();
        Iterator<Custody> iter = custodies.iterator();
        while (iter.hasNext()) {
            Custody custody = iter.next();
            foundCpr.add(custody.getIdentifier());
            switch (custody.getRelation()) {
            case ANDEN:
                assertThat(custody.getIdentifier(),anyOf(is(CPR1),is(CPR2)));
                break;
            case MOR:
            case FAR:
            case VAERGE:
            case MOR_UDEN:
            case FAR_UDEN:
            default:
                fail("Wrong relation: "+custody.toString());
                break;
            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),2,foundCpr.size());
    }

    @Test
    public void findMunicipalityCodeStartDate() throws SQLException {
        testHelper.createPerson(CPR);
        testHelper.prepareTestdata(connection);
        StamdataTestHelper.assertDates(Factories.YEAR_2000, cprDao.findMunicipalityCodeStartDate(CPR));
    }

    @Test
    public void findPriorCpr() throws SQLException {
        testHelper.createPerson(CPR);
        testHelper.prepareTestdata(connection);
        assertEquals(StamdataTestHelper.PRIOR_CPR, cprDao.findPriorCpr(CPR));
    }

    public void findRelations() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personMor = testHelper.createKvinde(CPR1);
        PersonVO personFar = testHelper.createMand(CPR2);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personFar, person);
        testHelper.prepareTestdata(connection);

        Collection<String> relations = cprDao.findRelations(CPR);
        assertEquals(2,relations.size());
        Iterator<String> iter = relations.iterator();
        String relations_cpr = iter.next();
        assertEquals(CPR1,relations_cpr);
        relations_cpr = iter.next();
        assertEquals(CPR2,relations_cpr);
    }

    @Test
    public void findRelationsVaerge() throws Exception {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO person1 = testHelper.createKvinde(CPR1);
        PersonVO person2 = testHelper.createKvinde(CPR2);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, person1);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI2, person2);
        testHelper.prepareTestdata(connection);

        Collection<String> relations = cprDao.findRelations(CPR);
        assertEquals(2,relations.size());
        Iterator<String> iter = relations.iterator();
        String relations_cpr = iter.next();
        assertEquals(CPR1,relations_cpr);
        relations_cpr = iter.next();
        assertEquals(CPR2,relations_cpr);
    }

    @Test
    public void findSpouse() throws SQLException {
        PersonVO person = testHelper.createMand(CPR);
        PersonVO personSpouse = testHelper.createKvinde(CPR1);
        testHelper.createSpouse(person, personSpouse);
        testHelper.prepareTestdata(connection);
        Spouse spouse = cprDao.findSpouse(CPR);
        assertEquals(CPR1,spouse.getIdentifier());
        assertEquals(SpouseRelation.AEGTEFAELLE,spouse.getRelation());
    }

    @Test
    public void findSpouseSeperated() throws SQLException {
        PersonVO person = testHelper.createPerson(CPR);
        PersonVO personSpouse = testHelper.createPerson(CPR1);
        testHelper.createSeperated(person, personSpouse);
        testHelper.prepareTestdata(connection);
        Spouse spouse = cprDao.findSpouse(CPR);
        assertEquals(CPR1,spouse.getIdentifier());
        assertNull(spouse.getRelation());
    }

    @Test
    public void findSpouseP() throws SQLException {
        PersonVO person = testHelper.createMand(CPR);
        PersonVO personSpouse = testHelper.createMand(CPR1);
        testHelper.createSpouse(person, personSpouse);
        testHelper.prepareTestdata(connection);
        Spouse spouse = cprDao.findSpouse(CPR);
        assertEquals(CPR1,spouse.getIdentifier());
        assertEquals(SpouseRelation.REGISTRET_PARTNER,spouse.getRelation());
    }

    @Test
    public void findWardMor() throws Exception {
        PersonVO person = testHelper.createKvinde(CPR);
        PersonVO barn1 = testHelper.createPerson(CPR1);
        PersonVO barn2 = testHelper.createPerson(CPR2);
        PersonVO barn3 = testHelper.createPerson(CPR3);
        PersonVO barn4 = testHelper.createPerson(CPR4);
        PersonVO personMedmor = testHelper.createKvinde(CPR5);
        testHelper.createForaeldreMyndighedRelation(barn1, FMRTypeKode.MOR);
        testHelper.createForaeldreMyndighedRelation(barn1, FMRTypeKode.FAR);
        testHelper.createForaeldreMyndighedRelation(barn2, FMRTypeKode.FAR);
        testHelper.createForaeldreMyndighedRelation(barn3, FMRTypeKode.AI1, person);
        testHelper.createBarnRelation(person, barn1);
        testHelper.createBarnRelation(personMedmor, barn1);
        testHelper.createBarnRelation(person, barn2);
        testHelper.createVaerge(barn4, person);
        testHelper.prepareTestdata(connection);

        Collection<Ward> wards = cprDao.findWard(CPR);
        assertEquals("Wards "+Arrays.toString(wards.toArray()),4,wards.size());
        Set<String> foundCpr = Sets.newHashSet();
        Iterator<Ward> iter = wards.iterator();
        while (iter.hasNext()) {
            Ward ward = iter.next();
            foundCpr.add(ward.getIdentifier());
            switch (ward.getRelation()) {
            case BARN:
                assertThat(ward.getIdentifier(),anyOf(is(CPR1),is(CPR3)));
                break;
            case BARN_UDEN:
                assertThat(ward.getIdentifier(),is(CPR2));
                break;
            case MYNDLING:
                assertThat(ward.getIdentifier(),is(CPR4));
                break;
            default:
                fail("Wrong relation: "+ward.toString());
                break;
            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),4,foundCpr.size());
    }

    @Test
    public void findWardFar() throws Exception {
        PersonVO person = testHelper.createMand(CPR);
        PersonVO barn1 = testHelper.createPerson(CPR1);
        PersonVO barn2 = testHelper.createPerson(CPR2);
        PersonVO barn3 = testHelper.createPerson(CPR3);
        PersonVO barn4 = testHelper.createPerson(CPR4);
        testHelper.createForaeldreMyndighedRelation(barn1, FMRTypeKode.MOR);
        testHelper.createForaeldreMyndighedRelation(barn1, FMRTypeKode.FAR);
        testHelper.createForaeldreMyndighedRelation(barn2, FMRTypeKode.FAR);
        testHelper.createForaeldreMyndighedRelation(barn3, FMRTypeKode.AI1, person);
        testHelper.createBarnRelation(person, barn1);
        testHelper.createBarnRelation(person, barn2);
        testHelper.createVaerge(barn4, person);
        testHelper.prepareTestdata(connection);

        Collection<Ward> wards = cprDao.findWard(CPR);
        assertEquals("Wards "+Arrays.toString(wards.toArray()),4,wards.size());
        Set<String> foundCpr = Sets.newHashSet();
        Iterator<Ward> iter = wards.iterator();
        while (iter.hasNext()) {
            Ward ward = iter.next();
            foundCpr.add(ward.getIdentifier());
            switch (ward.getRelation()) {
            case BARN:
                assertThat(ward.getIdentifier(),anyOf(is(CPR1),is(CPR2),is(CPR3)));
                break;
            case MYNDLING:
                assertThat(ward.getIdentifier(),is(CPR4));
                break;
            default:
                fail("Wrong relation: "+ward.toString());
                break;
            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),4,foundCpr.size());
    }

}
