/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.not;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.math.BigInteger;
import java.util.Collections;
import java.util.Date;

import javax.xml.datatype.DatatypeConfigurationException;

import org.junit.Before;
import org.junit.Test;

import com.trifork.stamdata.models.PersonVO;

import dk.nsi.stamdata.cpr.mapping.MunicipalityMapper;
import dk.nsi.stamdata.cpr.mapping.PersonMapper.CPRProtectionLevel;
import dk.nsi.stamdata.cpr.mapping.v100.PersonMapper;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.MockSecureTokenService;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.PersonGenderCodeType;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.PersonInformationStructureType;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.SystemIDCard;


public class PersonMapperFieldMappingForProtectedPersonTest {
    private static final String ADRESSEBESKYTTET = "ADRESSEBESKYTTET";
    private PersonVO person;
    private PersonInformationStructureType output;


    @Before
    public void setUp() throws Exception {
        person = Factories.createPersonWithAddressProtection();

        doMap();
    }


    @Test
    public void mapsGaeldendeToCurrentPersonCivilRegistrationIdentifier() throws Exception {
        assertEquals(person.getGaeldendeCPR(), output.getCurrentPersonCivilRegistrationIdentifier());
    }


    @Test
    public void alwaysUses01ForPersonCivilRegistrationCode() {
        assertThat(output.getRegularCPRPerson().getPersonCivilRegistrationStatusStructure().getPersonCivilRegistrationStatusCode(), is(new BigInteger("01")));
    }


    @Test
    public void setsCivilRegistrationStatusStartDateToStartOfEra() throws DatatypeConfigurationException {
        assertEquals(new Date(0), output.getRegularCPRPerson().getPersonCivilRegistrationStatusStructure().getPersonCivilRegistrationStatusStartDate().toGregorianCalendar().getTime());
    }


    @Test
    public void usesAddresseBeskyttetForPersonNameForAddressingName() throws DatatypeConfigurationException {
        assertEquals(ADRESSEBESKYTTET, output.getRegularCPRPerson().getPersonNameForAddressingName());
    }


    @Test
    public void omitsForStreetNameForAddressingName() throws DatatypeConfigurationException {
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetNameForAddressingName());
    }


    @Test
    public void usesAddresseBeskyttetForGivenName() throws Exception {
        assertEquals(ADRESSEBESKYTTET, output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName());
    }


    @Test
    public void omitsMiddleName() throws Exception {
        assertNull(output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonMiddleName());
    }


    @Test
    public void usesAddresseBeskyttetForSurname() throws Exception {
        assertEquals(ADRESSEBESKYTTET, output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonSurnameName());
    }


    @Test
    public void mapsCprToPersonCivilRegistrationIdentifier() throws Exception {
        assertEquals(person.getCpr(), output.getRegularCPRPerson().getSimpleCPRPerson().getPersonCivilRegistrationIdentifier());
    }


    @Test
    public void mapsKoenMToGenderCodeUnknown() throws Exception {
        person.setKoen("M");
        doMap();

        assertEquals(PersonGenderCodeType.UNKNOWN, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsKoenKToGenderCodeFemale() throws Exception {
        person.setKoen("K");
        doMap();

        assertEquals(PersonGenderCodeType.UNKNOWN, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsKoenEmptystringToGenderCodeUnknown() throws Exception {
        person.setKoen("");
        doMap();

        assertEquals(PersonGenderCodeType.UNKNOWN, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsKoenSinglespaceToGenderCodeUnknown() throws Exception {
        person.setKoen(" ");
        doMap();

        assertEquals(PersonGenderCodeType.UNKNOWN, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsKoenSomeSingleletterCodeToGenderCodeUnknown() throws Exception {
        person.setKoen("X");
        doMap();

        assertEquals(PersonGenderCodeType.UNKNOWN, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsFoedselsdatoToBirthdate() throws Exception {
        assertThat(output.getRegularCPRPerson().getPersonBirthDateStructure().getBirthDate().toGregorianCalendar().getTime(), is(not(person.getFoedselsdato())));
    }


    @Test
    public void omitsCareOfName() throws Exception {
        assertNull(output.getPersonAddressStructure().getCareOfName());
    }


    @Test
    public void uses0000ForMunicipalityCode() throws Exception {
        assertEquals("0000", output.getPersonAddressStructure().getAddressComplete().getAddressAccess().getMunicipalityCode());
    }


    @Test
    public void uses0000ForStreetCode() throws Exception {
        assertEquals("0000", output.getPersonAddressStructure().getAddressComplete().getAddressAccess().getStreetCode());
    }


    @Test
    public void uses1ForStreetBuildingIdentifierInAddressAccess() throws Exception {
        assertEquals("1", output.getPersonAddressStructure().getAddressComplete().getAddressAccess().getStreetBuildingIdentifier());
    }


    @Test
    public void uses1ForStreetBuildingIdentifierInAddressPostal() throws Exception {
        assertEquals("1", output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetBuildingIdentifier());
    }


    @Test
    public void omitsMailDeliverySublocationIdentifier() throws Exception {
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getMailDeliverySublocationIdentifier());
    }


    @Test
    public void usesAdressebeskyttetForStreetName() throws Exception {
        assertEquals(ADRESSEBESKYTTET, output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetName());
    }


    @Test
    public void omitsStreetnameForAddressingName() throws Exception {
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetNameForAddressingName());
    }


    @Test
    public void omitsFloorIdentifier() throws Exception {
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getFloorIdentifier());
    }


    @Test
    public void omitsSuiteIdentifier() throws Exception {
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getSuiteIdentifier());
    }


    @Test
    public void usesTrueForBirthdateUncertaintyIndicator() throws Exception {
        assertTrue(output.getRegularCPRPerson().getPersonBirthDateStructure().isBirthDateUncertaintyIndicator());
    }


    @Test
    public void uses0000ForPostCodeIdentifier() throws Exception {
        assertEquals("0000", output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getPostCodeIdentifier());
    }


    @Test
    public void usesAdressebeskyttetForDistrictName() throws Exception {
        assertEquals(ADRESSEBESKYTTET, output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getDistrictName());
    }


    @Test
    public void usesPersonInformationProtectionStartDate() throws Exception {
        assertThat(output.getPersonAddressStructure().getPersonInformationProtectionStartDate().toGregorianCalendar().getTime(), is(person.getNavnebeskyttelsestartdato()));
    }


    @Test
    public void usesTrueForPersonInformationProtectionIndicator() throws Exception {
        assertTrue(output.getRegularCPRPerson().isPersonInformationProtectionIndicator());
    }


    @Test
    public void omitsDistrictSubdivisionIdentifier() throws Exception {
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getDistrictSubdivisionIdentifier());
    }


    @Test
    public void omitsPostOfficeBoxIdentifier() throws Exception {
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getPostOfficeBoxIdentifier());
    }


    @Test
    public void omitsCountryIdentificationCode() {
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getCountryIdentificationCode());
    }


    @Test
    public void omitsCountyCode() {
        assertThat(output.getPersonAddressStructure().getCountyCode(), is("0000"));
    }


    private void doMap() throws Exception {
        SystemIDCard idCard = MockSecureTokenService.createSignedSystemIDCard("12345678", AuthenticationLevel.VOCES_TRUSTED_SYSTEM);
        MunicipalityMapper municipalityMapper = new MunicipalityMapper();

        PersonMapper personMapper = new PersonMapper(new StubWhitelistService(Collections.<String>emptyList()), idCard, municipalityMapper, null);

        output = personMapper.map(person, CPRProtectionLevel.DoNotCensorCPR);
    }
}
