/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import static dk.nsi.stamdata.cpr.mapping.PersonMapper.newXMLGregorianCalendar;
import static dk.nsi.stamdata.testing.Factories.YESTERDAY;
import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;

import java.math.BigInteger;
import java.util.Collections;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import com.trifork.stamdata.models.PersonVO;

import dk.nsi.stamdata.cpr.mapping.CivilRegistrationStatusCodes;
import dk.nsi.stamdata.cpr.mapping.MunicipalityMapper;
import dk.nsi.stamdata.cpr.mapping.v100.PersonMapper;
import dk.nsi.stamdata.testing.Factories;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.PersonGenderCodeType;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.PersonInformationStructureType;
import dk.sosi.seal.model.SystemIDCard;


public class PersonMapperFieldMappingForUnprotectedPersonTest {
    private SystemIDCard idCard;
    private PersonVO person;
    private PersonInformationStructureType output;

    private static final String VIBORG = "0791";
    private static final String REGION_MIDTJYLLAND = "1082";


    @Before
    public void setUp() throws Exception {
        person = Factories.createPerson();
        doMap();
    }


    @Test
    public void mapsUnknownFieldToUkendtString() throws Exception {
        person.setFornavn(null);

        doMap();

        assertThat(output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is("UKENDT"));
        assertThat(output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonSurnameName(), is(person.getEfternavn()));
    }


    @Test
    public void dontOutputAddressesForTheDeadOrPeopleThatAreInOneOfTheOtherNoAddressCatagories() throws Exception {
        for (String statusCode : CivilRegistrationStatusCodes.STATUSES_WITH_NO_ADDRESS) {
            person.setStatus(statusCode);

            doMap();

            assertThat(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetName(), is("UKENDT"));
        }
    }


    @Test
    public void mapsGaeldendeToCprCurrentPersonCivilRegistrationIdentifier() throws Exception {
        assertEquals(person.getGaeldendeCPR(), output.getCurrentPersonCivilRegistrationIdentifier());
    }


    @Test
    public void mapCivilRegistrationCode() throws Exception {
        assertThat(output.getRegularCPRPerson().getPersonCivilRegistrationStatusStructure().getPersonCivilRegistrationStatusCode(), is(new BigInteger("01")));
    }


    @Test
    public void mapCivilRegistrationCodeStartDate() throws Exception {
        assertThat(output.getRegularCPRPerson().getPersonCivilRegistrationStatusStructure().getPersonCivilRegistrationStatusStartDate(), is(newXMLGregorianCalendar(YESTERDAY)));
    }


    @Test
    public void mapNameForAddressingName() throws Exception {
        assertThat(output.getRegularCPRPerson().getPersonNameForAddressingName(), is("Peter,Andersen"));
    }


    @Test
    public void alwaysUses02ForPersonCivilRegistrationStatusCode() {
        assertThat(output.getRegularCPRPerson().getPersonCivilRegistrationStatusStructure().getPersonCivilRegistrationStatusCode(), is(new BigInteger("01")));
    }


    @Test
    public void mapStreetNameForAddressingName() throws Exception {
        assertThat(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetNameForAddressingName(), is("Østergd."));
    }


    @Test
    public void mapsFornavnToGivenName() throws Exception {
        assertEquals(person.getFornavn(), output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName());
    }


    @Test
    public void mapsMellemnavnToMiddleName() throws Exception {
        assertEquals(person.getMellemnavn(), output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonMiddleName());
    }


    @Test
    public void mapsEfternavnToSurname() throws Exception {
        assertEquals(person.getEfternavn(), output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonSurnameName());
    }


    @Test
    public void mapsCprToPersonCivilRegistrationIdentifier() throws Exception {
        assertEquals(person.getCpr(), output.getRegularCPRPerson().getSimpleCPRPerson().getPersonCivilRegistrationIdentifier());
    }


    @Test
    public void mapsKoenMToGenderCodeMale() throws Exception {
        person.setKoen("M");
        doMap();

        assertEquals(PersonGenderCodeType.MALE, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsKoenKToGenderCodeFemale() throws Exception {
        person.setKoen("K");
        doMap();

        assertEquals(PersonGenderCodeType.FEMALE, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsKoenEmptystringToGenderCodeUnknown() throws Exception {
        person.setKoen("");
        doMap();

        assertEquals(PersonGenderCodeType.UNKNOWN, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsKoenSinglespaceToGenderCodeUnknown() throws Exception {
        person.setKoen(" ");
        doMap();

        assertEquals(PersonGenderCodeType.UNKNOWN, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsKoenSomeSingleletterCodeToGenderCodeUnknown() throws Exception {
        person.setKoen("X");
        doMap();

        assertEquals(PersonGenderCodeType.UNKNOWN, output.getRegularCPRPerson().getPersonGenderCode());
    }


    @Test
    public void mapsFoedselsdatoToBirthdate() throws Exception {
        assertEquals(person.getFoedselsdato(), output.getRegularCPRPerson().getPersonBirthDateStructure().getBirthDate().toGregorianCalendar().getTime());
    }


    @Test
    public void mapsCoNavnToCareOfName() throws Exception {
        assertEquals(person.getCoNavn(), output.getPersonAddressStructure().getCareOfName());
    }


    @Test
    public void mapsKommuneKodeToMunicipalityCode() throws Exception {
        assertEquals(person.getKommuneKode(), output.getPersonAddressStructure().getAddressComplete().getAddressAccess().getMunicipalityCode());
    }


    @Test
    public void mapsVejkodeToStreetCode() throws Exception {
        assertEquals(person.getVejKode(), output.getPersonAddressStructure().getAddressComplete().getAddressAccess().getStreetCode());
    }


    @Test
    public void mapsHusnummerAndBygningsnummerToStreetBuildingIdentifierInAddressAccess() throws Exception {
        assertEquals(person.getHusnummer(), output.getPersonAddressStructure().getAddressComplete().getAddressAccess().getStreetBuildingIdentifier());
    }


    @Test
    public void mapsHusnummerAndBygningsnummerToStreetBuildingIdentifierInAddressPostal() throws Exception {
        assertEquals(person.getHusnummer(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetBuildingIdentifier());
    }


    @Test
    @Ignore("Is anything mapped to Maildelivery Subdivision identifier? Lokalitet is not, it's mapped to DistrictSubdivisionIdentifier")
    public void mapsLokalitetToMailDeliverySublocationIdentifier() throws Exception {

        // From the xsd's:
        //
        // DistrictSubdivisionIdentifier:
        // Name of a village, city or subdivision of a city or district, which
        // is determined as a part of the
        // official address specification for a certain street or specific parts
        // of a street, defined by intervals
        // of street building identifiers (da: house numbers).
        //
        // MailDeliverySublocationIdentifier:
        // The given name of a farm, estate, building or dwelling, which is used
        // as a additional postal address
        // identifier.

        assertEquals(person.getLokalitet(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getMailDeliverySublocationIdentifier());

    }


    @Test
    public void mapsVejnavnToStreetName() throws Exception {
        assertEquals(person.getVejnavn(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetName());
    }


    @Test
    public void mapsVejnavnTilAddresseringToStreetnameForAddressingName() throws Exception {
        assertEquals(person.getVejnavnTilAdressering(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetNameForAddressingName());
    }


    @Test
    public void mapsEtageToFloorIdentifier() throws Exception {
        assertEquals(person.getEtage(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getFloorIdentifier());
    }


    @Test
    public void mapsSideDoerNummerToSuiteIdentifier() throws Exception {
        assertEquals(person.getSideDoerNummer(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getSuiteIdentifier());
    }


    @Test
    public void mapsFoedselsdatoMarkeringToBirthdateUncertaintyIndicator() throws Exception {
        assertEquals(person.getFoedselsdatoMarkering().equals("*"), output.getRegularCPRPerson().getPersonBirthDateStructure().isBirthDateUncertaintyIndicator());
    }


    @Test
    public void mapsStatusToPersonCivilRegistrationStatusCode() throws Exception {
        assertEquals(new BigInteger(person.getStatus()), output.getRegularCPRPerson().getPersonCivilRegistrationStatusStructure().getPersonCivilRegistrationStatusCode());
    }


    @Test
    public void mapsStatusDatoToPersonCivilRegistrationStatusStartDate() throws Exception {
        assertEquals(person.getStatusDato(), output.getRegularCPRPerson().getPersonCivilRegistrationStatusStructure().getPersonCivilRegistrationStatusStartDate().toGregorianCalendar().getTime());
    }


    @Test
    public void mapsPostnummerToPostCodeIdentifier() throws Exception {
        assertEquals(person.getPostnummer(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getPostCodeIdentifier());
    }


    @Test
    public void mapsPostdistriktToDistrictName() throws Exception {
        assertEquals(person.getPostdistrikt(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getDistrictName());
    }


    @Test
    public void omitsNavnebeskyttelsestartdatoToPersonInformationProtectionStartDate() throws Exception {
        assertNull(output.getPersonAddressStructure().getPersonInformationProtectionStartDate());
    }


    @Test
    public void usesFalseForPersonInformationProtectionIndicator() throws Exception {
        assertFalse(output.getRegularCPRPerson().isPersonInformationProtectionIndicator());
    }


    @Test
    public void mapsLokalitetToDistrictSubdivisionIdentifier() throws Exception {
        assertEquals(person.getBynavn(), output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getDistrictSubdivisionIdentifier());
    }


    @Test
    public void nothingIsMappedToPostOfficeBoxIdentifier() throws Exception {
        // Just to document the fact
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getPostOfficeBoxIdentifier());
    }

    @Test
    public void countryDKIsMapped() {
        assertEquals(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().
                getCountryIdentificationCode().getValue(), "DK");
    }

    @Test
    public void countryGLIsMapped() throws Exception {
        person = Factories.createPerson();
        person.setStatus("05");
        doMap();
        assertEquals(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().
                getCountryIdentificationCode().getValue(), "GL");
    }

    @Test
    public void countryNotMappedWhenNotDKorGL() throws Exception {
        person = Factories.createPerson();
        person.setStatus("20");
        doMap();
        assertNull(output.getPersonAddressStructure().getAddressComplete().getAddressPostal().getCountryIdentificationCode());
    }

    @Test
    public void theMunicipalityIsMappedToTheCorrectCounty() throws Exception {
        person.setKommuneKode(VIBORG);

        doMap();

        assertThat(output.getPersonAddressStructure().getCountyCode(), is(REGION_MIDTJYLLAND));
    }


    private void doMap() throws Exception {

        MunicipalityMapper municipalityMapper = new MunicipalityMapper();
        PersonMapper personMapper = new PersonMapper(new StubWhitelistService(Collections.<String>emptyList()), idCard, municipalityMapper, null);

        output = personMapper.map(person, dk.nsi.stamdata.cpr.mapping.PersonMapper.CPRProtectionLevel.DoNotCensorCPR);
    }
}
