/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertThat;

import java.util.Collections;

import javax.xml.datatype.DatatypeConfigurationException;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;

import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.YderVO;

import dk.nsi.stamdata.cpr.mapping.MunicipalityMapper;
import dk.nsi.stamdata.cpr.mapping.PersonMapper.CPRProtectionLevel;
import dk.nsi.stamdata.cpr.mapping.v100.PersonMapper;
import dk.nsi.stamdata.cpr.models.HealthCareInformation;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.MockSecureTokenService;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.PersonInformationStructureType;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.PersonWithHealthCareInformationStructureType;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.SystemIDCard;

@RunWith(MockitoJUnitRunner.class)
public class PersonMapperProtectionTest {
    private final boolean FOR_AUTORITY_CLIENT = true;
    private final boolean FOR_NON_AUTORITY_CLIENT = false;

    private PersonVO person;

    private final static String WHITELISTED_CVR = "12345678";
    private SystemIDCard whitelistedIDCard;

    private static final String NON_WHITELISTED_CVR = "23456789";
    private SystemIDCard nonWhitelistedIDCard;

    private static final String CENSORED = "ADRESSEBESKYTTET";

    private WhitelistService whitelist;
    private MunicipalityMapper municipalityMapper;
    private YderVO yder;
    private HealthCareInformation healthCareGroup;

    @Before
    public void setUp() {
        whitelist = new StubWhitelistService(Collections.singletonList(WHITELISTED_CVR));
        nonWhitelistedIDCard = MockSecureTokenService.createSignedSystemIDCard(NON_WHITELISTED_CVR, AuthenticationLevel.VOCES_TRUSTED_SYSTEM);
        whitelistedIDCard = MockSecureTokenService.createSignedSystemIDCard(WHITELISTED_CVR, AuthenticationLevel.VOCES_TRUSTED_SYSTEM);

        municipalityMapper = new MunicipalityMapper();
        person = Factories.createPerson();
        yder = Factories.createYder("1234");
        healthCareGroup = new HealthCareInformation(new DateTime(Factories.YESTERDAY).toDate(), "2", "1234");
    }

    public PersonMapper mapper(boolean isClientAnAuthority) {
        return new PersonMapper(whitelist, isClientAnAuthority ? whitelistedIDCard : nonWhitelistedIDCard, municipalityMapper, null);
    }

    @Test
    public void shouldNotProtectPersonWithNoActiveProtection() throws DatatypeConfigurationException {
        person.setNavnebeskyttelsestartdato(null);
        person.setNavnebeskyttelseslettedato(null);

        assertThatPersonIsNotProtected(mapper(FOR_AUTORITY_CLIENT).map(person,  CPRProtectionLevel.DoNotCensorCPR));
        assertThatPersonIsNotProtected(mapper(FOR_NON_AUTORITY_CLIENT).map(person, CPRProtectionLevel.DoNotCensorCPR));
    }

    @Test
    public void shouldProtectPersonWithActiveProtectionIfClientIsNotAuthority() throws DatatypeConfigurationException {
        person.setNavnebeskyttelsestartdato(Factories.YESTERDAY);
        person.setNavnebeskyttelseslettedato(Factories.TOMORROW);

        assertThatPersonIsProtected(mapper(FOR_NON_AUTORITY_CLIENT).map(person, CPRProtectionLevel.DoNotCensorCPR));
    }

    @Test
    public void shouldProtectPersonWithActiveProtectionIfClientIsAuthority() throws DatatypeConfigurationException {
        person.setNavnebeskyttelsestartdato(Factories.YESTERDAY);
        person.setNavnebeskyttelseslettedato(Factories.TOMORROW);

        assertThatPersonIsProtected(mapper(FOR_AUTORITY_CLIENT).map(person, CPRProtectionLevel.DoNotCensorCPR));
    }

    @Test
    public void shouldAlwaysProtectPersonWithActiveProtectionIfAuthoritiesHaveNoSpecialRights() throws DatatypeConfigurationException {
        person.setNavnebeskyttelsestartdato(Factories.YESTERDAY);
        person.setNavnebeskyttelseslettedato(Factories.TOMORROW);

        assertThatPersonIsProtected(mapper(FOR_AUTORITY_CLIENT).map(person, CPRProtectionLevel.DoNotCensorCPR));
        assertThatPersonIsProtected(mapper(FOR_NON_AUTORITY_CLIENT).map(person, CPRProtectionLevel.DoNotCensorCPR));
    }

    @Test
    public void shouldNotProtectPersonIfProtectionHasNotStartedYet() throws DatatypeConfigurationException {
        person.setNavnebeskyttelsestartdato(Factories.TOMORROW);
        person.setNavnebeskyttelseslettedato(Factories.IN_TWO_DAYS);

        assertThatPersonIsNotProtected(mapper(FOR_NON_AUTORITY_CLIENT).map(person, CPRProtectionLevel.DoNotCensorCPR));
    }

    @Test
    public void shouldNotProtectPersonIfProtectionHasEnded() throws DatatypeConfigurationException {
        person.setNavnebeskyttelsestartdato(Factories.TWO_DAYS_AGO);
        person.setNavnebeskyttelseslettedato(Factories.YESTERDAY);

        assertThatPersonIsNotProtected(mapper(FOR_NON_AUTORITY_CLIENT).map(person, CPRProtectionLevel.DoNotCensorCPR));
    }

    @Test
    public void shouldNotProtectPersonHealthCareInfoIfProtectionIsActive() throws DatatypeConfigurationException {
        person.setNavnebeskyttelsestartdato(Factories.YESTERDAY);
        person.setNavnebeskyttelseslettedato(Factories.TOMORROW);
        PersonWithHealthCareInformationStructureType output = mapper(FOR_NON_AUTORITY_CLIENT).map(person, healthCareGroup, yder);
        assertThatHealthCareInfoIsNotProtected(mapper(FOR_NON_AUTORITY_CLIENT).map(person, healthCareGroup, yder));
    }

    @Test
    public void shouldNotProtectPersonHealthCareInfoIfProtectionIsNotActive() throws DatatypeConfigurationException {
        person.setNavnebeskyttelsestartdato(Factories.TWO_DAYS_AGO);
        person.setNavnebeskyttelseslettedato(Factories.YESTERDAY);

        assertThatHealthCareInfoIsNotProtected(mapper(FOR_NON_AUTORITY_CLIENT).map(person, healthCareGroup, yder));
    }

    private void assertThatPersonIsNotProtected(PersonInformationStructureType output) {
        assertThat(output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is(person.getFornavn()));
    }

    private void assertThatPersonIsProtected(PersonInformationStructureType output) {
        assertThat(output.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is(CENSORED));
    }

    private void assertThatHealthCareInfoIsNotProtected(PersonWithHealthCareInformationStructureType output) {
        assertThat(output.getPersonHealthCareInformationStructure().getAssociatedGeneralPractitionerStructure().getDistrictName(), is(yder.getPostdistYder()));
    }

}
