/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import java.sql.Connection;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.Date;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.inject.Inject;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.SikredeVO;
import com.trifork.stamdata.models.YderVO;

import dk.nsi.stamdata.cpr.dao.SikredeDAO;
import dk.nsi.stamdata.cpr.models.HealthCareInformation;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;

@RunWith(GuiceTestRunner.class)
public class SikredeDaoTest {
    private static final DateTime YESTERDAY = new DateTime(Factories.YESTERDAY);
    private static final Date CURRENT_TIME = new Date();
    private static final String CPR = "1234567890";
    @Inject
    private Connection connection;
    @Inject
    private SikredeDAO sikredeDao;

    StamdataTestHelper testHelper = new StamdataTestHelper();

    @Before
    public void setUp() throws Exception {
        testHelper.clear();
    }

    @Test
    public void testFind() throws SQLException {
        PersonVO person = testHelper.createPerson(CPR);
        YderVO yder = testHelper.createYder("1234");
        testHelper.createSikrede(person, yder, "1", YESTERDAY);
        testHelper.prepareTestdata(connection);
        HealthCareInformation healthCareInfomation = sikredeDao.findHealthCareInformation(CPR, CURRENT_TIME);
        assertEquals("1",healthCareInfomation.getGroup());
        assertEquals("1234",healthCareInfomation.getProviderNo());
        assertNull(healthCareInfomation.getSBSStatsborgerskab());
        assertNull(healthCareInfomation.getSBSStatsborgerskabKode());
        assertNull(healthCareInfomation.getSSKBopelsLand());
        assertNull(healthCareInfomation.getSSKBopelsLAndKode());
        assertNull(healthCareInfomation.getSSKGyldigFra());
        assertNull(healthCareInfomation.getSSKGyldigTil());
        assertNull(healthCareInfomation.getSSLForsikringsinstans());
        assertNull(healthCareInfomation.getSSLForsikringsinstansKode());
        assertNull(healthCareInfomation.getSSLForsikringsnr());
        assertNull(healthCareInfomation.getSSLSocSikretLand());
        assertNull(healthCareInfomation.getSSLSocSikretLandKode());
        assertNull(healthCareInfomation.getSStatus());
        StamdataTestHelper.assertDates(YESTERDAY,healthCareInfomation.getStartDate());
        assertNull(healthCareInfomation.getSYdernrGl());
    }

    @Test
    public void testFindNothing() throws SQLException {
        PersonVO person = testHelper.createPerson(CPR);
        YderVO yder = testHelper.createYder("1234");
        testHelper.createSikrede(person, yder, "1", YESTERDAY);
        testHelper.prepareTestdata(connection);
        HealthCareInformation healthCareInfomation = sikredeDao.findHealthCareInformation(CPR, YESTERDAY.toDate());
        assertNull(healthCareInfomation);
    }

    @Test
    public void testFindWithFrontierInfomation() throws SQLException, ParseException {
        PersonVO person = testHelper.createPerson(CPR);
        YderVO yder = testHelper.createYder("1234");
        SikredeVO sikrede = testHelper.createSikredeWithFrontierInfo(person, yder, "1", YESTERDAY, YESTERDAY);
        sikrede.setSYdernrGl("4321");
        testHelper.prepareTestdata(connection);
        HealthCareInformation healthCareInfomation = sikredeDao.findHealthCareInformationWithFrontierInformation(CPR, CURRENT_TIME);
        assertEquals("1",healthCareInfomation.getGroup());
        assertEquals("1234",healthCareInfomation.getProviderNo());
        assertEquals("England",healthCareInfomation.getSBSStatsborgerskab());
        assertEquals("GB",healthCareInfomation.getSBSStatsborgerskabKode());
        assertEquals("Tyskland",healthCareInfomation.getSSKBopelsLand());
        assertEquals("DE",healthCareInfomation.getSSKBopelsLAndKode());
        StamdataTestHelper.assertDates(YESTERDAY,healthCareInfomation.getSSKGyldigFra());
        assertEquals(Factories.YESTERDAY_PLUS_2_YEARS,healthCareInfomation.getSSKGyldigTil());
        assertEquals("FOO sikring",healthCareInfomation.getSSLForsikringsinstans());
        assertEquals("BAR",healthCareInfomation.getSSLForsikringsinstansKode());
        assertEquals("12345",healthCareInfomation.getSSLForsikringsnr());
        assertEquals("Danmark",healthCareInfomation.getSSLSocSikretLand());
        assertEquals("DK",healthCareInfomation.getSSLSocSikretLandKode());
        assertEquals("D",healthCareInfomation.getSStatus());
        StamdataTestHelper.assertDates(YESTERDAY,healthCareInfomation.getStartDate());
        assertEquals("4321",healthCareInfomation.getSYdernrGl());

    }
}
