/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;

import dk.nsi._2011._09._23.stamdatacpr.NamePersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType;
import dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType;
import dk.nsi._2018._06._08.stamdatacpr.StamdataPersonLookup;
import dk.nsi._2018._06._08.stamdatacpr.StamdataPersonLookupService;

import org.apache.jmeter.config.Arguments;
import org.apache.jmeter.protocol.java.sampler.AbstractJavaSamplerClient;
import org.apache.jmeter.protocol.java.sampler.JavaSamplerContext;
import org.apache.jmeter.samplers.SampleResult;

import com.trifork.stamdata.jaxws.SealNamespaceResolver;

import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;


/**
 * Performance Test used by JMeter.
 */
public class StamdataPersonLookupSampler extends AbstractJavaSamplerClient
{
    private static final String REQUESTED_GIVENNAME_PARAM = "RequestedGivenName";
    private static final String REQUESTED_SURNAME_PARAM = "RequestedSurname";
    private static final String ENDPOINT_URL_PARAM = "EndpointURL";
    private static final String CLIENT_CVR_PARAM = "ClientCVR";


    @Override
    public Arguments getDefaultParameters()
    {
        Arguments args = new Arguments();

        args.addArgument(ENDPOINT_URL_PARAM, "http://localhost:80/stamdata-cpr-ws/service/StamdataPersonLookup-1.1.0");
        args.addArgument(CLIENT_CVR_PARAM, "12345678");
        args.addArgument(REQUESTED_GIVENNAME_PARAM, "Thomas");
        args.addArgument(REQUESTED_SURNAME_PARAM, "Kristensen");

        return args;
    }


    @Override
    public SampleResult runTest(JavaSamplerContext context)
    {
        String endpointURL = context.getParameter(ENDPOINT_URL_PARAM);
        String clientCVR = context.getParameter(CLIENT_CVR_PARAM);
        String requestedGivenname = context.getParameter(REQUESTED_GIVENNAME_PARAM);
        String requestedSurname = context.getParameter(REQUESTED_SURNAME_PARAM);

        SampleResult result = new SampleResult();

        try
        {
            StamdataPersonLookup client = createClient(endpointURL);

            SecurityWrapper headers = createHeaders(clientCVR);
            PersonLookupRequestType query = new PersonLookupRequestType();

            NamePersonQueryType namePersonQuery = new NamePersonQueryType();
            namePersonQuery.setPersonGivenName(requestedGivenname);
            namePersonQuery.setPersonSurnameName(requestedSurname);
            query.setNamePersonQuery(namePersonQuery);

            // Wait until the last minute before starting the
            // timer.
            result.sampleStart();
            PersonLookupResponseType responseType = client.getPersonDetails(headers.getSecurity(), headers.getMedcomHeader(), query);
            result.sampleEnd();

            int numberOfReturnedPersons = responseType.getPersonInformationStructure().size();
            if (numberOfReturnedPersons == 0)
            {
                result.setSuccessful(false);
                result.setResponseMessage("Expected at least 1 person in result, but found none");
            }
            else
            {
                result.setResponseMessage("StamdataPersonLookup getPersonDetails request for " + requestedGivenname + " " + requestedSurname + ", " + numberOfReturnedPersons + " persons in response");
                result.setResponseOK();
            }
        }
        catch (Exception e)
        {
            result.sampleEnd();
            e.printStackTrace();
            result.setSuccessful(false);
        }

        return result;
    }


    // used by jmeter scripts
    @SuppressWarnings("unused")
    private String firstCprFromResponse(PersonLookupResponseType responseType)
    {
        return responseType.getPersonInformationStructure().get(0).getRegularCPRPerson().getSimpleCPRPerson().getPersonCivilRegistrationIdentifier();
    }


    private StamdataPersonLookup createClient(String endpointURL) throws MalformedURLException
    {
        QName PVIT_SERVICE_QNAME = new QName("http://nsi.dk/2018/06/08/StamdataCpr/", "StamdataPersonLookupService");

        URL wsdlLocation = new URL(endpointURL + "?wsdl");
        StamdataPersonLookupService serviceCatalog = new StamdataPersonLookupService(wsdlLocation, PVIT_SERVICE_QNAME);

        // SEAL enforces that the XML prefixes are exactly
        // as it creates them. So we have to make sure we
        // don't change them.

        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());
        StamdataPersonLookup client = serviceCatalog.getStamdataPersonLookup();

        return client;
    }


    private SecurityWrapper createHeaders(String clientCVR) throws Exception
    {
        return DGWSHeaderUtil.getVocesTrustedSecurityWrapper(clientCVR, "foo", "bar");
    }
}
