/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.integrationtest;

import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.Properties;

import javax.xml.namespace.QName;

import org.hisrc.hifaces20.testing.webappenvironment.testing.junit4.AbstractWebAppEnvironmentJUnit4Test;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO.FMRTypeKode;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.SikredeVO;
import com.trifork.stamdata.models.YderVO;
import com.trifork.stamdata.persistence.PersistenceModule;

import dk.nsi.stamdata.cpr.medcom.AssertResponse104;
import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import dk.nsi.stamdata.testing.TestHelper;
import oio.medcom.cprservice._1_0_4.DetGodeCPROpslag;
import oio.medcom.cprservice._1_0_4.DetGodeCPROpslagService;
import oio.medcom.cprservice._1_0_4.GetPersonInformationIn;
import oio.medcom.cprservice._1_0_4.GetPersonInformationOut;
import oio.medcom.cprservice._1_0_4.GetPersonWithHealthCareInformationIn;
import oio.medcom.cprservice._1_0_4.GetPersonWithHealthCareInformationOut;
import oio.medcom.cprservice._1_0_4.PersonWithHealthCareInformationStructureType;

@RunWith(GuiceTestRunner.class)
public class DetGodeCPROpslag104IT extends AbstractWebAppEnvironmentJUnit4Test {

    public static final QName DET_GODE_CPR_OPSLAG_104_SERVICE = new QName("urn:oio:medcom:cprservice:1.0.4", "DetGodeCPROpslagService");
    public static final String WHITELISTED_CVR = "12345678";

    @Inject
    private Provider<Connection> connectionProvider;

    private DetGodeCPROpslag client;

    private GetPersonInformationIn request = new GetPersonInformationIn();
    private GetPersonInformationOut response;
    private GetPersonWithHealthCareInformationOut healthCareResponse;

    private GetPersonWithHealthCareInformationIn healthCareRequest = new GetPersonWithHealthCareInformationIn();

    private StamdataTestHelper testHelper = new StamdataTestHelper();

    TestHelper helper = new TestHelper();

    @Inject
    Connection connection;

    @Before
    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    @Before
    public void setUp() throws Exception {
        Properties properties = TestHelper.loadProperties(this.getClass());
        String appserverUrl = properties.getProperty("integrationtest.appserver.url");
        URL wsdlLocation = new URL(appserverUrl+"/DetGodeCPROpslag-1.0.4?wsdl");
        DetGodeCPROpslagService serviceCatalog =
                new DetGodeCPROpslagService(wsdlLocation, DET_GODE_CPR_OPSLAG_104_SERVICE);
        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());
        client = serviceCatalog.getDetGodeCPROpslag();
    }

    private void sendPersonRequest(String cpr) throws Exception {
        request.setPersonCivilRegistrationIdentifier(cpr);
        performRequest();
    }

    private void performRequest() throws Exception {
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        response = client.getPersonInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), request);
    }

    private void sendHealthCareRequest(String cpr) throws Exception {
        healthCareRequest.setPersonCivilRegistrationIdentifier(cpr);
        sendHealthCareRequest();
    }

    private void sendHealthCareRequest() throws Exception {
        testHelper.prepareTestdata(connection);
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        healthCareResponse = client.getPersonWithHealthCareInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), healthCareRequest);
    }

    @Test
    public void requestForPersonInformation() throws Exception {
        PersonVO person = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person);
        testHelper.createVaerge(personBarn4, person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());

        AssertResponse104.assertPersonInformationStructure(response.getPersonInformationStructure());
    }

    @Test
    public void requestForPersonWithHealthCareInformation() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person1, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person1);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person1);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person1, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person1, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person1);
        testHelper.createVaerge(personBarn4, person1);
        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        AssertResponse104.assertPersonWithHealthCareInformationStructure(personWithHealthCareInformationStructure);
    }
}
