/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.mapping;

import static org.junit.Assert.assertEquals;

import dk.nsi.stamdata.cpr.dao.SikredeDAO;
import dk.nsi.stamdata.cpr.mapping.v100.HealthCareGroupToPersonPublicHealthInsuranceMapper;
import dk.nsi.stamdata.cpr.models.HealthCareInformation;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.PersonPublicHealthInsuranceType;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.PublicHealthInsuranceGroupIdentifierType;
import org.junit.Test;

public class SikredeRecordToPersonPublicHealthInsuranceTest {

    @Test
    public void testSimpleMappingContainingAllFieldsInCategory1() {
        testSimpleMappingContainingAllFieldsHelper("1", PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_1, "20110628", 2011, 6, 28);
    }

    @Test
    public void testSimpleMappingContainingAllFieldsInCategory2() {
        testSimpleMappingContainingAllFieldsHelper("2", PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_2, "20110628", 2011, 6, 28);
    }

    @Test(expected = IllegalStateException.class)
    public void testSimpleMappingWithIlleagalSygesikringsgruppe() {
        testSimpleMappingContainingAllFieldsHelper("3", null, "20110628", 0, 0, 0);
    }

    @Test(expected = NumberFormatException.class)
    public void testSimpleMappingWithIlleagalSygesikringsGruppeStartDato() {
        testSimpleMappingContainingAllFieldsHelper("1", null, "A0110628", 0, 0, 0);
    }

    @Test
    public void testSimpleMappingContainingAllFieldsInCategory2_v102() {
        testSimpleMappingContainingAllFieldsHelperVersion102("2",
                oio.medcom.cprservice._1_0.PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_2, "20110628", 2011, 6, 28);
    }

    @Test
    public void testSimpleMappingContainingAllFieldsInCategory9_v102() {
        testSimpleMappingContainingAllFieldsHelperVersion102("9",
                oio.medcom.cprservice._1_0.PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_9, "20110628", 2011, 6, 28);
    }

    @Test(expected = IllegalStateException.class)
    public void testSimpleMappingWithIlleagalSygesikringsgruppe_v102() {
        testSimpleMappingContainingAllFieldsHelperVersion102("3", null, "20110628", 0, 0, 0);
    }

    @Test(expected = NumberFormatException.class)
    public void testSimpleMappingWithIlleagalSygesikringsGruppeStartDato_v102() {
        testSimpleMappingContainingAllFieldsHelperVersion102("1", null, "A0110628", 0, 0, 0);
    }

    private void testSimpleMappingContainingAllFieldsHelperVersion102(
            String sygesikringsGruppeKategori, oio.medcom.cprservice._1_0.PublicHealthInsuranceGroupIdentifierType expectedCategory,
            String sygesikringsGruppeStartDato, int expectedYear, int expectedMonth, int expectedDay) {
        // These record names are taken from the document "NSI - NOTUS Sikrede -def.pdf"

        HealthCareInformation healthCareGroup = new HealthCareInformation(
                SikredeDAO.dateFromSIKraftDatoGrp(sygesikringsGruppeStartDato), sygesikringsGruppeKategori, null);

        dk.nsi.stamdata.cpr.mapping.v102.HealthCareGroupToPersonPublicHealthInsuranceMapper mapper =
                new dk.nsi.stamdata.cpr.mapping.v102.HealthCareGroupToPersonPublicHealthInsuranceMapper();
        oio.medcom.cprservice._1_0.PersonPublicHealthInsuranceType xmlStructurer = mapper.map(healthCareGroup);

        assertEquals(expectedCategory, xmlStructurer.getPublicHealthInsuranceGroupIdentifier());
        assertEquals(expectedYear, xmlStructurer.getPublicHealthInsuranceGroupStartDate().getYear());
        assertEquals(expectedMonth, xmlStructurer.getPublicHealthInsuranceGroupStartDate().getMonth());
        assertEquals(expectedDay, xmlStructurer.getPublicHealthInsuranceGroupStartDate().getDay());
    }

    private void testSimpleMappingContainingAllFieldsHelper(
            String sygesikringsGruppeKategori, PublicHealthInsuranceGroupIdentifierType expectedCategory,
            String sygesikringsGruppeStartDato, int expectedYear, int expectedMonth, int expectedDay) {

        HealthCareInformation healthCareGroup = new HealthCareInformation(
                SikredeDAO.dateFromSIKraftDatoGrp(sygesikringsGruppeStartDato), sygesikringsGruppeKategori, null);

        HealthCareGroupToPersonPublicHealthInsuranceMapper mapper = new HealthCareGroupToPersonPublicHealthInsuranceMapper();
        PersonPublicHealthInsuranceType xmlStructurer = mapper.map(healthCareGroup);

        assertEquals(expectedCategory, xmlStructurer.getPublicHealthInsuranceGroupIdentifier());
        assertEquals(expectedYear, xmlStructurer.getPublicHealthInsuranceGroupStartDate().getYear());
        assertEquals(expectedMonth, xmlStructurer.getPublicHealthInsuranceGroupStartDate().getMonth());
        assertEquals(expectedDay, xmlStructurer.getPublicHealthInsuranceGroupStartDate().getDay());
    }
}
