/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.medcom;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.xml.datatype.XMLGregorianCalendar;

import com.google.common.collect.Sets;

import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.CountryIdentificationCodeType;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.CountryIdentificationSchemeType;
import dk.oio.rep.ebxml.xml.schemas.dkcc._2003._02._13.PersonGenderCodeType;
import dk.oio.rep.itst_dk.xml.schemas._2006._01._17.PersonNameStructureType;
import dk.oio.rep.medcom_sundcom_dk.xml.schemas._2007._02._01.AssociatedGeneralPractitionerStructureType;
import oio.medcom.cprservice._1_0_4.AddressCompleteType;
import oio.medcom.cprservice._1_0_4.AddressPostalType;
import oio.medcom.cprservice._1_0_4.CurrentGeneralPractitionerStructureType;
import oio.medcom.cprservice._1_0_4.FrontierInformationStructureType;
import oio.medcom.cprservice._1_0_4.PersonAddressStructureType;
import oio.medcom.cprservice._1_0_4.PersonCivilRegistrationStatusStructureType;
import oio.medcom.cprservice._1_0_4.PersonCustodyRelationType;
import oio.medcom.cprservice._1_0_4.PersonCustodyStructureType;
import oio.medcom.cprservice._1_0_4.PersonHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4.PersonInformationStructureType;
import oio.medcom.cprservice._1_0_4.PersonPublicHealthInsuranceType;
import oio.medcom.cprservice._1_0_4.PersonSpouseRelationType;
import oio.medcom.cprservice._1_0_4.PersonSpouseStructureType;
import oio.medcom.cprservice._1_0_4.PersonWardRelationType;
import oio.medcom.cprservice._1_0_4.PersonWardStructureType;
import oio.medcom.cprservice._1_0_4.PersonWithHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4.PriorGeneralPractitionerStructureType;
import oio.medcom.cprservice._1_0_4.PublicHealthInsuranceGroupIdentifierType;
import oio.medcom.cprservice._1_0_4.PublicHealthInsuranceStatusType;
import oio.medcom.cprservice._1_0_4.RegularCPRPersonType;
import oio.medcom.cprservice._1_0_4.SimpleCPRPersonType;

public class AssertResponse104 {

    private static final String ZERO = "0000";
    private static final String ADRESSEBESKYTTET = "ADRESSEBESKYTTET";

    public static void assertPersonInformationStructure(PersonInformationStructureType personInformationStructure) {
        assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson());
        assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure());
        assertPersonSpouseStructure(personInformationStructure.getPersonSpouseStructure());
        assertPersonCustodyStructure(personInformationStructure.getPersonCustodyStructure());
        assertPersonWardStructure(personInformationStructure.getPersonWardStructure());
    }

    public static void assertPersonInformationStructureProtected(PersonInformationStructureType personInformationStructure) {
        assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(),true);
        assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),true);
        assertPersonSpouseStructure(personInformationStructure.getPersonSpouseStructure());
        assertPersonCustodyStructure(personInformationStructure.getPersonCustodyStructure());
        assertPersonWardStructure(personInformationStructure.getPersonWardStructure());
    }

    public static void assertPersonWardStructure(List<PersonWardStructureType> personWardStructure) {
        assertEquals(4,personWardStructure.size());
        Iterator<PersonWardStructureType> iter = personWardStructure.iterator();
        Set<String> foundCpr = Sets.newHashSet();
        while (iter.hasNext()) {
            PersonWardStructureType ward = iter.next();
            foundCpr.add(ward.getPersonWardIdentifier());
            switch (ward.getPersonWardIdentifier()) {
                case StamdataTestHelper.CPR_BARN1:
                    assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
                    assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
                    break;
                case StamdataTestHelper.CPR_BARN2:
                    assertEquals(PersonWardRelationType.BARN_UDEN,ward.getPersonWardRelation());
                    assertEquals(StamdataTestHelper.CPR_BARN2, ward.getPersonWardIdentifier());
                    break;
                case StamdataTestHelper.CPR_BARN3:
                    assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
                    assertEquals(StamdataTestHelper.CPR_BARN3, ward.getPersonWardIdentifier());
                    break;
                case StamdataTestHelper.CPR_BARN4:
                    assertEquals(PersonWardRelationType.MYNDLING,ward.getPersonWardRelation());
                    assertEquals(StamdataTestHelper.CPR_BARN4, ward.getPersonWardIdentifier());
                    break;
                default:
                    fail("Unknown CPR in PersonWardStructure - "+ward.getPersonWardIdentifier());

            }
        }
        assertEquals("Found cpr "+Arrays.toString(foundCpr.toArray()),4,foundCpr.size());
    }

    public static void assertPersonCustodyStructure(List<PersonCustodyStructureType> personCustodyStructure) {
        assertEquals(2,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        assertPersonCustodyStructure(personCustody);
        PersonCustodyStructureType personCustody2 = personCustodyStructure.get(1);
        assertPersonCustodyStructure(personCustody2);
        assertNotSame(personCustody.getPersonCustodyRelation(), personCustody2.getPersonCustodyRelation());
    }

    public static void assertPersonCustodyStructure(PersonCustodyStructureType personCustody) {
        if (personCustody.getPersonCustodyRelation().equals(PersonCustodyRelationType.FAR)) {
            assertpersonCustody(personCustody, StamdataTestHelper.CPR_FAR, PersonCustodyRelationType.FAR);
        } else if (personCustody.getPersonCustodyRelation().equals(PersonCustodyRelationType.MOR)) {
            assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.MOR);
        }
    }

    public static void assertpersonCustody(PersonCustodyStructureType personCustody, String cpr, PersonCustodyRelationType relation) {
        assertEquals(cpr,personCustody.getPersonCustodyIdentifier());
        assertEquals(relation,personCustody.getPersonCustodyRelation());
    }

    public static void assertPersonSpouseStructure(PersonSpouseStructureType personSpouseStructure) {
        assertEquals(StamdataTestHelper.CPR_SPOUSE,personSpouseStructure.getPersonSpouseIdentifier());
        assertEquals(PersonSpouseRelationType.AEGTEFAELLE.toString(),personSpouseStructure.getPersonSpouseRelation().toString());
    }

    public static void assertPersonAddressStructure(PersonAddressStructureType personAddressStructure) {
        assertPersonAddressStructure(personAddressStructure,false);
    }

    public static void assertPersonAddressStructure(PersonAddressStructureType personAddressStructure, boolean censored) {
        assertEquals((censored) ? null : "Søren Petersen",personAddressStructure.getCareOfName());
        assertAddressStructure(personAddressStructure.getAddressComplete(),censored);
        assertDate((censored) ? Factories.YEAR_2000 : null,personAddressStructure.getPersonInformationProtectionStartDate());
        assertEquals((censored) ? ZERO : "1083",personAddressStructure.getCountyCode());
    }

    public static void assertDate(String string, XMLGregorianCalendar date) {
        assertEquals("2000",date.toString());
    }

    public static void assertDate(Date date, XMLGregorianCalendar calendar) {
        if (date != null || calendar != null) {
            assertEquals(date.toString(),calendar.toGregorianCalendar().getTime().toString());
        }
    }

    public static void assertAddressStructure(AddressCompleteType addressComplete) {
        assertAddressStructure(addressComplete,false);
    }

    public static void assertAddressStructure(AddressCompleteType addressComplete, boolean censored) {
        assertAddressAccess(addressComplete,censored);
        assertAddressPostal(addressComplete.getAddressPostal(),censored);
    }

    public static void assertAddressPostal(AddressPostalType addressPostal,boolean censored) {
        assertNull(addressPostal.getMailDeliverySublocationIdentifier());
        assertEquals((censored) ? ADRESSEBESKYTTET : "Ørstedgade",addressPostal.getStreetName());
        assertEquals((censored) ? null : "Østergd.",addressPostal.getStreetNameForAddressingName());
        assertEquals((censored) ? null : "10",addressPostal.getStreetBuildingIdentifier());
        assertEquals((censored) ? null : "12",addressPostal.getFloorIdentifier());
        assertEquals((censored) ? null : "tv",addressPostal.getSuiteIdentifier());
        assertEquals((censored) ? null : "BirkeBy",addressPostal.getDistrictSubdivisionIdentifier());
        assertNull(addressPostal.getPostOfficeBoxIdentifier());
        assertEquals((censored) ? ZERO : "6666",addressPostal.getPostCodeIdentifier());
        assertEquals((censored) ? ADRESSEBESKYTTET : "Überwald",addressPostal.getDistrictName());
        if (censored) {
            assertNull(addressPostal.getCountryIdentificationCode());
        } else {
            assertCountryIdentificationCode(addressPostal.getCountryIdentificationCode());
        }
    }

    public static void assertCountryIdentificationCode(CountryIdentificationCodeType countryIdentificationCode) {
        assertEquals(CountryIdentificationSchemeType.ISO_3166_ALPHA_2.toString(), countryIdentificationCode.getScheme().toString());
        assertEquals("DK",countryIdentificationCode.getValue());
    }

    public static void assertAddressAccess(AddressCompleteType addressComplete, boolean censored) {
        assertEquals((censored) ? ZERO : "0461",addressComplete.getAddressAccess().getMunicipalityCode());
        assertDate((censored) ? null : Factories.YEAR_2000,addressComplete.getAddressAccess().getMunicipalityCodeStartDate());
        assertEquals((censored) ? ZERO : "0234",addressComplete.getAddressAccess().getStreetCode());
        assertEquals((censored) ? null : "10",addressComplete.getAddressAccess().getStreetBuildingIdentifier());
    }

    public static void assertRegularCPRPerson(RegularCPRPersonType regularCPRPerson) {
        assertRegularCPRPerson(regularCPRPerson, false);
    }

    public static void assertRegularCPRPerson(RegularCPRPersonType regularCPRPerson, boolean censored) {
        assertSimpleCPRPerson(regularCPRPerson.getSimpleCPRPerson(), censored);
        assertEquals((censored) ? ADRESSEBESKYTTET : "Peter,Andersen",regularCPRPerson.getPersonNameForAddressingName());
        assertEquals((censored) ? PersonGenderCodeType.UNKNOWN : PersonGenderCodeType.MALE,regularCPRPerson.getPersonGenderCode());
        assertEquals(censored, regularCPRPerson.isPersonInformationProtectionIndicator());
        assertDate((censored) ? Factories.FIRST_UNIX_DATE : Factories.TWO_DAYS_AGO,regularCPRPerson.getPersonBirthDateStructure().getBirthDate());
        assertPersonCivilRegistrationStatusStructure(regularCPRPerson.getPersonCivilRegistrationStatusStructure(),censored);
    }

    public static void assertPersonCivilRegistrationStatusStructure(PersonCivilRegistrationStatusStructureType personCivilRegistrationStatusStructure) {
        assertPersonCivilRegistrationStatusStructure(personCivilRegistrationStatusStructure,false);
    }

    public static void assertPersonCivilRegistrationStatusStructure(PersonCivilRegistrationStatusStructureType personCivilRegistrationStatusStructure, boolean censored) {
        assertEquals("01",personCivilRegistrationStatusStructure.getPersonCivilRegistrationStatusCode());
        assertDate((censored) ? Factories.FIRST_UNIX_DATE : Factories.YESTERDAY,personCivilRegistrationStatusStructure.getPersonCivilRegistrationStatusStartDate());
    }

    public static void assertSimpleCPRPerson(SimpleCPRPersonType simpleCPRPerson) {
        assertSimpleCPRPerson(simpleCPRPerson,false);
    }

    public static void assertSimpleCPRPerson(SimpleCPRPersonType simpleCPRPerson, boolean censored) {
        if (censored) {
            assertPersonNameStructureCensored(simpleCPRPerson.getPersonNameStructure());
        } else {
            assertPersonNameStructure(simpleCPRPerson.getPersonNameStructure());
        }
        assertPersonCivilRegistrationIdentifier(simpleCPRPerson.getPersonCivilRegistrationIdentifier());
    }

    public static void assertPersonNameStructure(PersonNameStructureType personNameStructure) {
        assertEquals("Peter",personNameStructure.getPersonGivenName());
        assertEquals("Sigurd",personNameStructure.getPersonMiddleName());
        assertEquals("Andersen",personNameStructure.getPersonSurnameName());
    }

    public static void assertPersonNameStructureCensored(PersonNameStructureType personNameStructure) {
        assertEquals(ADRESSEBESKYTTET,personNameStructure.getPersonGivenName());
        assertNull(personNameStructure.getPersonMiddleName());
        assertEquals(ADRESSEBESKYTTET,personNameStructure.getPersonSurnameName());
    }

    public static void assertPersonCivilRegistrationIdentifier(String personCivilRegistrationIdentifier) {
        assertNotNull("Must not be null",personCivilRegistrationIdentifier);
    }

    public static void assertPersonHealthCareInformationStructure(PersonHealthCareInformationStructureType personHealthCareInformationStructure, PublicHealthInsuranceGroupIdentifierType groupCode) {
        assertCurrentGeneralPractitionerStructure(personHealthCareInformationStructure.getCurrentGeneralPractitionerStructure());
        assertPriorGeneralPractitionerStructure(personHealthCareInformationStructure.getPriorGeneralPractitionerStructure());
        assertPersonPublicHealthInsurance(personHealthCareInformationStructure.getPersonPublicHealthInsurance(), groupCode);
    }

    static void assertPersonPublicHealthInsurance(PersonPublicHealthInsuranceType personPublicHealthInsurance, PublicHealthInsuranceGroupIdentifierType groupCode) {
        assertEquals(groupCode,personPublicHealthInsurance.getPublicHealthInsuranceGroupIdentifier());
        assertDate(Factories.YESTERDAY,personPublicHealthInsurance.getPublicHealthInsuranceGroupStartDate());
        assertEquals(PublicHealthInsuranceStatusType.D,personPublicHealthInsurance.getPublicHealthInsuranceStatus());
    }

    static void assertPriorGeneralPractitionerStructure(PriorGeneralPractitionerStructureType priorGeneralPractitionerStructure) {
        assertAssociatedGeneralPractitionerStructure(priorGeneralPractitionerStructure.getAssociatedGeneralPractitionerStructure(), "4321");
    }

    static void assertCurrentGeneralPractitionerStructure(CurrentGeneralPractitionerStructureType currentGeneralPractitionerStructure) {
        assertAssociatedGeneralPractitionerStructure(currentGeneralPractitionerStructure.getAssociatedGeneralPractitionerStructure(), "1234");
        assertEquals("12345678",currentGeneralPractitionerStructure.getCVRnumberIdentifier());
    }

    public static void assertAssociatedGeneralPractitionerStructure(AssociatedGeneralPractitionerStructureType associatedGeneralPractitionerStructure, String ydernumber) {
        assertEquals(ydernumber,associatedGeneralPractitionerStructure.getAssociatedGeneralPractitionerIdentifier().toString());
        assertEquals("Klinikken",associatedGeneralPractitionerStructure.getAssociatedGeneralPractitionerOrganisationName());
        assertEquals("Margrethepladsen 44, 8000 Århus",associatedGeneralPractitionerStructure.getStandardAddressIdentifier());
        assertEquals("8000",associatedGeneralPractitionerStructure.getPostCodeIdentifier());
        assertEquals("Århus",associatedGeneralPractitionerStructure.getDistrictName());
        assertEquals("12345678",associatedGeneralPractitionerStructure.getTelephoneSubscriberIdentifier());
        assertEquals("test@example.com",associatedGeneralPractitionerStructure.getEmailAddressIdentifier());
    }

    public static void assertFrontierInformationStructure(FrontierInformationStructureType frontierInformationStructure) {
        assertDate(Factories.YESTERDAY,frontierInformationStructure.getFrontierStartDate());
        assertDate(Factories.YESTERDAY_PLUS_2_YEARS,frontierInformationStructure.getFrontierEndDate());
        assertEquals("GB",frontierInformationStructure.getFrontierCitizenshipStructure().getFrontierCitizenshipCode());
        assertEquals("England",frontierInformationStructure.getFrontierCitizenshipStructure().getFrontierCitizenship());
        assertEquals("DE",frontierInformationStructure.getFrontierResidenceCountryStructure().getFrontierResidenceCountryCode());
        assertEquals("Tyskland",frontierInformationStructure.getFrontierResidenceCountryStructure().getFrontierResidenceCountry());
        assertEquals("DK",frontierInformationStructure.getFrontierSocialSecurityStructure().getFrontierSocialSecurityCountryStructure().getFrontierSocialSecurityCountryCode());
        assertEquals("Danmark",frontierInformationStructure.getFrontierSocialSecurityStructure().getFrontierSocialSecurityCountryStructure().getFrontierSocialSecurityCountry());
    }

    public static void assertPersonWithHealthCareInformationStructure(PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure) {
        assertPersonInformationStructure(personWithHealthCareInformationStructure.getPersonInformationStructure());
        assertPersonHealthCareInformationStructure(personWithHealthCareInformationStructure.getPersonHealthCareInformationStructure(), PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_1);
        assertFrontierInformationStructure(personWithHealthCareInformationStructure.getFrontierInformationStructure());
    }

    public static void assertPersonWithHealthCareInformationStructureProtected(PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure) {
        assertPersonInformationStructureProtected(personWithHealthCareInformationStructure.getPersonInformationStructure());
        assertPersonHealthCareInformationStructure(personWithHealthCareInformationStructure.getPersonHealthCareInformationStructure(), PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_1);
        assertFrontierInformationStructure(personWithHealthCareInformationStructure.getFrontierInformationStructure());
    }

}
