/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.medcom;

import static junit.framework.Assert.assertNull;
import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;

import java.math.BigInteger;
import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;

import javax.xml.namespace.QName;
import javax.xml.ws.soap.SOAPFaultException;

import org.hisrc.hifaces20.testing.webappenvironment.testing.junit4.AbstractWebAppEnvironmentJUnit4Test;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.SikredeVO;
import com.trifork.stamdata.models.YderVO;

import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import junit.framework.Assert;
import oio.medcom.cprservice._1_0.DetGodeCPROpslag;
import oio.medcom.cprservice._1_0.DetGodeCPROpslagService;
import oio.medcom.cprservice._1_0.GetPersonInformationIn;
import oio.medcom.cprservice._1_0.GetPersonInformationOut;
import oio.medcom.cprservice._1_0.GetPersonWithHealthCareInformationIn;
import oio.medcom.cprservice._1_0.GetPersonWithHealthCareInformationOut;
import oio.medcom.cprservice._1_0.PublicHealthInsuranceGroupIdentifierType;

@RunWith(GuiceTestRunner.class)
public class DetGodeCPROpslag102IntegrationTest extends AbstractWebAppEnvironmentJUnit4Test {

    public static final QName DET_GODE_CPR_OPSLAG_102_SERVICE = new QName("urn:oio:medcom:cprservice:1.0.2", "DetGodeCPROpslagService");
    public static final String WHITELISTED_CVR = "12345678";

    @Inject
    private Provider<Connection> connectionProvider;

    private DetGodeCPROpslag client;

    private GetPersonInformationIn request = new GetPersonInformationIn();
    private GetPersonInformationOut response;
    private GetPersonWithHealthCareInformationOut healthCareResponse;

    private GetPersonWithHealthCareInformationIn healthCareRequest = new GetPersonWithHealthCareInformationIn();

    private StamdataTestHelper testHelper = new StamdataTestHelper();

    @Inject
    Connection connection;

    @Before
    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    @Before
    public void setUp() throws Exception {
        URL wsdlLocation = new URL("http://localhost:8100/service/DetGodeCPROpslag-1.0.2?wsdl");
        DetGodeCPROpslagService serviceCatalog =
                new DetGodeCPROpslagService(wsdlLocation, DET_GODE_CPR_OPSLAG_102_SERVICE);
        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());
        client = serviceCatalog.getDetGodeCPROpslag();
    }

    private void sendPersonRequest() throws Exception {
        sendPersonRequest(WHITELISTED_CVR);
    }

    private void sendPersonRequest(String cvr) throws Exception {
        testHelper.prepareTestdata(connection);
        performRequest(cvr);
    }

    private void performRequest(String cvr) throws Exception {
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(cvr, "foo", "bar");
        response = client.getPersonInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), request);
    }

    @Test
    @Ignore // ValidTo er not null i skemaet så man skal lige lave lidt manuelt først
    public void testValidToNull() throws Exception {
        PersonVO person = testHelper.createPerson("1234");
        request.setPersonCivilRegistrationIdentifier(person.getCpr());

        testHelper.deleteAllPersons(connection);
        testHelper.persist(connection,person);
        connection.createStatement().executeUpdate("UPDATE v2_Person SET ValidTo=NULL");
        connection.commit();
        performRequest(WHITELISTED_CVR);
    }

    @Test
    public void badCPRShouldFail() throws Exception {
        request.setPersonCivilRegistrationIdentifier("bad_cpr");
        // Send request with a non whitelistet cvr, which should result in an exception
        try {
            sendPersonRequest(WHITELISTED_CVR);
            fail("Expected validation error");
        } catch (SOAPFaultException e) {
            assertEquals("cvc-type.3.1.3: The value 'bad_cpr' of element 'ANY_NAMESPACE:PersonCivilRegistrationIdentifier' is not valid.",
            		e.getMessage().replaceAll("ns\\d+", "ANY_NAMESPACE"));
        }
    }

    @Test(expected = SOAPFaultException.class)
    public void testMissingPerson() throws Exception {
        request.setPersonCivilRegistrationIdentifier("1212121111");
        sendPersonRequest();
    }

    // SDM-45 Håndtering af postnummer '0'
    @Test
    public void invalidPostCodeGetsPadded() throws Exception {
        PersonVO person = testHelper.createPersonWithPostcode("0");
        request.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendPersonRequest();
        assertThat(response.getPersonInformationStructure().getPersonAddressStructure().getAddressComplete().getAddressPostal().getPostCodeIdentifier(),
                is("0000"));
    }

    @Test(expected = SOAPFaultException.class)
    public void nonWhitelistCvrShouldFail() throws Exception {
        PersonVO person = testHelper.createPerson();
        request.setPersonCivilRegistrationIdentifier(person.getCpr());
        // Send request with a non whitelistet cvr, which should result in an exception
        sendPersonRequest("12341234");
    }

    @Test
    public void canRequestPersonWithoutPostDistrict1() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setPostdistrikt("");
        request.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendPersonRequest();
        String districtName = response.getPersonInformationStructure().getPersonAddressStructure().getAddressComplete().
                getAddressPostal().getDistrictName();
        Assert.assertEquals("UKENDT", districtName);
    }

    @Test
    public void canRequestPersonWithoutPostDistrict2() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setPostdistrikt(null);
        request.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendPersonRequest();

        String districtName = response.getPersonInformationStructure().getPersonAddressStructure().getAddressComplete().
                getAddressPostal().getDistrictName();
        Assert.assertEquals("UKENDT", districtName);
    }

    @Test
    public void canRequestPersonWithoutPostCode() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setPostnummer(null);
        request.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendPersonRequest();

        String postCode = response.getPersonInformationStructure().getPersonAddressStructure().getAddressComplete().
                getAddressPostal().getPostCodeIdentifier();
        Assert.assertEquals("0000", postCode);
    }

    @Test
    public void requestPersonWithHealthcareInformationButNoPublicHealthInsuranceInformation() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();

         YderVO yder1 = testHelper.createYder("1234");
         YderVO yder2 = testHelper.createYder("4321");

        testHelper.createSikrede(person1, yder1, "1", new DateTime(2011, 10, 10, 0, 0));
        testHelper.createSikrede(person2, yder2, "2", new DateTime(2011, 10, 10, 0, 0));

        // Having multiple persons ensures that we are selecting the
        // right one.

        healthCareRequest.setPersonCivilRegistrationIdentifier(person1.getCpr());

        sendHealthCareRequest();

        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonInformationStructure().getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is(person1.getFornavn()));
        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonHealthCareInformationStructure().getAssociatedGeneralPractitionerStructure().getAssociatedGeneralPractitionerIdentifier().intValue(), is(1234));
        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonHealthCareInformationStructure().getAssociatedGeneralPractitionerStructure().getPostCodeIdentifier(), is("8000"));
        assertEquals(PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_1, healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonHealthCareInformationStructure().getPersonPublicHealthInsurance().getPublicHealthInsuranceGroupIdentifier());
    }

    //SDM-967
    @Test
    public void requestPersonReturnStatusCodeAsString() throws Exception {
        PersonVO person = testHelper.createPerson();
        request.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendPersonRequest();

        assertEquals(BigInteger.ONE, response.getPersonInformationStructure().getRegularCPRPerson().getPersonCivilRegistrationStatusStructure().getPersonCivilRegistrationStatusCode());
    }

    @Test
    public void requestForExistingPersonGivesPersonInformation() throws Exception {
        PersonVO person = testHelper.createPerson();
        request.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendPersonRequest();

        assertThat(response.getPersonInformationStructure().getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is(person.getFornavn()));
    }

    @Test
    public void requestPersonWithNoStreetbuildingIdentifier() throws Exception {
        PersonVO person = Factories.createPerson(false, "6666");
        testHelper.addPerson(person);
        YderVO yder = testHelper.createYder("1234");
        testHelper.createSikrede(person, yder, "9", new DateTime(2011, 10, 10, 0, 0));

        healthCareRequest.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendHealthCareRequest();

        assertNull(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonInformationStructure().getPersonAddressStructure().getAddressComplete().getAddressAccess().getStreetBuildingIdentifier());
        assertNull(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonInformationStructure().getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetBuildingIdentifier());
        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonInformationStructure().getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is(person.getFornavn()));
    }

    @Test
    public void requestPersonWithFullHealthcareInformation() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();

        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");

        testHelper.createSikrede(person1, yder1, "9", new DateTime(2011, 10, 10, 0, 0));
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        healthCareRequest.setPersonCivilRegistrationIdentifier(person1.getCpr());
        sendHealthCareRequest();

        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonInformationStructure().getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is(person1.getFornavn()));
        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonHealthCareInformationStructure().getAssociatedGeneralPractitionerStructure().getAssociatedGeneralPractitionerIdentifier().intValue(), is(1234));
        assertEquals(PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_9, healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonHealthCareInformationStructure().getPersonPublicHealthInsurance().getPublicHealthInsuranceGroupIdentifier());
    }

    @Test
    public void requestForPersonWithActiveProtectionResultsInProtectedData() throws Exception {
        PersonVO person = Factories.createPersonWithAddressProtection();
        testHelper.addPerson(person);

        request.setPersonCivilRegistrationIdentifier(person.getCpr());

        sendPersonRequest();

        assertThat(response.getPersonInformationStructure().getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is("ADRESSEBESKYTTET"));
    }


    @Test
    public void doctorMissingReturnUnknown() throws Exception {
        PersonVO person = testHelper.createPersonWithPostcode("1234");
        YderVO yder = testHelper.createYder("1234");
        SikredeVO sikrede = testHelper.createSikrede(person, yder, "1", new DateTime(2011, 10, 10, 0, 0), new DateTime(2011, 10, 10, 0, 0));
        sikrede.setSYdernr("4321");

        healthCareRequest.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendHealthCareRequest();

        assertEquals(PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_1, healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonHealthCareInformationStructure().getPersonPublicHealthInsurance().getPublicHealthInsuranceGroupIdentifier());
        assertEquals("UKENDT", healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonHealthCareInformationStructure().getAssociatedGeneralPractitionerStructure().getAssociatedGeneralPractitionerOrganisationName());
        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonInformationStructure().getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName(), is(person.getFornavn()));
    }

    @Test
    public void requestPersonWithHealthcare_SikredeContainsExtraZeroes() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        testHelper.createYder("1234");
        YderVO yder1zeroes = testHelper.createYder("001234");
        testHelper.createSikrede(person1, yder1zeroes, "9", new DateTime(2011, 10, 10, 0, 0));

        healthCareRequest.setPersonCivilRegistrationIdentifier(person1.getCpr());
        sendHealthCareRequest();
        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().
                getPersonHealthCareInformationStructure().getAssociatedGeneralPractitionerStructure().
                getAssociatedGeneralPractitionerIdentifier().intValue(), is(1234));
    }

    @Test
    public void requestPersonWithHealthcare_TwoSikredes() throws Exception {
        PersonVO person1 = testHelper.createPerson();

        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("1235");
        testHelper.createSikrede(person1, yder1, "9", new DateTime(2011, 10, 10, 0, 0), new DateTime(2011, 10, 10, 0, 0));
        testHelper.createSikrede(person1, yder2, "9", new DateTime(2012, 10, 10, 0, 0), new DateTime(2012, 10, 10, 0, 0));

        healthCareRequest.setPersonCivilRegistrationIdentifier(person1.getCpr());
        sendHealthCareRequest();
        assertThat(healthCareResponse.getPersonWithHealthCareInformationStructure().
                getPersonHealthCareInformationStructure().getAssociatedGeneralPractitionerStructure().
                getAssociatedGeneralPractitionerIdentifier().intValue(), is(1235));
    }

    @Test
    public void requestProviderWithInvalidEmail() throws Exception {
        PersonVO person = testHelper.createPerson();
        YderVO yder = testHelper.createYder("1235");
        yder.setEmailYder("Skrammelmail.com");
        testHelper.createSikrede(person, yder, "9", new DateTime(2011, 10, 10, 0, 0), new DateTime(2011, 10, 10, 0, 0));

        healthCareRequest.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendHealthCareRequest();
        String emailAddressIdentifier = healthCareResponse.getPersonWithHealthCareInformationStructure().
                getPersonHealthCareInformationStructure().getAssociatedGeneralPractitionerStructure().getEmailAddressIdentifier();
        assertNull(emailAddressIdentifier);
    }

    @Test(expected = SOAPFaultException.class)
    public void requestPersonWithNoStatus() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setStatus(null);
        request.setPersonCivilRegistrationIdentifier(person.getCpr());
        sendPersonRequest();
    }

    private void sendHealthCareRequest() throws Exception {
        testHelper.prepareTestdata(connection);
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        healthCareResponse = client.getPersonWithHealthCareInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), healthCareRequest);
    }

}
