/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.medcom;

import static junit.framework.Assert.assertNull;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPFault;
import javax.xml.ws.soap.SOAPFaultException;

import org.apache.log4j.Appender;
import org.apache.log4j.Layout;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;
import org.apache.log4j.WriterAppender;
import org.hisrc.hifaces20.testing.webappenvironment.testing.junit4.AbstractWebAppEnvironmentJUnit4Test;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO.FMRTypeKode;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.SikredeVO;
import com.trifork.stamdata.models.YderVO;

import dk.nsi.stamdata.cpr.SoapUtils;
import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.RequestHelper;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import dk.nsi.stamdata.testing.TestHelper;
import oio.medcom.cprservice._1_0_4.DGWSFault;
import oio.medcom.cprservice._1_0_4.DetGodeCPROpslag;
import oio.medcom.cprservice._1_0_4.DetGodeCPROpslagService;
import oio.medcom.cprservice._1_0_4.GetPersonInformationIn;
import oio.medcom.cprservice._1_0_4.GetPersonInformationOut;
import oio.medcom.cprservice._1_0_4.GetPersonWithHealthCareInformationIn;
import oio.medcom.cprservice._1_0_4.GetPersonWithHealthCareInformationOut;
import oio.medcom.cprservice._1_0_4.PersonCustodyRelationType;
import oio.medcom.cprservice._1_0_4.PersonCustodyStructureType;
import oio.medcom.cprservice._1_0_4.PersonHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4.PersonInformationStructureType;
import oio.medcom.cprservice._1_0_4.PersonSpouseStructureType;
import oio.medcom.cprservice._1_0_4.PersonWardRelationType;
import oio.medcom.cprservice._1_0_4.PersonWardStructureType;
import oio.medcom.cprservice._1_0_4.PersonWithHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4.PublicHealthInsuranceGroupIdentifierType;

@RunWith(GuiceTestRunner.class)
public class DetGodeCPROpslag104IntegrationTest extends AbstractWebAppEnvironmentJUnit4Test {

    public static final QName DET_GODE_CPR_OPSLAG_104_SERVICE = new QName("urn:oio:medcom:cprservice:1.0.4", "DetGodeCPROpslagService");
    public static final String WHITELISTED_CVR = "12345678";

    @Inject
    private Provider<Connection> connectionProvider;

    private DetGodeCPROpslag client;

    private GetPersonInformationIn request = new GetPersonInformationIn();
    private GetPersonInformationOut response;
    private GetPersonWithHealthCareInformationOut healthCareResponse;

    private GetPersonWithHealthCareInformationIn healthCareRequest = new GetPersonWithHealthCareInformationIn();

    private StamdataTestHelper testHelper = new StamdataTestHelper();

    TestHelper helper = new TestHelper();

    @Inject
    Connection connection;

    @Before
    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    @Before
    public void setUp() throws Exception {
        URL wsdlLocation = new URL("http://localhost:8100/service/DetGodeCPROpslag-1.0.4?wsdl");
        DetGodeCPROpslagService serviceCatalog =
                new DetGodeCPROpslagService(wsdlLocation, DET_GODE_CPR_OPSLAG_104_SERVICE);
        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());
        client = serviceCatalog.getDetGodeCPROpslag();
    }

    private void sendPersonRequest(String cpr) throws Exception {
    	helper.enableAuditlogging(true);
    	ByteArrayOutputStream outStream = helper.redirectSystemOutToStream();
        request.setPersonCivilRegistrationIdentifier(cpr);
        try {
            performRequest();
        } finally {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
        }
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4 getPersonInformation\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getPersonCivilRegistrationIdentifier()+"\"}"+System.lineSeparator(), outStream.toString());
    }

    private void performRequest() throws Exception {
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        response = client.getPersonInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), request);
    }

    private void sendHealthCareRequest(String cpr) throws Exception {
        healthCareRequest.setPersonCivilRegistrationIdentifier(cpr);
        sendHealthCareRequest();
    }

    private void sendHealthCareRequest() throws Exception {
    	helper.enableAuditlogging(true);
    	ByteArrayOutputStream outStream = helper.redirectSystemOutToStream();

        testHelper.prepareTestdata(connection);
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        healthCareResponse = client.getPersonWithHealthCareInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), healthCareRequest);

        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4 getPersonWithHealthcareInformation\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+healthCareRequest.getPersonCivilRegistrationIdentifier()+"\"}"+System.lineSeparator(), outStream.toString());
        helper.enableAuditlogging(false);
        helper.resetSystemOut();
    }

    @Test
    public void requestForPersonInformation() throws Exception {
        PersonVO person = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person);
        testHelper.createVaerge(personBarn4, person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());

        AssertResponse104.assertPersonInformationStructure(response.getPersonInformationStructure());
    }

    @Test
    public void requestForMissingPerson() throws Exception {
        final ByteArrayOutputStream outContent = getLogsFrom(SoapUtils.class);

        try {
            sendPersonRequest(StamdataTestHelper.CPR_MISSING);
        } catch (SOAPFaultException e) {
            assertEquals(true,outContent.toString().contains("Fejl under processering af besked. Ingen data fundet"));
            SOAPFault soapfault = e.getFault();
            assertNotNull(soapfault);
            assertEquals("Sender",soapfault.getFaultCode().split(":")[1]);
            assertEquals("Ingen data fundet",soapfault.getFaultString());
            return;
        }
        fail();
    }

    @Test
    public void requestForMissingPersonSoapFault() throws Exception {
        try {
            request.setPersonCivilRegistrationIdentifier(StamdataTestHelper.CPR_MISSING);
            SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
            response = client.getPersonInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), request);
        } catch (SOAPFaultException e) {
            SOAPFault soapfault = e.getFault();
            assertNotNull(soapfault);
            assertEquals("Sender",soapfault.getFaultCode().split(":")[1]);
            assertEquals("Ingen data fundet",soapfault.getFaultString());
            return;
        } catch (Exception e) {
            System.out.println("DEBUG HMA: exception class "+e.getClass().getCanonicalName());
            DGWSFault fault = (DGWSFault) e;
            System.out.println("DEBUG HMA: exception class "+fault.toString());
        }
        fail();
    }

    public ByteArrayOutputStream getLogsFrom(Class logClass) {
        final String APPENDER_NAME = "log4jRuleAppender";
        final Layout LAYOUT = new SimpleLayout();

        Logger logger;
        final ByteArrayOutputStream outContent = new ByteArrayOutputStream();

        logger = Logger.getLogger(logClass);
        Appender appender = new WriterAppender(LAYOUT, outContent);
        appender.setName(APPENDER_NAME);
        logger.addAppender(appender);
        return outContent;
    }

    @Test
    public void requestForPersonInformationWithProtection() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setNavnebeskyttelsestartdato(testHelper.formatSimpleDate("2000-01-01"));
        person.setNavnebeskyttelseslettedato(testHelper.formatSimpleDate("2050-01-01"));
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person);
        testHelper.createVaerge(personBarn4, person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());

        AssertResponse104.assertPersonInformationStructureProtected(response.getPersonInformationStructure());
    }

    @Test
    public void requestForPersonWithHealthCareInformation() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person1, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person1);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person1);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person1, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person1, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person1);
        testHelper.createVaerge(personBarn4, person1);
        YderVO yderRecord1 = testHelper.createYder("1234");
        YderVO yderRecord2 = testHelper.createYder("4321");
        SikredeVO sikredeRecord1 = testHelper.createSikredeWithFrontierInfo(person1, yderRecord1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikredeRecord1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yderRecord2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        AssertResponse104.assertPersonWithHealthCareInformationStructure(personWithHealthCareInformationStructure);
    }

    @Test
    public void requestForPersonWithHealthCareInformationWithProtection() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        person1.setNavnebeskyttelsestartdato(testHelper.formatSimpleDate("2000-01-01"));
        person1.setNavnebeskyttelseslettedato(testHelper.formatSimpleDate("2050-01-01"));
        PersonVO person2 = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person1, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person1);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person1);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person1, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person1, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person1);
        testHelper.createVaerge(personBarn4, person1);
        YderVO yderRecord1 = testHelper.createYder("1234");
        YderVO yderRecord2 = testHelper.createYder("4321");
        SikredeVO sikredeRecord1 = testHelper.createSikredeWithFrontierInfo(person1, yderRecord1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikredeRecord1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yderRecord2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        AssertResponse104.assertPersonWithHealthCareInformationStructureProtected(personWithHealthCareInformationStructure);
    }

    @Test
    public void requestWithGroupCode0() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();
        YderVO yderRecord1 = testHelper.createYder("1234");
        YderVO yderRecord2 = testHelper.createYder("4321");
        SikredeVO sikredeRecord1 = testHelper.createSikredeWithFrontierInfo(person1, yderRecord1, "0", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikredeRecord1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yderRecord2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        PersonHealthCareInformationStructureType personHealthCareInformationStructure = personWithHealthCareInformationStructure.getPersonHealthCareInformationStructure();
        AssertResponse104.assertPersonHealthCareInformationStructure(personHealthCareInformationStructure, PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_0);
    }

    @Test
    public void requestForSeperatedPerson() throws Exception {
        PersonVO person = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createPerson(StamdataTestHelper.CPR_SPOUSE);
        testHelper.createSeperated(person, personSpouse);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        assertNull(personInformationStructure.getPersonSpouseStructure());
    }

    @Test
    public void requestForUnmarriedPerson() throws Exception {
        PersonVO person = testHelper.createPerson();
        testHelper.createUnmarried(person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        assertNull(personInformationStructure.getPersonSpouseStructure());
    }

    @Test
    public void requestForRPPerson() throws Exception {
        PersonVO person = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personSpouse = testHelper.createMand(StamdataTestHelper.CPR_MEDFAR);
        testHelper.createSpouse(person, personSpouse);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        PersonSpouseStructureType personSpouseStructure = personInformationStructure.getPersonSpouseStructure();
        assertEquals(StamdataTestHelper.CPR_MEDFAR,personSpouseStructure.getPersonSpouseIdentifier());
        assertEquals("registreret_partner",personSpouseStructure.getPersonSpouseRelation().value());
    }

    @Test
    public void testSignatureInvalidModelBuildException() throws Exception {

    		String fileName = "src/test/resources/data/validXml.xml";
    		String xml = RequestHelper.readFile(fileName);
    		String replication_endpoint = "http://localhost:8100/service/DetGodeCPROpslag-1.0.4";

    		String responseXml = RequestHelper.sendRequest(replication_endpoint, "http://rep.oio.dk/medcom.sundcom.dk/xml/wsdl/2007/06/28/getPersonInformation", xml);

    		String faultstring = RequestHelper.getFaultstring(responseXml);
    		assertEquals(faultstring.substring(0, 114), "The signature used to sign the message was incorrectly signed or no longer valid. Signature could not be validated");
    }

    @Test
    public void requestForCustodyAndenIndehaver() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);

        testHelper.createBarnRelation(personFar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR, personFar);

        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, personMor);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        assertEquals(2,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_FAR, PersonCustodyRelationType.FAR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.ANDEN);

        sendPersonRequest(StamdataTestHelper.CPR_FAR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForCustodyMedmor() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personMedmor = testHelper.createKvinde(StamdataTestHelper.CPR_MEDMOR);

        testHelper.createBarnRelation(personFar, person);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personMedmor, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR, personFar);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR, personMor);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        assertEquals(3,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.MOR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MEDMOR, PersonCustodyRelationType.MOR);
        personCustody = personCustodyStructure.get(2);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_FAR, PersonCustodyRelationType.FAR);

        sendPersonRequest(StamdataTestHelper.CPR_FAR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MEDMOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForCustodyMedfar() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personMedfar = testHelper.createMand(StamdataTestHelper.CPR_MEDFAR);

        testHelper.createBarnRelation(personFar, person);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personMedfar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR, personFar);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        assertEquals(3,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_FAR, PersonCustodyRelationType.FAR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MEDFAR, PersonCustodyRelationType.FAR);
        personCustody = personCustodyStructure.get(2);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.MOR_UDEN);

        sendPersonRequest(StamdataTestHelper.CPR_FAR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN_UDEN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MEDFAR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForVaerge() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personVaerge = testHelper.createPerson(StamdataTestHelper.CPR_VAERGE);
        testHelper.createVaerge(person, personVaerge);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();

        assertEquals(1,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse104.assertpersonCustody(personCustody, StamdataTestHelper.CPR_VAERGE, PersonCustodyRelationType.VAERGE);

        sendPersonRequest(personVaerge.getCpr());

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.MYNDLING,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForVaergeUdenCPR() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personVaerge = testHelper.createPerson(StamdataTestHelper.CPR_VAERGE);
        testHelper.createVaerge(person, null);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();

        assertEquals(0,personCustodyStructure.size());

        sendPersonRequest(personVaerge.getCpr());

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(0,personWardStructure.size());
    }

}
