/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.medcom;

import static junit.framework.Assert.assertNull;
import static org.junit.Assert.assertEquals;

import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.List;

import javax.xml.namespace.QName;

import org.hisrc.hifaces20.testing.webappenvironment.testing.junit4.AbstractWebAppEnvironmentJUnit4Test;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO.FMRTypeKode;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.SikredeVO;
import com.trifork.stamdata.models.YderVO;

import dk.nsi.stamdata.cpr.mapping.CivilRegistrationStatusCodes;
import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.RequestHelper;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import dk.nsi.stamdata.testing.TestHelper;
import oio.medcom.cprservice._1_0_4a.DetGodeCPROpslag;
import oio.medcom.cprservice._1_0_4a.DetGodeCPROpslagService;
import oio.medcom.cprservice._1_0_4a.GetPersonInformationIn;
import oio.medcom.cprservice._1_0_4a.GetPersonInformationOut;
import oio.medcom.cprservice._1_0_4a.GetPersonInformationWithHistoricIn;
import oio.medcom.cprservice._1_0_4a.GetPersonInformationWithHistoricOut;
import oio.medcom.cprservice._1_0_4a.GetPersonWithHealthCareInformationAndHistoricIn;
import oio.medcom.cprservice._1_0_4a.GetPersonWithHealthCareInformationAndHistoricOut;
import oio.medcom.cprservice._1_0_4a.GetPersonWithHealthCareInformationIn;
import oio.medcom.cprservice._1_0_4a.GetPersonWithHealthCareInformationOut;
import oio.medcom.cprservice._1_0_4a.PersonCustodyRelationType;
import oio.medcom.cprservice._1_0_4a.PersonCustodyStructureType;
import oio.medcom.cprservice._1_0_4a.PersonHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4a.PersonInformationStructureType;
import oio.medcom.cprservice._1_0_4a.PersonSpouseStructureType;
import oio.medcom.cprservice._1_0_4a.PersonWardRelationType;
import oio.medcom.cprservice._1_0_4a.PersonWardStructureType;
import oio.medcom.cprservice._1_0_4a.PersonWithHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4a.PublicHealthInsuranceGroupIdentifierType;

@RunWith(GuiceTestRunner.class)
public class DetGodeCPROpslag104aIntegrationTest extends AbstractWebAppEnvironmentJUnit4Test {

    public static final QName DET_GODE_CPR_OPSLAG_104A_SERVICE = new QName("urn:oio:medcom:cprservice:1.0.4a", "DetGodeCPROpslagService");
    public static final String WHITELISTED_CVR = "12345678";

    @Inject
    private Provider<Connection> connectionProvider;

    private DetGodeCPROpslag client;

    private GetPersonInformationIn request = new GetPersonInformationIn();
    private GetPersonInformationOut response;
    private GetPersonInformationWithHistoricIn requestWithHistoric = new GetPersonInformationWithHistoricIn();
    private GetPersonInformationWithHistoricOut responseWithHistoric;
    private GetPersonWithHealthCareInformationIn healthCareRequest = new GetPersonWithHealthCareInformationIn();
    private GetPersonWithHealthCareInformationOut healthCareResponse;
    private GetPersonWithHealthCareInformationAndHistoricIn healthCareRequestWithHistoric = new GetPersonWithHealthCareInformationAndHistoricIn();
    private GetPersonWithHealthCareInformationAndHistoricOut healthCareResponseWithHistoric;


    private StamdataTestHelper testHelper = new StamdataTestHelper();

    TestHelper helper = new TestHelper();

    @Inject
    Connection connection;

    @Before
    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    @Before
    public void setUp() throws Exception {
        URL wsdlLocation = new URL("http://localhost:8100/service/DetGodeCPROpslag-1.0.4a?wsdl");
        // Change to run test in docker
        //URL wsdlLocation = new URL("http://localhost:9090/stamdata-cpr-ws/service/DetGodeCPROpslag-1.0.4?wsdl");
        DetGodeCPROpslagService serviceCatalog =
                new DetGodeCPROpslagService(wsdlLocation, DET_GODE_CPR_OPSLAG_104A_SERVICE);
        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());
        client = serviceCatalog.getDetGodeCPROpslag();
    }

    private void sendPersonRequest(String cpr) throws Exception {
    	helper.enableAuditlogging(true);
    	helper.redirectSystemOutToStream();
        request.setPersonCivilRegistrationIdentifier(cpr);
        try {
            performRequest();
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();
        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4a getPersonInformation\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getPersonCivilRegistrationIdentifier()+"\"}";
        assertEquals(expectedLog, actualLog);
    }

    private void sendPersonRequestWithHistoric(String cpr) throws Exception {
        helper.enableAuditlogging(true);
        helper.redirectSystemOutToStream();
        requestWithHistoric.setPersonCivilRegistrationIdentifier(cpr);
        try {
            performRequestWithHistoric();
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();
        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4a getPersonInformationWithHistoric\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+requestWithHistoric.getPersonCivilRegistrationIdentifier()+"\"}";
        assertEquals(expectedLog, actualLog);
    }

    private void sendHealthCareRequest() throws Exception {
        helper.enableAuditlogging(true);
        helper.redirectSystemOutToStream();
        testHelper.prepareTestdata(connection);
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        try {
            healthCareResponse = client.getPersonWithHealthCareInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), healthCareRequest);
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();

        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4a getPersonWithHealthcareInformation\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+healthCareRequest.getPersonCivilRegistrationIdentifier()+"\"}";
        assertEquals(expectedLog, actualLog);
    }

    private void sendHealthCareRequest(String cpr) throws Exception {
        healthCareRequest.setPersonCivilRegistrationIdentifier(cpr);
        sendHealthCareRequest();
    }

    private void sendHealthCareRequestWithHistoric() throws Exception {
        helper.enableAuditlogging(true);
        helper.redirectSystemOutToStream();
        testHelper.prepareTestdata(connection);
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        try {
            healthCareResponseWithHistoric = client.getPersonWithHealthCareInformationAndHistoric(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), healthCareRequestWithHistoric);
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();

        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4a getPersonWithHealthcareInformationAndHistoric\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+healthCareRequestWithHistoric.getPersonCivilRegistrationIdentifier()+"\"}";
        assertEquals(expectedLog, actualLog);
    }

    private void sendHealthCareRequestWithHistoric(String cpr) throws Exception {
        healthCareRequestWithHistoric.setPersonCivilRegistrationIdentifier(cpr);
        sendHealthCareRequestWithHistoric();
    }

    private void performRequest() throws Exception {
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        response = client.getPersonInformation(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), request);
    }

    private void performRequestWithHistoric() throws Exception {
        SecurityWrapper securityHeaders = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo", "bar");
        responseWithHistoric = client.getPersonInformationWithHistoric(securityHeaders.getSecurity(), securityHeaders.getMedcomHeader(), requestWithHistoric);
    }

    @Test
    public void requestForPersonInformation() throws Exception {
        PersonVO person = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person);
        testHelper.createVaerge(personBarn4, person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());

        AssertResponse104a.assertPersonInformationStructure(response.getPersonInformationStructure());
    }

    @Test
    public void requestForPersonInformationWithProtection() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setNavnebeskyttelsestartdato(testHelper.formatSimpleDate("2000-01-01"));
        person.setNavnebeskyttelseslettedato(testHelper.formatSimpleDate("2050-01-01"));
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person);
        testHelper.createVaerge(personBarn4, person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());

        AssertResponse104a.assertPersonInformationStructureProtected(response.getPersonInformationStructure());
    }

    @Test
    public void requestForPersonWithHealthCareInformation() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person1, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person1);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person1);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person1, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person1, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person1);
        testHelper.createVaerge(personBarn4, person1);
        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        AssertResponse104a.assertPersonWithHealthCareInformationStructure(personWithHealthCareInformationStructure);
    }

    @Test
    public void requestForPersonWithHealthCareInformationWithProtection() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        person1.setNavnebeskyttelsestartdato(testHelper.formatSimpleDate("2000-01-01"));
        person1.setNavnebeskyttelseslettedato(testHelper.formatSimpleDate("2050-01-01"));
        PersonVO person2 = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person1, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(personFar, person1);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.MOR);
        testHelper.createBarnRelation(personMor, person1);
        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createBarnRelation(person1, personBarn1);
        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createBarnRelation(person1, personBarn2);
        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person1);
        testHelper.createVaerge(personBarn4, person1);
        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        AssertResponse104a.assertPersonWithHealthCareInformationStructureProtected(personWithHealthCareInformationStructure);
    }

    @Test
    public void requestWithGroupCode0() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();
        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "0", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        PersonHealthCareInformationStructureType personHealthCareInformationStructure = personWithHealthCareInformationStructure.getPersonHealthCareInformationStructure();
        AssertResponse104a.assertPersonHealthCareInformationStructure(personHealthCareInformationStructure, PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_0);
    }

    @Test
    public void requestForSeperatedPerson() throws Exception {
        PersonVO person = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createPerson(StamdataTestHelper.CPR_SPOUSE);
        testHelper.createSeperated(person, personSpouse);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        assertNull(personInformationStructure.getPersonSpouseStructure());
    }

    @Test
    public void requestForUnmarriedPerson() throws Exception {
        PersonVO person = testHelper.createPerson();
        testHelper.createUnmarried(person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        assertNull(personInformationStructure.getPersonSpouseStructure());
    }

    @Test
    public void requestForRPPerson() throws Exception {
        PersonVO person = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personSpouse = testHelper.createMand(StamdataTestHelper.CPR_MEDFAR);
        testHelper.createSpouse(person, personSpouse);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        PersonSpouseStructureType personSpouseStructure = personInformationStructure.getPersonSpouseStructure();
        assertEquals(StamdataTestHelper.CPR_MEDFAR,personSpouseStructure.getPersonSpouseIdentifier());
        assertEquals("registreret_partner",personSpouseStructure.getPersonSpouseRelation().value());
    }

    @Test
    public void testSignatureInvalidModelBuildException() throws Exception {

    		String fileName = "src/test/resources/data/validXml.xml";
    		String xml = RequestHelper.readFile(fileName);
    		String replication_endpoint = "http://localhost:8100/service/DetGodeCPROpslag-1.0.4";

    		String responseXml = RequestHelper.sendRequest(replication_endpoint, "http://rep.oio.dk/medcom.sundcom.dk/xml/wsdl/2007/06/28/getPersonInformation", xml);

    		String faultstring = RequestHelper.getFaultstring(responseXml);
    		assertEquals(faultstring.substring(0, 114), "The signature used to sign the message was incorrectly signed or no longer valid. Signature could not be validated");
    }

    @Test
    public void requestForCustodyAndenIndehaver() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);

        testHelper.createBarnRelation(personFar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR, personFar);

        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, personMor);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        assertEquals(2,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_FAR, PersonCustodyRelationType.FAR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.ANDEN);

        sendPersonRequest(StamdataTestHelper.CPR_FAR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForCustodyMedmor() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personMedmor = testHelper.createKvinde(StamdataTestHelper.CPR_MEDMOR);

        testHelper.createBarnRelation(personFar, person);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personMedmor, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR, personFar);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR, personMor);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        assertEquals(3,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.MOR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MEDMOR, PersonCustodyRelationType.MOR);
        personCustody = personCustodyStructure.get(2);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_FAR, PersonCustodyRelationType.FAR);

        sendPersonRequest(StamdataTestHelper.CPR_FAR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MEDMOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForCustodyMedfar() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personMedfar = testHelper.createMand(StamdataTestHelper.CPR_MEDFAR);

        testHelper.createBarnRelation(personFar, person);
        testHelper.createBarnRelation(personMor, person);
        testHelper.createBarnRelation(personMedfar, person);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR, personFar);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        assertEquals(3,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_FAR, PersonCustodyRelationType.FAR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MEDFAR, PersonCustodyRelationType.FAR);
        personCustody = personCustodyStructure.get(2);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.MOR_UDEN);

        sendPersonRequest(StamdataTestHelper.CPR_FAR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN_UDEN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MEDFAR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForVaerge() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personVaerge = testHelper.createPerson(StamdataTestHelper.CPR_VAERGE);
        testHelper.createVaerge(person, personVaerge);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();

        assertEquals(1,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse104a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_VAERGE, PersonCustodyRelationType.VAERGE);

        sendPersonRequest(personVaerge.getCpr());

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.MYNDLING,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForVaergeUdenCPR() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personVaerge = testHelper.createPerson(StamdataTestHelper.CPR_VAERGE);
        testHelper.createVaerge(person, null);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();

        assertEquals(0,personCustodyStructure.size());

        sendPersonRequest(personVaerge.getCpr());

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(0,personWardStructure.size());
    }

    @Test
    public void requestForDeadPersonInformation() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setStatus(CivilRegistrationStatusCodes.DEAD);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        AssertResponse104a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        AssertResponse104a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        AssertResponse104a.assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(), false, CivilRegistrationStatusCodes.DEAD);
        AssertResponse104a.assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),false, true);
    }

    @Test
    public void requestForDeadPersonInformationWithHistoric() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setStatus(CivilRegistrationStatusCodes.DEAD);

        testHelper.prepareTestdata(connection);
        sendPersonRequestWithHistoric(person.getCpr());
        PersonInformationStructureType personInformationStructure = responseWithHistoric.getPersonInformationStructure();

        AssertResponse104a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        AssertResponse104a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        AssertResponse104a.assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(), false, CivilRegistrationStatusCodes.DEAD);
        AssertResponse104a.assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),false, false);
    }

    @Test
    public void requestForDeadPersonInformationWithHealthCareInformation() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        person1.setStatus(CivilRegistrationStatusCodes.DEAD);
        PersonVO person2 = testHelper.createPerson();

        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));

        sendHealthCareRequest(person1.getCpr());
        PersonInformationStructureType personInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonInformationStructure();

        AssertResponse104a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        AssertResponse104a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        AssertResponse104a.assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(), false, CivilRegistrationStatusCodes.DEAD);
        AssertResponse104a.assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),false, true);
    }

    @Test
    public void requestForDeadPersonInformationWithHealthCareInformationAnndHistoric() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        person1.setStatus(CivilRegistrationStatusCodes.DEAD);
        PersonVO person2 = testHelper.createPerson();

        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));

        sendHealthCareRequestWithHistoric(person1.getCpr());
        PersonInformationStructureType personInformationStructure = healthCareResponseWithHistoric.getPersonWithHealthCareInformationStructure().getPersonInformationStructure();

        AssertResponse104a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        AssertResponse104a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        AssertResponse104a.assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(), false, CivilRegistrationStatusCodes.DEAD);
        AssertResponse104a.assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),false, false);
    }

}
