/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.pvit;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertThat;

import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.ws.Holder;

import org.hisrc.hifaces20.testing.webappenvironment.testing.junit4.AbstractWebAppEnvironmentJUnit4Test;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.common.collect.Lists;
import com.google.inject.Inject;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.PersonVO;

import dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType;
import dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType;
import dk.nsi._2018._06._08.stamdatacpr.DGWSFault;
import dk.nsi._2018._06._08.stamdatacpr.StamdataPersonLookup;
import dk.nsi._2018._06._08.stamdatacpr.StamdataPersonLookupService;
import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.jaxws.generated.Header;
import dk.nsi.stamdata.jaxws.generated.Security;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.sosi.seal.model.AuthenticationLevel;

@RunWith(GuiceTestRunner.class)
public class StamdataPersonLookupIdCardLevelAttackTest extends AbstractWebAppEnvironmentJUnit4Test {
    public static final QName PVIT_SERVICE_QNAME = new QName("http://nsi.dk/2018/06/08/StamdataCpr/",
            "StamdataPersonLookupService");

    private List<PersonVO> persons = Lists.newArrayList();
    private PersonLookupRequestType request = new PersonLookupRequestType();
    private PersonLookupResponseType response;

    public static final String WHITELISTED_CVR = "12345678";

    private static StamdataPersonLookup client;
    private static StamdataPersonLookupService serviceCatalog;

    @Inject
    Connection connection;

    @Before
    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    @Before
    public void setUp() throws Exception {

        URL wsdlLocation = new URL("http://localhost:8100/service/StamdataPersonLookup-1.1.0?wsdl");
        serviceCatalog = new StamdataPersonLookupService(wsdlLocation, PVIT_SERVICE_QNAME);

        // SEAL enforces that the XML prefixes are exactly
        // as it creates them. So we have to make sure we
        // don't change them.

        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());

        client = serviceCatalog.getStamdataPersonLookup();
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelOneIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(1);
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelTwoIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(2);
    }

    @Test
    public void requestWithLevelthreeIdCardDoesNotRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(3);
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelFourIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(4);
    }

    private void sendRequestWithGivenSecurityLevel(int levelOfIdCard) throws DGWSFault, Exception {
        persons.add(Factories.createPersonWithCPR("2905853347"));

        request.setCivilRegistrationNumberPersonQuery("0103952595");

        sendRequest(levelOfIdCard);

        assertThat(response.getPersonInformationStructure().size(), is(0));
    }

    private static Map<Integer, AuthenticationLevel> mapFromIntegerLevelToEnum;
    static {
        mapFromIntegerLevelToEnum = new HashMap<Integer, AuthenticationLevel>();

        mapFromIntegerLevelToEnum.put(AuthenticationLevel.NO_AUTHENTICATION.getLevel(),
                AuthenticationLevel.NO_AUTHENTICATION);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.USERNAME_PASSWORD_AUTHENTICATION.getLevel(),
                AuthenticationLevel.USERNAME_PASSWORD_AUTHENTICATION);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.VOCES_TRUSTED_SYSTEM.getLevel(),
                AuthenticationLevel.VOCES_TRUSTED_SYSTEM);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.MOCES_TRUSTED_USER.getLevel(),
                AuthenticationLevel.MOCES_TRUSTED_USER);
    }

    private void sendRequest(int levelOfIdCard) throws Exception, DGWSFault {
        Holder<Security> securityHeader;
        Holder<Header> medcomHeader;

        SecurityWrapper secutityHeadersWhitelisted = DGWSHeaderUtil.getSecurityWrapper(
                mapFromIntegerLevelToEnum.get(levelOfIdCard), WHITELISTED_CVR, "foo", "bar");
        securityHeader = secutityHeadersWhitelisted.getSecurity();
        medcomHeader = secutityHeadersWhitelisted.getMedcomHeader();

        response = client.getPersonDetails(securityHeader, medcomHeader, request);
    }
}
