/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.pvit;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.List;

import javax.inject.Inject;
import javax.xml.namespace.QName;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import org.hisrc.hifaces20.testing.webappenvironment.testing.junit4.AbstractWebAppEnvironmentJUnit4Test;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.common.collect.Lists;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.PersonVO;

import dk.nsi._2011._09._23.stamdatacpr.NamePersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType;
import dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType;
import dk.nsi._2014._03._01.stamdatacpr.StamdataPersonLookup;
import dk.nsi._2014._03._01.stamdatacpr.StamdataPersonLookupService;
import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.jaxws.generated.Header;
import dk.nsi.stamdata.jaxws.generated.Security;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import dk.nsi.stamdata.testing.TestHelper;

@RunWith(GuiceTestRunner.class)
public class StamdataPersonLookupIntegration101Test  extends AbstractWebAppEnvironmentJUnit4Test {
    public static final QName PVIT_SERVICE_QNAME = new QName("http://nsi.dk/2014/03/01/StamdataCpr/", "StamdataPersonLookupService");

    private List<PersonVO> persons = Lists.newArrayList();
    private PersonLookupRequestType request = new PersonLookupRequestType();
    private PersonLookupResponseType response;
    TestHelper helper = new TestHelper();

    public static final String WHITELISTED_CVR = "12345678";

    private static StamdataPersonLookup client;
    private static StamdataPersonLookupService serviceCatalog;

    @Inject
    Connection connection;

    @Before
    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    @Before
    public void setUp() throws Exception {

        URL wsdlLocation = new URL("http://localhost:8100/service/StamdataPersonLookup-1.0.1?wsdl");
        serviceCatalog = new StamdataPersonLookupService(wsdlLocation, PVIT_SERVICE_QNAME);

        // SEAL enforces that the XML prefixes are exactly
        // as it creates them. So we have to make sure we
        // don't change them.
        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());
        client = serviceCatalog.getStamdataPersonLookup();
    }

    @Test(expected = SOAPFaultException.class)
    public void requestWithoutAnyQueryTypeGivesSenderSoapFault() throws Exception {
        request = new PersonLookupRequestType();

        prepareDatabaseAndSendRequest();
    }

    @Test(expected = SOAPFaultException.class)
    public void requestWithTwoQueryTypeGivesSenderSoapFault() throws Exception {
        request.setCivilRegistrationNumberPersonQuery("2805842569");

        NamePersonQueryType namePersonQueryType = new NamePersonQueryType();
        namePersonQueryType.setPersonGivenName("Thomas");
        namePersonQueryType.setPersonMiddleName("Greve");
        namePersonQueryType.setPersonSurnameName("Kristensen");
        request.setNamePersonQuery(namePersonQueryType);

        prepareDatabaseAndSendRequest();
    }


    @Test
    public void requestWithACprNumberNotPresentInDatabaseReturnsNothing() throws Exception {
        persons.add(Factories.createPersonWithCPR("2905853347"));

        request.setCivilRegistrationNumberPersonQuery("0103952595");

        prepareDatabaseAndSendRequest();

        assertThat(response.getPersonInformationStructure().size(), is(0));

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.1 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getCivilRegistrationNumberPersonQuery()+"\"}", logs[0]);
    }

    @Test
    public void requestWithoutFoedselsdatoMarkeringV1() throws Exception {
        PersonVO person = Factories.createPersonWithCPR("0103952595");
        person.setFoedselsdatoMarkering("0");
        persons.add(person);
        request.setCivilRegistrationNumberPersonQuery(person.getCpr());
        prepareDatabaseAndSendRequest();
        assertFalse(response.getPersonInformationStructure().get(0).getRegularCPRPerson().getPersonBirthDateStructure().
                isBirthDateUncertaintyIndicator());
    }

    @Test
    public void requestWithoutFoedselsdatoMarkeringV2() throws Exception {
        PersonVO person = Factories.createPersonWithCPR("0103952595");
        person.setFoedselsdatoMarkering(null);
        persons.add(person);
        request.setCivilRegistrationNumberPersonQuery(person.getCpr());
        prepareDatabaseAndSendRequest();
        assertFalse(response.getPersonInformationStructure().get(0).getRegularCPRPerson().getPersonBirthDateStructure().
                isBirthDateUncertaintyIndicator());
    }

    @Test
    public void requestWithFoedselsdatoMarkeringV1() throws Exception {
        PersonVO person = Factories.createPersonWithCPR("0103952595");
        person.setFoedselsdatoMarkering("1");
        persons.add(person);
        request.setCivilRegistrationNumberPersonQuery(person.getCpr());
        prepareDatabaseAndSendRequest();
        assertTrue(response.getPersonInformationStructure().get(0).getRegularCPRPerson().getPersonBirthDateStructure().isBirthDateUncertaintyIndicator());
    }

    @Test
    public void requestWithFoedselsdatoMarkeringV2() throws Exception {
        PersonVO person = Factories.createPersonWithCPR("2905853347");
        person.setFoedselsdatoMarkering("*");
        persons.add(person);
        request.setCivilRegistrationNumberPersonQuery(person.getCpr());
        prepareDatabaseAndSendRequest();
        assertTrue(response.getPersonInformationStructure().get(0).getRegularCPRPerson().getPersonBirthDateStructure().
                isBirthDateUncertaintyIndicator());
    }

    @Test
    public void testUnreliableBirthDay() throws Exception {
        PersonVO person = Factories.createPersonWithCPR("2905853347");
        persons.add(person);
        person.setUnreliable("Foedselsdato");
        request.setCivilRegistrationNumberPersonQuery(person.getCpr());
        prepareDatabaseAndSendRequest();
        assertEquals(631148400000L, response.getPersonInformationStructure().get(0).getRegularCPRPerson().
                getPersonBirthDateStructure().getBirthDate().toGregorianCalendar().getTime().getTime());
    }

    @Test(expected = SOAPFaultException.class)
    public void testUnreliableStatus() throws Exception {
        PersonVO person = Factories.createPersonWithCPR("2905853347");
        persons.add(person);
        person.setUnreliable("Status");
        request.setCivilRegistrationNumberPersonQuery(person.getCpr());
        prepareDatabaseAndSendRequest();
    }


    @Test
    // This is not possible on the old 1.0.0 ws
    public void requestPersonWithoutStreetBuilding() throws Exception {
        PersonVO person = Factories.createPerson();
        person.setHusnummer(null);
        persons.add(person);

        request.setCivilRegistrationNumberPersonQuery(person.getCpr());
        prepareDatabaseAndSendRequest();

        String streetBuildingIdentifier = response.getPersonInformationStructure().get(0).getPersonAddressStructure().getAddressComplete().getAddressPostal().getStreetBuildingIdentifier();
        assertNull(streetBuildingIdentifier);
    }

    private void prepareDatabaseAndSendRequest() throws Exception {
        connection.createStatement().executeUpdate("DELETE FROM v2_Person");
        connection.commit();
        for (PersonVO person : persons) {
            StamdataTestHelper.persist(connection, person);
        }

        sendRequest();
    }

    private void sendRequest() throws Exception {
    	helper.enableAuditlogging(true);
    	ByteArrayOutputStream outStream = helper.redirectSystemOutToStream();
        Holder<Security> securityHeader;
        Holder<Header> medcomHeader;

        SecurityWrapper secutityHeadersNotWhitelisted = DGWSHeaderUtil.
                getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo2", "bar2");
        securityHeader = secutityHeadersNotWhitelisted.getSecurity();
        medcomHeader = secutityHeadersNotWhitelisted.getMedcomHeader();

        response = client.getPersonDetails(securityHeader, medcomHeader, request);

        helper.enableAuditlogging(false);
        helper.resetSystemOut();
    }

}
