/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.pvit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;

import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.inject.Inject;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.Holder;

import org.hisrc.hifaces20.testing.webappenvironment.testing.junit4.AbstractWebAppEnvironmentJUnit4Test;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.common.collect.Lists;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.PersonVO;

import dk.nsi._2011._09._23.stamdatacpr.CivilRegistrationNumberListPersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.NamePersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType;
import dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType;
import dk.nsi._2018._06._08.stamdatacpr.StamdataPersonLookup;
import dk.nsi._2018._06._08.stamdatacpr.StamdataPersonLookupService;
import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.jaxws.generated.Header;
import dk.nsi.stamdata.jaxws.generated.Security;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import dk.nsi.stamdata.testing.TestHelper;
import dk.oio.rep.cpr_dk.xml.schemas.core._2006._01._17.RegularCPRPersonType;
import dk.oio.rep.itst_dk.xml.schemas._2006._01._17.PersonNameStructureType;
import oio.medcom.cprservice._1_0.PersonAddressStructureType;
import oio.medcom.cprservice._1_0.PersonInformationStructureType;

@RunWith(GuiceTestRunner.class)
public class StamdataPersonLookupIntegration110Test  extends AbstractWebAppEnvironmentJUnit4Test {
    public static final QName PVIT_SERVICE_QNAME = new QName("http://nsi.dk/2018/06/08/StamdataCpr/", "StamdataPersonLookupService");

    private List<PersonVO> persons = Lists.newArrayList();
    private PersonLookupRequestType request = new PersonLookupRequestType();
    private PersonLookupRequestType requestWithConsent = new PersonLookupRequestType();
    private PersonLookupResponseType response;

    TestHelper helper = new TestHelper();

    public static final String WHITELISTED_CVR = "12345678";

    private static StamdataPersonLookup client;
    private static StamdataPersonLookupService serviceCatalog;

    SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
    SimpleDateFormat birthdateFormatter = new SimpleDateFormat("yyyy-MM-ddXXX");

    @Inject
    Connection connection;

    @Before
    public void setup() throws Exception {
        //setUpDB();
        setUpClient();
    }

    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='SDMwithConsent', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    public void setUpClient() throws Exception {

        //URL wsdlLocation = new URL("http://localhost:9090/stamdata-cpr-ws/service/StamdataPersonLookup-1.1.0?wsdl");
        URL wsdlLocation = new URL("http://localhost:8100/service/StamdataPersonLookup-1.1.0?wsdl");
        serviceCatalog = new StamdataPersonLookupService(wsdlLocation, PVIT_SERVICE_QNAME);

        // SEAL enforces that the XML prefixes are exactly
        // as it creates them. So we have to make sure we
        // don't change them.
        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());
        client = serviceCatalog.getStamdataPersonLookup();
    }

    @Test
    public void requestCPRprotectedWithoutConsent1() throws Exception {
        PersonVO person = addPerson("0103952595");
        request.setCivilRegistrationNumberPersonQuery("0103952595");
        prepareDatabaseAndSendRequest();
        assertResponseWithProtectedAddress();
        assertCpr(person);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getCivilRegistrationNumberPersonQuery()+"\"}", logs[0]);
    }

    @Test
    public void requestCPRprotectedWithoutConsent2() throws Exception {
        PersonVO person = addPerson("0103952595");
        addPerson("0103952596");

        CivilRegistrationNumberListPersonQueryType cprList = new CivilRegistrationNumberListPersonQueryType();
        cprList.getCivilRegistrationNumber().add("0103952595");
        cprList.getCivilRegistrationNumber().add("0103952596");
        request.setCivilRegistrationNumberListPersonQuery(cprList);
        prepareDatabaseAndSendRequest();
        assertResponseWithProtectedAddress(0);
        assertResponseWithProtectedAddress(1);
        assertCpr(person);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr list\",\"value\":\""+request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber()+ "\"}", logs[0]);
    }

    @Test
    public void requestCPRprotectedWithoutConsent3() throws Exception {
        addPerson("0103952595");
        GregorianCalendar cal = new GregorianCalendar();
        cal.setTime(persons.get(0).getFoedselsdato());
        XMLGregorianCalendar birthday = DatatypeFactory.newInstance().newXMLGregorianCalendar(cal);
        request.setBirthDatePersonQuery(birthday);
        prepareDatabaseAndSendRequest();
        assertResponseWithProtectedAddress();
        assertCpr("0000000000",null);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"birthdate\",\"value\":\""+birthdateFormatter.format(request.getBirthDatePersonQuery().toGregorianCalendar().getTime())+"\"}", logs[0]);
    }

    @Test
    public void requestCPRprotectedWithoutConsent4() throws Exception {
        addPerson("0103952595");
        NamePersonQueryType name = new NamePersonQueryType();
        name.setPersonGivenName("Peter");
        name.setPersonMiddleName("Sigurd");
        name.setPersonSurnameName("Andersen");
        request.setNamePersonQuery(name);
        prepareDatabaseAndSendRequest();
        assertResponseWithProtectedAddress();
        assertCpr("0000000000",null);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"name\",\"value\":\""+request.getNamePersonQuery().getPersonGivenName()+" "+request.getNamePersonQuery().getPersonMiddleName()+" "+request.getNamePersonQuery().getPersonSurnameName()+"\"}", logs[0]);
    }

    @Test
    public void requestCPRprotectedWithoutMiddlenameWithoutConsent() throws Exception {
        PersonVO person = addPerson("0103952595");
        person.setMellemnavn("");
        NamePersonQueryType name = new NamePersonQueryType();
        name.setPersonGivenName("Peter");
        name.setPersonSurnameName("Andersen");
        request.setNamePersonQuery(name);
        prepareDatabaseAndSendRequest();
        assertResponseWithProtectedAddress();
        assertCpr("0000000000",null);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"name\",\"value\":\""+request.getNamePersonQuery().getPersonGivenName()+" "+request.getNamePersonQuery().getPersonMiddleName()+" "+request.getNamePersonQuery().getPersonSurnameName()+"\"}", logs[0]);
    }

    @Test
    public void requestCPROutdatedProtectedWithConsent() throws Exception {
        addPersonWithOutdatedProtection("0103952595");
        request.setCivilRegistrationNumberPersonQuery("0103952595");
        prepareDatabaseAndSendRequest(true);
        assertResponseWithFullAddress(false);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetailsWithConsent\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getCivilRegistrationNumberPersonQuery()+"\"}", logs[0]);
    }

    @Test
    public void requestCPROutdatedProtectedWithoutConsent() throws Exception {
        addPersonWithOutdatedProtection("0103952595");
        request.setCivilRegistrationNumberPersonQuery("0103952595");
        prepareDatabaseAndSendRequest();
        assertResponseWithFullAddress(false);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getCivilRegistrationNumberPersonQuery()+"\"}", logs[0]);
    }

    @Test
    public void requestCPRprotectedWithConsent1() throws Exception {
        PersonVO person = addPerson("0103952595");
        request.setCivilRegistrationNumberPersonQuery("0103952595");
        prepareDatabaseAndSendRequest(true);
        assertResponseWithFullAddress();
        assertCpr(person);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetailsWithConsent\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getCivilRegistrationNumberPersonQuery()+"\"}", logs[0]);
    }

    @Test
    public void requestCPRprotectedWithConsent2() throws Exception {
        PersonVO person = addPerson("0103952595");
        addPerson("0103952596");

        CivilRegistrationNumberListPersonQueryType cprList = new CivilRegistrationNumberListPersonQueryType();
        cprList.getCivilRegistrationNumber().add("0103952595");
        cprList.getCivilRegistrationNumber().add("0103952596");
        request.setCivilRegistrationNumberListPersonQuery(cprList);
        prepareDatabaseAndSendRequest(true);
        assertResponseWithFullAddress(0);
        assertResponseWithFullAddress(1);
        assertCpr(person);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetailsWithConsent\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr list\",\"value\":\""+request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber()+ "\"}", logs[0]);
    }

    @Test
    public void requestCPRprotectedWithConsent3() throws Exception {
        addPerson("0103952595");
        GregorianCalendar cal = new GregorianCalendar();
        cal.setTime(persons.get(0).getFoedselsdato());
        XMLGregorianCalendar birthday = DatatypeFactory.newInstance().newXMLGregorianCalendar(cal);
        request.setBirthDatePersonQuery(birthday);
        prepareDatabaseAndSendRequest(true);
        assertResponseWithFullAddress();
        assertCpr("0000000000",null);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetailsWithConsent\",\"type\":\"SensitivePersonalInformation\",\"key\":\"birthdate\",\"value\":\""+birthdateFormatter.format(request.getBirthDatePersonQuery().toGregorianCalendar().getTime())+"\"}", logs[0]);
    }

    private void assertCpr(PersonVO person) {
        assertCpr(person.getCpr(),person.getGaeldendeCPR());
    }
    private void assertCpr(String cpr, String gaeldendeCpr) {
        assertString(cpr,response.getPersonInformationStructure().get(0).getRegularCPRPerson().getSimpleCPRPerson().getPersonCivilRegistrationIdentifier());
        assertString(gaeldendeCpr,response.getPersonInformationStructure().get(0).getCurrentPersonCivilRegistrationIdentifier());
    }

    @Test
    public void requestCPRprotectedWithConsent4() throws Exception {
        addPerson("0103952595");
        NamePersonQueryType name = new NamePersonQueryType();
        name.setPersonGivenName("Peter");
        name.setPersonMiddleName("Sigurd");
        name.setPersonSurnameName("Andersen");
        request.setNamePersonQuery(name);
        prepareDatabaseAndSendRequest(true);
        assertResponseWithFullAddress();
        assertCpr("0000000000",null);

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetailsWithConsent\",\"type\":\"SensitivePersonalInformation\",\"key\":\"name\",\"value\":\""+request.getNamePersonQuery().getPersonGivenName()+" "+request.getNamePersonQuery().getPersonMiddleName()+" "+request.getNamePersonQuery().getPersonSurnameName()+"\"}", logs[0]);
    }

    private PersonVO addPerson(String cpr) throws ParseException {
        PersonVO person = Factories.createPersonWithCPR(cpr);
        person.setNavnebeskyttelsestartdato(dateFormatter.parse("2000-01-01"));
        person.setNavnebeskyttelseslettedato(dateFormatter.parse("2050-01-01"));
        persons.add(person);
        return person;
    }

    private PersonVO addPersonWithOutdatedProtection(String cpr) throws ParseException {
        PersonVO person = Factories.createPersonWithCPR(cpr);
        person.setNavnebeskyttelsestartdato(dateFormatter.parse("2000-01-01"));
        person.setNavnebeskyttelseslettedato(dateFormatter.parse("2010-01-01"));
        persons.add(person);
        return person;
    }

    private void assertResponseWithProtectedAddress() {
        assertResponseWithProtectedAddress(0, true);
    }

    private void assertResponseWithProtectedAddress(boolean piprotected) {
        assertResponseWithProtectedAddress(0, piprotected);
    }

    private void assertResponseWithProtectedAddress(int index) {
        assertResponseWithProtectedAddress(index, true);
    }

    private void assertResponseWithProtectedAddress(int index, boolean piprotected) {
        PersonInformationStructureType personInformationStructure = assertPersonInformantionProtected(index, piprotected);
        PersonAddressStructureType personAddressStructure = assertPersonInformantionWithProtectedName(personInformationStructure);
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getCountryIdentificationCode());
        assertString("ADRESSEBESKYTTET",personAddressStructure.getAddressComplete().getAddressPostal().getDistrictName());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getDistrictSubdivisionIdentifier());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getFloorIdentifier());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getMailDeliverySublocationIdentifier());
        assertString("0000",personAddressStructure.getAddressComplete().getAddressPostal().getPostCodeIdentifier());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getPostOfficeBoxIdentifier());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getStreetBuildingIdentifier());
        assertString("ADRESSEBESKYTTET",personAddressStructure.getAddressComplete().getAddressPostal().getStreetName());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getStreetNameForAddressingName());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getSuiteIdentifier());
        assertString("0000",personAddressStructure.getCountyCode());
        assertString(null,personAddressStructure.getCareOfName());
    }

    private void assertString(String expected, Object actual) {
        if (expected == null) {
            assertNull((actual == null) ? "" : "Excepted null but got "+actual.toString(), actual);
        } else {
            assertEquals(expected, actual);
        }
    }

    @Test
    public void requestCPRunprotectedWithoutConsent() throws Exception {
        PersonVO person = Factories.createPersonWithCPR("0103952595");
        persons.add(person);

        request.setCivilRegistrationNumberPersonQuery("0103952595");
        prepareDatabaseAndSendRequest(false);

        PersonInformationStructureType personInfoStructure = response.getPersonInformationStructure().get(0);
        assertFalse(personInfoStructure.getRegularCPRPerson().isPersonInformationProtectionIndicator());

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getCivilRegistrationNumberPersonQuery()+"\"}", logs[0]);
    }

    @Test
    public void requestCPRunprotectedWithConsent() throws Exception {
        PersonVO person = Factories.createPersonWithCPR("0103952595");
        persons.add(person);

        request.setCivilRegistrationNumberPersonQuery("0103952595");
        prepareDatabaseAndSendRequest(true);

        PersonInformationStructureType personInfoStructure = response.getPersonInformationStructure().get(0);
        assertFalse(personInfoStructure.getRegularCPRPerson().isPersonInformationProtectionIndicator());

        String[] logs = helper.getSystemLogs();
        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.1.0 getPersonDetailsWithConsent\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+request.getCivilRegistrationNumberPersonQuery()+"\"}", logs[0]);
    }

    private void assertResponseWithFullAddress() {
        assertResponseWithFullAddress(true);
    }

    private void assertResponseWithFullAddress(boolean piprotected) {
        assertResponseWithFullAddress(0, piprotected);
    }

    private void assertResponseWithFullAddress(int index) {
        assertResponseWithFullAddress(index, true);
    }

    private void assertResponseWithFullAddress(int index, boolean piprotected) {
        PersonInformationStructureType personInformationStructure = assertPersonInformantionProtected(index, piprotected);
        PersonAddressStructureType personAddressStructure = assertPersonInformantionWithFullName(personInformationStructure);
        assertString("DK",personAddressStructure.getAddressComplete().getAddressPostal().getCountryIdentificationCode().getValue());
        assertString("Überwald",personAddressStructure.getAddressComplete().getAddressPostal().getDistrictName());
        assertString("BirkeBy",personAddressStructure.getAddressComplete().getAddressPostal().getDistrictSubdivisionIdentifier());
        assertString("12",personAddressStructure.getAddressComplete().getAddressPostal().getFloorIdentifier());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getMailDeliverySublocationIdentifier());
        assertString("6666",personAddressStructure.getAddressComplete().getAddressPostal().getPostCodeIdentifier());
        assertString(null,personAddressStructure.getAddressComplete().getAddressPostal().getPostOfficeBoxIdentifier());
        assertString("10",personAddressStructure.getAddressComplete().getAddressPostal().getStreetBuildingIdentifier());
        assertString("Ørstedgade",personAddressStructure.getAddressComplete().getAddressPostal().getStreetName());
        assertString("Østergd.",personAddressStructure.getAddressComplete().getAddressPostal().getStreetNameForAddressingName());
        assertString("tv",personAddressStructure.getAddressComplete().getAddressPostal().getSuiteIdentifier());
        assertString("1083",personAddressStructure.getCountyCode());
        assertString("Søren Petersen",personAddressStructure.getCareOfName());
    }

    private PersonInformationStructureType assertPersonInformantionProtected(int index, boolean piprotected) {
        PersonInformationStructureType personInfoStructure = response.getPersonInformationStructure().get(index);
        boolean personInformationProtectionIndicator = personInfoStructure.getRegularCPRPerson().isPersonInformationProtectionIndicator();
        assertEquals(piprotected, personInformationProtectionIndicator);
        PersonAddressStructureType personAddressStructure = personInfoStructure.getPersonAddressStructure();
        Date protectionStartDate = personAddressStructure.getPersonInformationProtectionStartDate().toGregorianCalendar().getTime();
        assertEquals("2000-01-01",dateFormatter.format(protectionStartDate));
        return personInfoStructure;
    }

    private PersonAddressStructureType assertPersonInformantionWithFullName(PersonInformationStructureType personInfoStructure) {
        RegularCPRPersonType regularCPRPerson = personInfoStructure.getRegularCPRPerson();
        PersonNameStructureType personNameStructure = regularCPRPerson.getSimpleCPRPerson().getPersonNameStructure();
        assertString("Peter",personNameStructure.getPersonGivenName());
        assertString("Sigurd",personNameStructure.getPersonMiddleName());
        assertString("Andersen",personNameStructure.getPersonSurnameName());
        assertString("Peter,Andersen",regularCPRPerson.getPersonNameForAddressingName());
        PersonAddressStructureType personAddressStructure = personInfoStructure.getPersonAddressStructure();
        Date protectionStartDate = personAddressStructure.getPersonInformationProtectionStartDate().toGregorianCalendar().getTime();
        assertEquals("2000-01-01",dateFormatter.format(protectionStartDate));
        return personAddressStructure;
    }

    private PersonAddressStructureType assertPersonInformantionWithProtectedName(PersonInformationStructureType personInfoStructure) {
        RegularCPRPersonType regularCPRPerson = personInfoStructure.getRegularCPRPerson();
        PersonNameStructureType personNameStructure = regularCPRPerson.getSimpleCPRPerson().getPersonNameStructure();
        assertString("ADRESSEBESKYTTET",personNameStructure.getPersonGivenName());
        assertString(null,personNameStructure.getPersonMiddleName());
        assertString("ADRESSEBESKYTTET",personNameStructure.getPersonSurnameName());
        assertString("ADRESSEBESKYTTET",regularCPRPerson.getPersonNameForAddressingName());
        PersonAddressStructureType personAddressStructure = personInfoStructure.getPersonAddressStructure();
        Date protectionStartDate = personAddressStructure.getPersonInformationProtectionStartDate().toGregorianCalendar().getTime();
        assertEquals("2000-01-01",dateFormatter.format(protectionStartDate));
        return personAddressStructure;
    }

    private void prepareDatabaseAndSendRequest() throws Exception {
        prepareDatabaseAndSendRequest(false);
    }

    private void prepareDatabaseAndSendRequest(boolean withConsent) throws Exception {
        connection.createStatement().executeUpdate("DELETE FROM v2_Person");
        for (PersonVO person : persons) {
            StamdataTestHelper.persist(connection, person);
        }
        connection.commit();

        sendRequest(withConsent);
    }

    private void sendRequest(boolean withConsent) throws Exception {
    	helper.enableAuditlogging(true);
    	ByteArrayOutputStream outStream = helper.redirectSystemOutToStream();

        Holder<Security> securityHeader;
        Holder<Header> medcomHeader;

        SecurityWrapper secutityHeadersNotWhitelisted = DGWSHeaderUtil.
                getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo2", "bar2");
        securityHeader = secutityHeadersNotWhitelisted.getSecurity();
        medcomHeader = secutityHeadersNotWhitelisted.getMedcomHeader();

        if (withConsent) {
            response = client.getPersonDetailsWithConsent(securityHeader, medcomHeader, request);
        } else {
            response = client.getPersonDetails(securityHeader, medcomHeader, request);
        }

        helper.enableAuditlogging(false);
        helper.resetSystemOut();
    }
}
