/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.pvit;

import static dk.nsi.stamdata.testing.Factories.TWO_DAYS_AGO;
import static dk.nsi.stamdata.testing.Factories.YESTERDAY;
import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import org.hisrc.hifaces20.testing.webappenvironment.testing.junit4.AbstractWebAppEnvironmentJUnit4Test;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.common.collect.Lists;
import com.google.inject.Inject;
import com.trifork.stamdata.Nullable;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.models.PersonVO;

import dk.nsi._2011._09._23.stamdatacpr.CivilRegistrationNumberListPersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.NamePersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType;
import dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType;
import dk.nsi._2018._06._08.stamdatacpr.StamdataPersonLookup;
import dk.nsi._2018._06._08.stamdatacpr.StamdataPersonLookupService;
import dk.nsi.stamdata.cpr.mapping.v100.PersonMapper;
import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.jaxws.generated.Header;
import dk.nsi.stamdata.jaxws.generated.Security;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.StamdataTestHelper;


@RunWith(GuiceTestRunner.class)
public class StamdataPersonLookupIntegrationTest extends AbstractWebAppEnvironmentJUnit4Test {
    public static final QName PVIT_SERVICE_QNAME = new QName("http://nsi.dk/2018/06/08/StamdataCpr/", "StamdataPersonLookupService");

    private List<PersonVO> persons = Lists.newArrayList();
    private PersonLookupRequestType request = new PersonLookupRequestType();
    private PersonLookupResponseType response;

    public static final String WHITELISTED_CVR = "12345678";

    private static StamdataPersonLookup client;
    private static StamdataPersonLookupService serviceCatalog;

    @Inject
    Connection connection;

    @Before
    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    @Before
    public void setUp() throws Exception {

        URL wsdlLocation = new URL("http://localhost:8100/service/StamdataPersonLookup-1.1.0?wsdl");
        serviceCatalog = new StamdataPersonLookupService(wsdlLocation, PVIT_SERVICE_QNAME);

        // SEAL enforces that the XML prefixes are exactly
        // as it creates them. So we have to make sure we
        // don't change them.

        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());

        client = serviceCatalog.getStamdataPersonLookup();
    }


    @Test(expected = SOAPFaultException.class)
    public void requestWithoutAnyQueryTypeGivesSenderSoapFault() throws Exception {
        request = new PersonLookupRequestType();

        prepareDatabaseAndSendRequest();
    }


    @Test(expected = SOAPFaultException.class)
    public void requestWithTwoQueryTypeGivesSenderSoapFault() throws Exception {
        request.setCivilRegistrationNumberPersonQuery("2805842569");

        NamePersonQueryType namePersonQueryType = new NamePersonQueryType();
        namePersonQueryType.setPersonGivenName("Thomas");
        namePersonQueryType.setPersonMiddleName("Greve");
        namePersonQueryType.setPersonSurnameName("Kristensen");
        request.setNamePersonQuery(namePersonQueryType);

        prepareDatabaseAndSendRequest();
    }

    @Test
    public void requestWithACprNumberNotPresentInDatabaseReturnsNothing() throws Exception {
        persons.add(Factories.createPersonWithCPR("2905853347"));

        request.setCivilRegistrationNumberPersonQuery("0103952595");

        prepareDatabaseAndSendRequest();

        assertThat(response.getPersonInformationStructure().size(), is(0));
    }


    @Test
    public void requestWithACprNumberPresentInDatabase() throws Exception {
        PersonVO person = Factories.createPerson();

        persons.add(person);
        persons.add(Factories.createPerson());

        request.setCivilRegistrationNumberPersonQuery(person.getCpr());

        prepareDatabaseAndSendRequest();

        assertThat(response.getPersonInformationStructure().size(), is(1));
    }


    @Test
    public void requestWithSeveralCprNumbersNoneOfWhichAreInTheDatabase() throws Exception {
        persons.add(Factories.createPersonWithCPR("0000000000"));

        request.setCivilRegistrationNumberListPersonQuery(new CivilRegistrationNumberListPersonQueryType());
        request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber().add("0206562469");
        request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber().add("0302801961");

        prepareDatabaseAndSendRequest();

        assertThat(response.getPersonInformationStructure().size(), is(0));
    }


    @Test
    public void requestWithSeveralCprNumbersOfWhichSomeAreInTheDatabase() throws Exception {
        String EXISTING_CPR_1 = "0302801961";
        String EXISTING_CPR_2 = "0905852363";
        String NON_EXISTING_CPR = "0405852364";

        persons.add(Factories.createPersonWithCPR(EXISTING_CPR_1));
        persons.add(Factories.createPersonWithCPR(EXISTING_CPR_2));

        request.setCivilRegistrationNumberListPersonQuery(new CivilRegistrationNumberListPersonQueryType());
        request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber().add(EXISTING_CPR_1);
        request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber().add(EXISTING_CPR_2);
        request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber().add(NON_EXISTING_CPR);

        prepareDatabaseAndSendRequest();

        assertEquals(2, response.getPersonInformationStructure().size());
    }


    @Test
    public void requestWithBirthDateNotFoundInDatabase() throws Exception {
        persons.add(Factories.createPersonWithBirthday(TWO_DAYS_AGO));

        XMLGregorianCalendar REQUESTED_BIRTHDAY = PersonMapper.newXMLGregorianCalendar(YESTERDAY);
        request.setBirthDatePersonQuery(REQUESTED_BIRTHDAY);

        prepareDatabaseAndSendRequest();

        assertTrue(response.getPersonInformationStructure().isEmpty());
    }


    @Test
    public void requestWithBirthDateFoundInDatabaseSeveralTimes() throws Exception {
        persons.add(Factories.createPersonWithBirthday(Factories.YEAR_2000));
        persons.add(Factories.createPersonWithBirthday(Factories.YEAR_2000));
        persons.add(Factories.createPersonWithBirthday(Factories.YEAR_1999));

        XMLGregorianCalendar birthday = PersonMapper.newXMLGregorianCalendar(Factories.YEAR_2000);
        request.setBirthDatePersonQuery(birthday);

        prepareDatabaseAndSendRequest();

        assertEquals(2, response.getPersonInformationStructure().size());
    }


    @Test
    public void requestWithNameNotFoundInDatabase() throws Exception {
        createPersonWithName("Peter", "Konrad", "Sørensen");
        createPersonWithName("Anders", null, "Thuesen");

        NamePersonQueryType value = new NamePersonQueryType();
        value.setPersonGivenName("Ragna");
        value.setPersonSurnameName("Brock");
        request.setNamePersonQuery(value);

        prepareDatabaseAndSendRequest();

        assertTrue(response.getPersonInformationStructure().isEmpty());
    }


    @Test
    public void requestWithNameFoundOnceInDatabase() throws Exception {
        createPersonWithName("Peter", "Konrad", "Sørensen");
        createPersonWithName("Thomas", "Greve", "Kristensen");

        NamePersonQueryType name = new NamePersonQueryType();
        name.setPersonGivenName("Thomas");
        name.setPersonSurnameName("Kristensen");
        request.setNamePersonQuery(name);

        prepareDatabaseAndSendRequest();

        assertEquals(1, response.getPersonInformationStructure().size());
    }


    @Test
    public void requestWithNameFoundSeveralTimesInDatabase() throws Exception {
        createPersonWithName("Peter", "Konrad", "Sørensen");
        createPersonWithName("Thomas", "Greve", "Kristensen");
        createPersonWithName("Peter", null, "Sørensen");

        NamePersonQueryType value = new NamePersonQueryType();
        value.setPersonGivenName("Peter");
        value.setPersonSurnameName("Sørensen");
        request.setNamePersonQuery(value);

        prepareDatabaseAndSendRequest();

        assertEquals(2, response.getPersonInformationStructure().size());
    }


    @Test
    @Ignore // Hvis alle skal være whitelistet giver denne test / funktionalitet ikke mening
    public void requestWithNonWhitelistedCVRAndAPersonWithActiveProtectionShouldReturnCensoredData() throws Exception {
        PersonVO person = Factories.createPersonWithAddressProtection();
        persons.add(person);

        request.setCivilRegistrationNumberPersonQuery(person.getCpr());

        prepareDatabaseAndSendRequest();

        String givenName = response.getPersonInformationStructure().get(0).getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName();
        assertThat(givenName, is("ADRESSEBESKYTTET"));
    }


    @Test
    public void testThatServiceIsAbleToHandleTwentySuccessiveRequests() throws Exception {
        PersonVO person = Factories.createPersonWithAddressProtection();
        persons.add(person);

        request.setCivilRegistrationNumberPersonQuery(person.getCpr());

        prepareDatabaseAndSendRequest();

        for (int i = 0; i < 30; i++) {
            sendRequest();
        }
    }

    private PersonVO createPersonWithName(String givenName, @Nullable String middleName, String surName) {
        PersonVO person = Factories.createPerson();
        person.setFornavn(givenName);
        person.setMellemnavn(middleName);
        person.setEfternavn(surName);

        persons.add(person);

        return person;
    }


    private void prepareDatabaseAndSendRequest() throws Exception {
        connection.createStatement().executeUpdate("DELETE FROM v2_Person");
        for (PersonVO person : persons) {
            StamdataTestHelper.persist(connection, person);
        }
        connection.commit();

        sendRequest();
    }

    private void sendRequest() throws Exception {
        Holder<Security> securityHeader;
        Holder<Header> medcomHeader;

        SecurityWrapper secutityHeadersNotWhitelisted = DGWSHeaderUtil.
                getVocesTrustedSecurityWrapper(WHITELISTED_CVR, "foo2", "bar2");
        securityHeader = secutityHeadersNotWhitelisted.getSecurity();
        medcomHeader = secutityHeadersNotWhitelisted.getMedcomHeader();

        response = client.getPersonDetails(securityHeader, medcomHeader, request);
    }
}
