/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.trifork.stamdata.yder.models;


import static com.trifork.stamdata.Preconditions.checkState;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import com.google.inject.Inject;
import com.google.inject.Provider;

public class YderDao {
    private final Provider<Connection> connectionProvider;

    @Inject
	YderDao(Provider<Connection> connectionProvider) throws IOException {
        this.connectionProvider = connectionProvider;
	}

	public Yder getYder(String ydernummer) throws SQLException {
        PreparedStatement preparedStatement = null;
        Connection connection = null;
        try {
            String queryString = "SELECT AmtKodeYder, AmtTxtYder, YdernrYder, PrakBetegn, AdrYder, PostnrYder, " +
                    "PostdistYder, TilgDatoYder, AfgDatoYder, HvdSpecKode, HvdSpecTxt, HvdTlf, EmailYder, WWW, CVR " +
                    "FROM Yderregister_v3 " +
                    "WHERE  YdernrYder = ? and ValidFrom <= now() and (ValidTo > now() OR ValidTo IS NULL)";
            connection = connectionProvider.get();
            preparedStatement = connection.prepareStatement(queryString);
            preparedStatement.setObject(1, ydernummer);
            ResultSet resultSet = preparedStatement.executeQuery();
            if (resultSet.next()) {
                Yder yder = new Yder(ydernummer);
                yder.setAmtkode(resultSet.getString("AmtKodeYder"));
                yder.setAmtText(resultSet.getString("AmtTxtYder"));
                yder.setPraksisbetegnelse(resultSet.getString("PrakBetegn"));
                yder.setAddresse(resultSet.getString("AdrYder"));
                yder.setPostnummer(resultSet.getString("PostnrYder"));
                yder.setPostdistrikt(resultSet.getString("PostdistYder"));
                yder.setTilgangsdato(resultSet.getString("TilgDatoYder"));
                yder.setAfgangsdato(resultSet.getString("AfgDatoYder"));
                yder.setSpecialkode(resultSet.getString("HvdSpecKode"));
                yder.setSpecialetekst(resultSet.getString("HvdSpecTxt"));
                yder.setTelefon(resultSet.getString("HvdTlf"));
                yder.setEmail(resultSet.getString("EmailYder"));
                yder.setWeb(resultSet.getString("WWW"));
                yder.setCvr(resultSet.getString("CVR"));
                checkState(!resultSet.next(),"Select in Yderregister_v3 return more than one row.");
                return yder;
            } else {
                return null;
            }
        } finally {
            if(preparedStatement != null) {
                preparedStatement.close();
            }
            if (connection != null) {
                connection.close();
            }
        }
	}

    public List<YderPerson> getPersonList(String ydernummer) throws SQLException {
        PreparedStatement preparedStatement = null;
        Connection connection = null;
        try {
            String queryString = "SELECT TilgDatoPerson, AfgDatoPerson, CprNr, PersonrolleKode, PersonrolleTxt " +
                    "FROM YderregisterPerson_v3 " +
                    "WHERE  YdernrPerson = ? and ValidFrom <= now() and (ValidTo > now() OR ValidTo IS NULL)";
            connection = connectionProvider.get();
            preparedStatement = connection.prepareStatement(queryString);
            preparedStatement.setObject(1, ydernummer);
            ResultSet resultSet = preparedStatement.executeQuery();
            List<YderPerson> personList = new ArrayList<YderPerson>();
            while (resultSet.next()) {
                YderPerson person = new YderPerson(ydernummer);
                person.setTilgangsdato(resultSet.getString("TilgDatoPerson"));
                person.setAfgangsdato(resultSet.getString("AfgDatoPerson"));
                person.setCpr(resultSet.getString("CprNr"));
                person.setPersonrollekode(resultSet.getString("PersonrolleKode"));
                person.setPersonrolletekst(resultSet.getString("PersonrolleTxt"));
                personList.add(person);
            }
            return personList;
        } finally {
            if(preparedStatement != null) {
                preparedStatement.close();
            }
            if (connection != null) {
                connection.close();
            }
        }
    }
}
