/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.trifork.stamdata.yder.webservice;

import java.sql.SQLException;
import java.util.List;

import javax.jws.WebService;
import javax.xml.ws.Holder;

import com.google.inject.Inject;
import com.sun.xml.ws.developer.SchemaValidation;
import com.trifork.stamdata.jaxws.GuiceInstanceResolver.GuiceWebservice;
import com.trifork.stamdata.util.SlaConsts;
import com.trifork.stamdata.yder.SoapUtils;
import com.trifork.stamdata.yder.models.Yder;
import com.trifork.stamdata.yder.models.YderDao;
import com.trifork.stamdata.yder.models.YderPerson;

import dk.nsi.stamdata.jaxws.generated.DGWSFault;
import dk.nsi.stamdata.jaxws.generated.Header;
import dk.nsi.stamdata.jaxws.generated.ObjectFactory;
import dk.nsi.stamdata.jaxws.generated.Security;
import dk.nsi.stamdata.jaxws.generated.YderPersonType;
import dk.nsi.stamdata.jaxws.generated.YderPortType;
import dk.nsi.stamdata.jaxws.generated.YderRequestType;
import dk.nsi.stamdata.jaxws.generated.YderResponseType;
import dk.nsi.stamdata.jaxws.generated.YderType;
import dk.nsi.stamdata.security.ClientVocesCvr;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.security.Whitelisted;
import dk.sds.nsp.audit.Audit;
import dk.sds.nsp.audit.AuditBuilder;
import dk.sds.nsp.audit.AuditBuilder.TypeOfInformation;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sosi.seal.model.constants.FaultCodeValues;


@WebService(endpointInterface="dk.nsi.stamdata.jaxws.generated.YderPortType")
@GuiceWebservice
@SchemaValidation
public class YderLookupImpl implements YderPortType
{
	private final WhitelistService whitelist;
	private final YderDao yderDao;
    private final String cvr;
    private final SLALogItem slaLogItem;

	@Inject
	YderLookupImpl(@ClientVocesCvr String cvr, WhitelistService whitelist, YderDao yderDao,
                            SLALogItem slaLogItem)
	{
		this.cvr = cvr;
        this.yderDao = yderDao;
		this.whitelist = whitelist;
        this.slaLogItem = slaLogItem;
	}

	@Override
    @Whitelisted
	public YderResponseType yder(Holder<Security> wsseHeader,
	        Holder<Header> medcomHeader, YderRequestType parameters)
	        throws DGWSFault {
        Header value = medcomHeader.value;
        if (value.getLinking() != null && value.getLinking().getMessageID() != null) {
            String messageID = value.getLinking().getMessageID();
            slaLogItem.setMessageId(messageID);
            slaLogItem.setSourceOperation("yder");
            slaLogItem.addCallParameter(SlaConsts.SLA_PARAM_CVR, cvr);
        }
        SoapUtils.setHeadersToOutgoing(wsseHeader, medcomHeader);

        String ydernummer = parameters.getYdernummer();

        TypeOfInformation informationType = AuditBuilder.TypeOfInformation.SensitivePersonalInformation;
        String component = "SDM";
        String context = "SYES YderLookup";

        AuditBuilder auditBuilder = Audit.createAuditBuilder();
        auditBuilder.addAuditInformation(component, context, informationType, "yder", ydernummer);

        YderResponseType response = new ObjectFactory().createYderResponseType();
        Yder yder;
        try {
            yder = yderDao.getYder(ydernummer);
        } catch (SQLException e) {
            e.printStackTrace();
            throw new DGWSFault("Internal error.", FaultCodeValues.PROCESSING_PROBLEM);
        }

        if (yder != null) {
            response.setYder(copyYder(yder));
        }

        List<YderPerson> personList;
        try {
            personList = yderDao.getPersonList(ydernummer);
        } catch (SQLException e) {
            e.printStackTrace();
            throw new DGWSFault("Internal error.", FaultCodeValues.PROCESSING_PROBLEM);
        }

        if (personList.size() > 0) {
            for (YderPerson person : personList) {
                response.getPerson().add(copyPerson(person));
            }
        }

        return response;
    }

    private YderPersonType copyPerson(YderPerson from) {
        YderPersonType to = new YderPersonType();

        to.setCPR(from.getCpr());
        to.setTilgangsdato(from.getTilgangsdato());
        to.setAfgangsdato(from.getAfgangsdato());
        to.setPersonrolleKode(from.getPersonrollekode());
        to.setPersonrolleTekst(from.getPersonrolletekst());

        return to;
    }

    private YderType copyYder(Yder from) {
        YderType to = new YderType();

        to.setAmtKode(from.getAmtkode());
        to.setAmtTekst(from.getAmtText());
        to.setPraksisbetegnelse(from.getPraksisbetegnelse());
        to.setAdresse(from.getAddresse());
        to.setPostnummer(from.getPostnummer());
        to.setPostdistrikt(from.getPostdistrikt());
        to.setTilgangsdato(from.getTilgangsdato());
        to.setAfgangsdato(from.getAfgangsdato());
        to.setHovedspecialeKode(from.getSpecialkode());
        to.setHovedspecialeTekst(from.getSpecialetekst());
        to.setTelefonnummer(from.getTelefon());
        to.setEmail(from.getEmail());
        to.setWeb(from.getWeb());
        to.setCVR(from.getCvr());

        return to;
    }
}
