/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.trifork.stamdata.yder;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.io.ByteArrayOutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import javax.inject.Inject;
import javax.xml.namespace.QName;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.common.collect.Lists;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;
import com.trifork.stamdata.yder.models.Yder;
import com.trifork.stamdata.yder.models.YderPerson;

import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.jaxws.generated.ObjectFactory;
import dk.nsi.stamdata.jaxws.generated.YderPersonType;
import dk.nsi.stamdata.jaxws.generated.YderPortType;
import dk.nsi.stamdata.jaxws.generated.YderRequestType;
import dk.nsi.stamdata.jaxws.generated.YderResponseType;
import dk.nsi.stamdata.jaxws.generated.YderService;
import dk.nsi.stamdata.jaxws.generated.YderType;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.TestHelper;
import dk.nsi.stamdata.testing.TestServer;


@RunWith(GuiceTestRunner.class)
public class TestYderServlet
{
    TestServer server;
    List<Yder> yders = Lists.newArrayList();
    List<YderPerson> persons = Lists.newArrayList();
    TestHelper helper = new TestHelper();

    static final String WHITELISTED_CVR = "19343634";
    static final String NON_WHITELISTED_CVR = "12341234";

    String cvr = WHITELISTED_CVR;

    @Inject
    Connection connection;

    private YderRequestType request = new ObjectFactory().createYderRequestType();
    private YderResponseType response = null;
    private YderPortType client;

    @Before
    public void setUp() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='" + WHITELISTED_CVR + "'");
        connection.commit();

        server = new TestServer().start();
    }


    @After
    public void tearDown() throws Exception
    {
        server.stop();
    }

    @Test
    public void shouldReturnEnEmptyResponseIfNoYdersWereFound() throws Exception
    {
        request.setYdernummer("0");

        sendRequest();

        assertEquals(null,response.getYder());
        assertEquals(0,response.getPerson().size());
    }

    @Test
    public void oneYder() throws Exception
    {
        createYder("1234");

        request.setYdernummer("1234");

        sendRequest();

        YderType yder = response.getYder();
        assertNotNull(yder);
        assertEquals("81",yder.getAmtKode());
        assertEquals("Nordjylland",yder.getAmtTekst());
        assertEquals("Kraniosekral",yder.getPraksisbetegnelse());
        assertEquals("Vejen 1",yder.getAdresse());
        assertEquals("5000",yder.getPostnummer());
        assertEquals("Vejen",yder.getPostdistrikt());
        assertEquals("20000101",yder.getTilgangsdato());
        assertEquals("20200101",yder.getAfgangsdato());
        assertEquals("8",yder.getHovedspecialeKode());
        assertEquals("Intern Medicin",yder.getHovedspecialeTekst());
        assertEquals("12345678",yder.getTelefonnummer());
        assertEquals("test@dk.dk",yder.getEmail());
        assertEquals("www.test.dk",yder.getWeb());
        assertEquals("12345678",yder.getCVR());
        assertEquals(0,response.getPerson().size());
    }

    @Test
    public void oneYder2() throws Exception
    {
        createYder2("1234");
        createPerson("1234","1234567890");
        createPerson("1234","1234567891");

        request.setYdernummer("1234");

        sendRequest();

        YderType yder = response.getYder();
        assertNotNull(yder);
        assertEquals("81",yder.getAmtKode());
        assertEquals("Nordjylland",yder.getAmtTekst());
        assertEquals("Kraniosekral",yder.getPraksisbetegnelse());
        assertEquals("Vejen 1",yder.getAdresse());
        assertEquals("5000",yder.getPostnummer());
        assertEquals("Vejen",yder.getPostdistrikt());
        assertEquals("20000101",yder.getTilgangsdato());
        assertEquals(null,yder.getAfgangsdato());
        assertEquals("8",yder.getHovedspecialeKode());
        assertEquals("Intern Medicin",yder.getHovedspecialeTekst());
        assertEquals("12345678",yder.getTelefonnummer());
        assertEquals("test@dk.dk",yder.getEmail());
        assertEquals("www.test.dk",yder.getWeb());
        assertEquals("12345678",yder.getCVR());
        List<YderPersonType> personList = response.getPerson();
        assertEquals(2,personList.size());
        YderPersonType person = personList.get(0);
        assertEquals("1234567890",person.getCPR());
        assertEquals("20000202",person.getTilgangsdato());
        assertEquals("20200202",person.getAfgangsdato());
        assertEquals("11",person.getPersonrolleKode());
        assertEquals("Ejer",person.getPersonrolleTekst());
        person = personList.get(1);
        assertEquals("1234567891",person.getCPR());
        assertEquals("20000202",person.getTilgangsdato());
        assertEquals("20200202",person.getAfgangsdato());
        assertEquals("11",person.getPersonrolleKode());
        assertEquals("Ejer",person.getPersonrolleTekst());
    }

    public Yder createYder(String ydernummer)
    {
        Yder yder = new Yder(ydernummer);
        yder.setAmtkode("81");
        yder.setAmtText("Nordjylland");
        yder.setPraksisbetegnelse("Kraniosekral");
        yder.setAddresse("Vejen 1");
        yder.setPostnummer("5000");
        yder.setPostdistrikt("Vejen");
        yder.setTilgangsdato("20000101");
        yder.setAfgangsdato("20200101");
        yder.setSpecialkode("8");
        yder.setSpecialetekst("Intern Medicin");
        yder.setTelefon("12345678");
        yder.setEmail("test@dk.dk");
        yder.setWeb("www.test.dk");
        yder.setCvr("12345678");
        yders.add(yder);
        return yder;
    }

    public Yder createYder2(String ydernummer)
    {
        Yder yder = new Yder(ydernummer);
        yder.setAmtkode("81");
        yder.setAmtText("Nordjylland");
        yder.setPraksisbetegnelse("Kraniosekral");
        yder.setAddresse("Vejen 1");
        yder.setPostnummer("5000");
        yder.setPostdistrikt("Vejen");
        yder.setTilgangsdato("20000101");
        yder.setSpecialkode("8");
        yder.setSpecialetekst("Intern Medicin");
        yder.setTelefon("12345678");
        yder.setEmail("test@dk.dk");
        yder.setWeb("www.test.dk");
        yder.setCvr("12345678");
        yders.add(yder);
        return yder;
    }

    public YderPerson createPerson(String ydernummer,String cpr)
    {
        YderPerson person = new YderPerson(ydernummer,cpr);
        person.setTilgangsdato("20000202");
        person.setAfgangsdato("20200202");
        person.setPersonrollekode("11");
        person.setPersonrolletekst("Ejer");
        persons.add(person);
        return person;
    }

    public void persistYders() throws SQLException
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("DELETE FROM Yderregister_v3");
        statement.executeUpdate("DELETE FROM YderregisterPerson_v3");

        for (Yder yder : yders) {
            statement.executeUpdate("INSERT INTO Yderregister_v3 "+
                                    "(YdernrYder, AmtKodeYder, AmtTxtYder, PrakBetegn, AdrYder, PostnrYder, PostdistYder, TilgDatoYder, AfgDatoYder, "+
                                    "HvdSpecKode, HvdSpecTxt, HvdTlf, EmailYder, WWW, CVR,ModifiedDate,ValidFrom) "+
                                    "VALUES ("+
                                    "'"+yder.getYdernummer()+"','"+yder.getAmtkode()+"','"+yder.getAmtText()+
                                    "','"+yder.getPraksisbetegnelse()+"','"+yder.getAddresse()+"','"+
                                    yder.getPostnummer()+"','"+yder.getPostdistrikt()+"','"+yder.getTilgangsdato()+"',"+quote(yder.getAfgangsdato())+",'"+yder.getSpecialkode()+"','"+yder.getSpecialetekst()+"','"+
                                    yder.getTelefon()+"','"+yder.getEmail()+"','"+yder.getWeb()+"','"+yder.getCvr()+
                                    "',now(),now())");
        }
        for (YderPerson person : persons) {
            statement.executeUpdate("INSERT INTO YderregisterPerson_v3 "+
                                    "(YdernrPerson, TilgDatoPerson, AfgDatoPerson, CprNr, "+
                                    "PersonrolleKode, PersonrolleTxt, ModifiedDate, ValidFrom) "+
                                    "VALUES ("+
                                    "'"+person.getYdernummer()+"','"+person.getTilgangsdato()+"','"+person.getAfgangsdato()+
                                    "','"+person.getCpr()+"','"+person.getPersonrollekode()+"','"+person.getPersonrolletekst()+
                                    "',now(),now())");
        }
        connection.commit();
    }

    private String quote(String afgangsdato) {
        if (afgangsdato != null) {
            return "'"+afgangsdato+"'";
        }
        return null;
    }


    public void sendRequest() throws Exception
    {
    	helper.enableAuditlogging(true);
    	ByteArrayOutputStream outStream = helper.redirectSystemOutToStream();

    	persistYders();

        final QName SERVICE_QNAME = new QName("http://nsi.dk/-/stamdata/3.0", "YderService");

        //URL wsdlLocation = new URL("http://localhost:9090/stamdata-yder-lookup-ws/service/YderService?wsdl");
        URL wsdlLocation = new URL("http://localhost:8972/service/YderService?wsdl");
        YderService serviceCatalog = new YderService(wsdlLocation, SERVICE_QNAME);

        // SEAL enforces that the XML prefixes are exactly
        // as it creates them. So we have to make sure we
        // don't change them.

        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());

        client = serviceCatalog.getYderPort();

        SecurityWrapper headers = DGWSHeaderUtil.getVocesTrustedSecurityWrapper(cvr, "foo2", "bar2");

        response = client.yder(headers.getSecurity(), headers.getMedcomHeader(), request);

		String[] logs = helper.getSystemLogs();

        assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SYES YderLookup\",\"type\":\"SensitivePersonalInformation\",\"key\":\"yder\",\"value\":\""+request.getYdernummer()+"\"}", logs[0]);
        helper.enableAuditlogging(false);
        helper.resetSystemOut();

    }
}
