/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.trifork.stamdata.yder;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertThat;

import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.xml.namespace.QName;

import dk.nsi.stamdata.security.WhitelistService;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.google.common.collect.Lists;
import com.trifork.stamdata.yder.models.Yder;
import com.trifork.stamdata.yder.models.YderPerson;
import com.trifork.stamdata.jaxws.SealNamespaceResolver;

import dk.nsi.stamdata.dgws.DGWSHeaderUtil;
import dk.nsi.stamdata.dgws.SecurityWrapper;
import dk.nsi.stamdata.jaxws.generated.YderPortType;
import dk.nsi.stamdata.jaxws.generated.YderRequestType;
import dk.nsi.stamdata.jaxws.generated.YderResponseType;
import dk.nsi.stamdata.jaxws.generated.YderService;
import dk.nsi.stamdata.jaxws.generated.YderType;
import dk.nsi.stamdata.jaxws.generated.DGWSFault;
import dk.nsi.stamdata.jaxws.generated.ObjectFactory;
import dk.nsi.stamdata.testing.TestServer;
import dk.sosi.seal.model.AuthenticationLevel;


@RunWith(GuiceTestRunner.class)
public class YderServletIdCardLevelAttackTest
{
    TestServer server;
    List<Yder> yders = Lists.newArrayList();

    static final String WHITELISTED_CVR = "19343634";
    static final String NON_WHITELISTED_CVR = "12341234";

    String cvr = WHITELISTED_CVR;

    @Inject
    Connection connection;

    private YderRequestType request = new ObjectFactory().createYderRequestType();
    private YderResponseType response = null;
    private YderPortType client;

    @Before
    public void setUp() throws Exception
    {
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();

        server = new TestServer().start();
    }


    @After
    public void tearDown() throws Exception
    {
        if (server != null) {
            server.stop();
        }
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelOneIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(1);
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelTwoIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(2);
    }

    @Test
    public void requestWithLevelthreeIdCardDoesNotRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(3);
    }

    @Test(expected = DGWSFault.class)
    public void requestWithLevelFourIdCardRaisesAnException() throws Exception {
        sendRequestWithGivenSecurityLevel(4);
    }
    private void sendRequestWithGivenSecurityLevel(int securityLevel) throws Exception
    {
        request.setYdernummer("0");
        sendRequest(securityLevel);
    }

    public Yder createYder()
    {
        Yder yder = new Yder("12345");
        yders.add(yder);
        return yder;
    }

    private static Map<Integer, AuthenticationLevel> mapFromIntegerLevelToEnum;
    static {
        mapFromIntegerLevelToEnum = new HashMap<Integer, AuthenticationLevel>();

        mapFromIntegerLevelToEnum.put(AuthenticationLevel.NO_AUTHENTICATION.getLevel(),
                AuthenticationLevel.NO_AUTHENTICATION);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.USERNAME_PASSWORD_AUTHENTICATION.getLevel(),
                AuthenticationLevel.USERNAME_PASSWORD_AUTHENTICATION);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.VOCES_TRUSTED_SYSTEM.getLevel(),
                AuthenticationLevel.VOCES_TRUSTED_SYSTEM);
        mapFromIntegerLevelToEnum.put(AuthenticationLevel.MOCES_TRUSTED_USER.getLevel(),
                AuthenticationLevel.MOCES_TRUSTED_USER);
    }

    public void sendRequest(int securityLevel) throws Exception
    {
        //persistYders();

        final QName SERVICE_QNAME = new QName("http://nsi.dk/-/stamdata/3.0", "YderService");

        URL wsdlLocation = new URL("http://localhost:8972/service/YderService?wsdl");
        YderService serviceCatalog = new YderService(wsdlLocation, SERVICE_QNAME);

        // SEAL enforces that the XML prefixes are exactly
        // as it creates them. So we have to make sure we
        // don't change them.

        serviceCatalog.setHandlerResolver(new SealNamespaceResolver());

        client = serviceCatalog.getYderPort();

        SecurityWrapper headers = DGWSHeaderUtil.getSecurityWrapper(
                mapFromIntegerLevelToEnum.get(securityLevel), WHITELISTED_CVR, "foo", "bar");

        response = client.yder(headers.getSecurity(), headers.getMedcomHeader(), request);
    }
}
